=== modified file '.bzrignore'
--- .bzrignore	2009-11-13 18:55:43 +0000
+++ .bzrignore	2009-11-23 20:04:18 +0000
@@ -280,3 +280,5 @@
 .plugin.ini.stamp
 pandora-plugin.ac
 pandora-plugin.am
+lint-rules.am
+*.linted

=== modified file 'Makefile.am'
--- Makefile.am	2009-11-12 18:38:18 +0000
+++ Makefile.am	2009-11-23 20:04:18 +0000
@@ -22,8 +22,6 @@
 
 VERSION=$(PANDORA_RELEASE_VERSION)
 
-TESTS= ${top_srcdir}/extra/run_cpplint.sh
-
 SUBDIRS = ${po} \
 	  gnulib \
 	  drizzled/hash \
@@ -39,7 +37,8 @@
 			config/bzr_revinfo \
 			extra/run_cpplint.sh  \
 			extra/cpplint.py \
-			extra/clean_source.sh
+			extra/clean_source.sh \
+			config/make-lint.py
 
 
 CLEANFILES=		${BUILT_SOURCES}
@@ -209,8 +208,11 @@
 
 EXTRA_DIST+=	${man_MANS}
 
-.PHONY: test \
-    indent
+.PHONY: \
+	indent \
+	linted \
+	test
+
 
 # Target 'test' will run the regression test suite using the built server.
 #
@@ -298,3 +300,5 @@
 clean-local:
 	find . -name '*.gcno' | xargs rm -f 
 
+
+include config/lint-source.am

=== added file 'config/lint-source.am'
--- config/lint-source.am	1970-01-01 00:00:00 +0000
+++ config/lint-source.am	2009-11-23 20:04:18 +0000
@@ -0,0 +1,29 @@
+LINT_COMMAND=python ${top_srcdir}/extra/cpplint.py  --filter=-whitespace,-runtime,-readability,+legal,-build,+build/header_guard,+build/include_config,+build/namespaces
+
+# make a dep file from a cc by linting
+%.cc.linted: %.cc
+	@test -d `dirname $@` || mkdir -p `dirname $@`
+	$(LINT_COMMAND) --deps $@ $<
+
+# make a dep file from a h by linting
+%.h.linted: %.h
+	@test -d `dirname $@` || mkdir -p `dirname $@`
+	$(LINT_COMMAND) --deps $@ $<
+
+clean-local: cleanlints
+
+
+check-local: linted ${srcdir}/config/lint-rules.am
+
+# Build the things to lint dynamically
+${srcdir}/config/lint-rules.am: lint-rules-gen
+	@# Make make think it did something so it sees the file changed.
+
+# This is a ChangeProtected output
+lint-rules-gen:
+	srcdir=${srcdir} ${top_srcdir}/config/make-lint.py
+
+.PHONY: cleanlints lint-rules-gen
+
+# dynamic rules
+include config/lint-rules.am

=== added file 'config/make-lint.py'
--- config/make-lint.py	1970-01-01 00:00:00 +0000
+++ config/make-lint.py	2009-11-23 20:04:18 +0000
@@ -0,0 +1,149 @@
+#!/usr/bin/env python
+#  Copyright (C) 2009 Sun Microsystems
+#  Copyright (C) 2009 Robert Collins
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; version 2 of the License.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  You should have received a copy of the GNU General Public License
+#  along with this program; if not, write to the Free Software
+#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+import os.path
+srcdir = os.environ.get('srcdir','.')
+path = os.path.join(srcdir, 'config', 'lint-rules.am')
+
+# relpath import (available in Python 2.6 and above)
+try:
+    relpath = os.path.relpath
+except AttributeError:
+
+    from os.path import curdir, sep, pardir, join
+
+    def relpath(path, start=curdir):
+        """Return a relative version of a path"""
+
+        if not path:
+            raise ValueError("no path specified")
+
+        start_list = os.path.abspath(start).split(sep)
+        path_list = os.path.abspath(path).split(sep)
+
+        # Work out how much of the filepath is shared by start and path.
+        i = len(os.path.commonprefix([start_list, path_list]))
+
+        rel_list = [pardir] * (len(start_list)-i) + path_list[i:]
+        if not rel_list:
+            return curdir
+        return join(*rel_list)
+
+class ChangeProtectedFile(object):
+  # from register_plugins.py. XXX: duplication. fix.
+
+  def __init__(self, fname):
+    self.bogus_file= False
+    self.real_fname= fname
+    self.new_fname= "%s.new" % fname
+    try:
+      self.new_file= open(self.new_fname,'w+')
+    except IOError:
+      self.bogus_file= True
+
+  def write(self, text):
+    if not self.bogus_file:
+      self.new_file.write(text)
+
+  # We've written all of this out into .new files, now we only copy them
+  # over the old ones if they are different, so that we don't cause 
+  # unnecessary recompiles
+  def close(self):
+    """Return True if the file had changed."""
+    if self.bogus_file:
+      return
+    self.new_file.seek(0)
+    new_content = self.new_file.read()
+    self.new_file.close()
+    try:
+        old_file = file(self.real_fname, 'r')
+        old_content = old_file.read()
+        old_file.close()
+    except IOError:
+        old_content = None
+    if new_content != old_content:
+      if old_content != None:
+        os.unlink(self.real_fname)
+      os.rename(self.new_fname, self.real_fname)
+      return True
+    else:
+        try:
+          os.unlink(self.new_fname)
+        except:
+          pass
+output = ChangeProtectedFile(path)
+
+# We write a makefile that causes:
+# linted to depend on linting all the source files we find
+# linting a source file to depend on the output dep file for that linted file.
+
+
+def lint_path(path):
+    # linted depends on linting this:
+    output.write('linted: %s.linted\n' % path)
+    output.write('%s.linted: %s\n' % (path, path))
+    # the thing being linted depends on the dependencies included in
+    # the lint output
+    #output.write('@am__include@ @am__quote@%s.linted@am__quote@\n' % path)
+    # If the lint file doesn't exist, we'll make one, or else we have to do
+    # a full lint run on every fresh bzr checkout, which is sort of silly
+    #if not os.path.exists("%s.linted" % path):
+    #    lint_file = open("%s.linted" % path,"w")
+    #    lint_file.write("# Placeholder to make empty file")
+    #    lint_file.close()
+
+
+def clean_lints(paths):
+    output.write('cleanlints:\n')
+    # batch in 50
+    for pos in range(len(paths)/50 + 1):
+        path_str = ' '.join((path + '.linted') for path in paths[pos *50:(pos + 1)*50])
+        if not path_str:
+            continue
+        output.write('\trm -f %s\n' % path_str)
+
+
+def should_lint(path):
+    if not (path.endswith('.cc') or path.endswith('.h')):
+        return False
+    if not (path.startswith('plugin/') or path.startswith('drizzled/') or
+        path.startswith('client/')):
+        return False
+    for exclude in ['innobase', 'gnulib', '.pb.', 'bak-header', 'm4',
+        'sql_yacc', 'gperf', 'drizzled/probes.h',
+        'drizzled/function_hash.h', 'drizzled/symbol_hash.h',
+        'util/dummy.cc', 'drizzled/sql_yacc.h', 'drizzled/configmake.h',
+        'drizzled/plugin/config.h']:
+        if exclude in path:
+            return False
+    return True
+
+def accumulate_sources(arg, dirname, fnames):
+    for fname in fnames:
+        path = os.path.join(dirname, fname)
+        path = relpath(path, srcdir)
+        if not should_lint(path):
+            continue
+        arg.append(path)
+
+sources_list = []
+os.path.walk(srcdir,accumulate_sources,sources_list)
+for path in sources_list:
+    lint_path(path)
+clean_lints(sources_list)
+
+output.close()

=== modified file 'configure.ac'
--- configure.ac	2009-11-18 01:39:00 +0000
+++ configure.ac	2009-11-23 20:04:18 +0000
@@ -41,6 +41,7 @@
 
 
 PANDORA_WITH_GETTEXT
+PANDORA_RUN_CPPLINT
 
 PANDORA_REQUIRE_LIBPROTOBUF
 PANDORA_PROTOBUF_REQUIRE_VERSION([2.1.0])

=== modified file 'drizzled/alter_info.h'
--- drizzled/alter_info.h	2009-09-15 21:35:52 +0000
+++ drizzled/alter_info.h	2009-11-16 02:35:00 +0000
@@ -81,6 +81,7 @@
   List<AlterColumn> alter_list;
   List<Key> key_list;
   List<CreateField> create_list;
+  drizzled::message::AlterTable alter_proto;
   std::bitset<32> flags;
   enum enum_enable_or_disable keys_onoff;
   enum tablespace_op_type tablespace_op;

=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2009-11-11 19:42:27 +0000
+++ drizzled/cursor.cc	2009-11-16 20:50:08 +0000
@@ -112,9 +112,9 @@
   return inited == INDEX ? ha_index_end() : inited == RND ? ha_rnd_end() : 0;
 }
 
-Cursor::Table_flags Cursor::ha_table_flags() const
+plugin::StorageEngine::Table_flags Cursor::ha_table_flags() const
 {
-  return cached_table_flags;
+  return engine->table_flags();
 }
 
 void Cursor::ha_start_bulk_insert(ha_rows rows)
@@ -218,7 +218,6 @@
     }
     else
       dup_ref=ref+ALIGN_SIZE(ref_length);
-    cached_table_flags= table_flags();
   }
   return error;
 }
@@ -651,266 +650,6 @@
   }
 }
 
-
-void Cursor::print_keydup_error(uint32_t key_nr, const char *msg)
-{
-  /* Write the duplicated key in the error message */
-  char key[MAX_KEY_LENGTH];
-  String str(key,sizeof(key),system_charset_info);
-
-  if (key_nr == MAX_KEY)
-  {
-    /* Key is unknown */
-    str.copy("", 0, system_charset_info);
-    my_printf_error(ER_DUP_ENTRY, msg, MYF(0), str.c_ptr(), "*UNKNOWN*");
-  }
-  else
-  {
-    /* Table is opened and defined at this point */
-    key_unpack(&str,table,(uint32_t) key_nr);
-    uint32_t max_length=DRIZZLE_ERRMSG_SIZE-(uint32_t) strlen(msg);
-    if (str.length() >= max_length)
-    {
-      str.length(max_length-4);
-      str.append(STRING_WITH_LEN("..."));
-    }
-    my_printf_error(ER_DUP_ENTRY, msg,
-		    MYF(0), str.c_ptr(), table->key_info[key_nr].name);
-  }
-}
-
-
-/**
-  Print error that we got from Cursor function.
-
-  @note
-    In case of delete table it's only safe to use the following parts of
-    the 'table' structure:
-    - table->s->path
-    - table->alias
-*/
-void Cursor::print_error(int error, myf errflag)
-{
-  int textno= ER_GET_ERRNO;
-  switch (error) {
-  case EACCES:
-    textno=ER_OPEN_AS_READONLY;
-    break;
-  case EAGAIN:
-    textno=ER_FILE_USED;
-    break;
-  case ENOENT:
-    textno=ER_FILE_NOT_FOUND;
-    break;
-  case HA_ERR_KEY_NOT_FOUND:
-  case HA_ERR_NO_ACTIVE_RECORD:
-  case HA_ERR_END_OF_FILE:
-    textno=ER_KEY_NOT_FOUND;
-    break;
-  case HA_ERR_WRONG_MRG_TABLE_DEF:
-    textno=ER_WRONG_MRG_TABLE;
-    break;
-  case HA_ERR_FOUND_DUPP_KEY:
-  {
-    uint32_t key_nr=get_dup_key(error);
-    if ((int) key_nr >= 0)
-    {
-      print_keydup_error(key_nr, ER(ER_DUP_ENTRY_WITH_KEY_NAME));
-      return;
-    }
-    textno=ER_DUP_KEY;
-    break;
-  }
-  case HA_ERR_FOREIGN_DUPLICATE_KEY:
-  {
-    uint32_t key_nr= get_dup_key(error);
-    if ((int) key_nr >= 0)
-    {
-      uint32_t max_length;
-      /* Write the key in the error message */
-      char key[MAX_KEY_LENGTH];
-      String str(key,sizeof(key),system_charset_info);
-      /* Table is opened and defined at this point */
-      key_unpack(&str,table,(uint32_t) key_nr);
-      max_length= (DRIZZLE_ERRMSG_SIZE-
-                   (uint32_t) strlen(ER(ER_FOREIGN_DUPLICATE_KEY)));
-      if (str.length() >= max_length)
-      {
-        str.length(max_length-4);
-        str.append(STRING_WITH_LEN("..."));
-      }
-      my_error(ER_FOREIGN_DUPLICATE_KEY, MYF(0), table_share->table_name.str,
-        str.c_ptr(), key_nr+1);
-      return;
-    }
-    textno= ER_DUP_KEY;
-    break;
-  }
-  case HA_ERR_FOUND_DUPP_UNIQUE:
-    textno=ER_DUP_UNIQUE;
-    break;
-  case HA_ERR_RECORD_CHANGED:
-    textno=ER_CHECKREAD;
-    break;
-  case HA_ERR_CRASHED:
-    textno=ER_NOT_KEYFILE;
-    break;
-  case HA_ERR_WRONG_IN_RECORD:
-    textno= ER_CRASHED_ON_USAGE;
-    break;
-  case HA_ERR_CRASHED_ON_USAGE:
-    textno=ER_CRASHED_ON_USAGE;
-    break;
-  case HA_ERR_NOT_A_TABLE:
-    textno= error;
-    break;
-  case HA_ERR_CRASHED_ON_REPAIR:
-    textno=ER_CRASHED_ON_REPAIR;
-    break;
-  case HA_ERR_OUT_OF_MEM:
-    textno=ER_OUT_OF_RESOURCES;
-    break;
-  case HA_ERR_WRONG_COMMAND:
-    textno=ER_ILLEGAL_HA;
-    break;
-  case HA_ERR_OLD_FILE:
-    textno=ER_OLD_KEYFILE;
-    break;
-  case HA_ERR_UNSUPPORTED:
-    textno=ER_UNSUPPORTED_EXTENSION;
-    break;
-  case HA_ERR_RECORD_FILE_FULL:
-  case HA_ERR_INDEX_FILE_FULL:
-    textno=ER_RECORD_FILE_FULL;
-    break;
-  case HA_ERR_LOCK_WAIT_TIMEOUT:
-    textno=ER_LOCK_WAIT_TIMEOUT;
-    break;
-  case HA_ERR_LOCK_TABLE_FULL:
-    textno=ER_LOCK_TABLE_FULL;
-    break;
-  case HA_ERR_LOCK_DEADLOCK:
-    textno=ER_LOCK_DEADLOCK;
-    break;
-  case HA_ERR_READ_ONLY_TRANSACTION:
-    textno=ER_READ_ONLY_TRANSACTION;
-    break;
-  case HA_ERR_CANNOT_ADD_FOREIGN:
-    textno=ER_CANNOT_ADD_FOREIGN;
-    break;
-  case HA_ERR_ROW_IS_REFERENCED:
-  {
-    String str;
-    get_error_message(error, &str);
-    my_error(ER_ROW_IS_REFERENCED_2, MYF(0), str.c_ptr_safe());
-    return;
-  }
-  case HA_ERR_NO_REFERENCED_ROW:
-  {
-    String str;
-    get_error_message(error, &str);
-    my_error(ER_NO_REFERENCED_ROW_2, MYF(0), str.c_ptr_safe());
-    return;
-  }
-  case HA_ERR_TABLE_DEF_CHANGED:
-    textno=ER_TABLE_DEF_CHANGED;
-    break;
-  case HA_ERR_NO_SUCH_TABLE:
-    my_error(ER_NO_SUCH_TABLE, MYF(0), table_share->db.str,
-             table_share->table_name.str);
-    return;
-  case HA_ERR_RBR_LOGGING_FAILED:
-    textno= ER_BINLOG_ROW_LOGGING_FAILED;
-    break;
-  case HA_ERR_DROP_INDEX_FK:
-  {
-    const char *ptr= "???";
-    uint32_t key_nr= get_dup_key(error);
-    if ((int) key_nr >= 0)
-      ptr= table->key_info[key_nr].name;
-    my_error(ER_DROP_INDEX_FK, MYF(0), ptr);
-    return;
-  }
-  case HA_ERR_TABLE_NEEDS_UPGRADE:
-    textno=ER_TABLE_NEEDS_UPGRADE;
-    break;
-  case HA_ERR_TABLE_READONLY:
-    textno= ER_OPEN_AS_READONLY;
-    break;
-  case HA_ERR_AUTOINC_READ_FAILED:
-    textno= ER_AUTOINC_READ_FAILED;
-    break;
-  case HA_ERR_AUTOINC_ERANGE:
-    textno= ER_WARN_DATA_OUT_OF_RANGE;
-    break;
-  case HA_ERR_LOCK_OR_ACTIVE_TRANSACTION:
-    my_message(ER_LOCK_OR_ACTIVE_TRANSACTION,
-               ER(ER_LOCK_OR_ACTIVE_TRANSACTION), MYF(0));
-    return;
-  default:
-    {
-      /* 
-        The error was "unknown" to this function.
-        Ask Cursor if it has got a message for this error 
-      */
-      bool temporary= false;
-      String str;
-      temporary= get_error_message(error, &str);
-      if (!str.is_empty())
-      {
-        const char* engine_name= engine->getName().c_str();
-        if (temporary)
-          my_error(ER_GET_TEMPORARY_ERRMSG, MYF(0), error, str.ptr(),
-                   engine_name);
-        else
-          my_error(ER_GET_ERRMSG, MYF(0), error, str.ptr(), engine_name);
-      }
-      else
-      {
-	      my_error(ER_GET_ERRNO,errflag,error);
-      }
-      return;
-    }
-  }
-  my_error(textno, errflag, table_share->table_name.str, error);
-}
-
-
-/**
-  Return an error message specific to this Cursor.
-
-  @param error  error code previously returned by Cursor
-  @param buf    pointer to String where to add error message
-
-  @return
-    Returns true if this is a temporary error
-*/
-bool Cursor::get_error_message(int , String* )
-{
-  return false;
-}
-
-
-/* Code left, but Drizzle has no legacy yet (while MySQL did) */
-int Cursor::check_old_types()
-{
-  return 0;
-}
-
-/**
-  @return
-    key if error because of duplicated keys
-*/
-uint32_t Cursor::get_dup_key(int error)
-{
-  table->cursor->errkey  = (uint32_t) -1;
-  if (error == HA_ERR_FOUND_DUPP_KEY || error == HA_ERR_FOREIGN_DUPLICATE_KEY ||
-      error == HA_ERR_FOUND_DUPP_UNIQUE ||
-      error == HA_ERR_DROP_INDEX_FK)
-    info(HA_STATUS_ERRKEY | HA_STATUS_NO_LOCK);
-  return(table->cursor->errkey);
-}
-
 void Cursor::drop_table(const char *)
 {
   close();
@@ -1720,8 +1459,6 @@
 
   int error= external_lock(session, lock_type);
 
-  if (error == 0)
-    cached_table_flags= table_flags();
   return error;
 }
 

=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2009-11-11 19:42:27 +0000
+++ drizzled/cursor.h	2009-11-17 17:45:47 +0000
@@ -49,7 +49,7 @@
 
 typedef std::bitset<HA_MAX_ALTER_FLAGS> HA_ALTER_FLAGS;
 
-extern drizzled::atomic<uint32_t> refresh_version;  /* Increments on each reload */
+extern uint64_t refresh_version;  /* Increments on each reload */
 
 
 typedef bool (*qc_engine_callback)(Session *session, char *table_key,
@@ -163,13 +163,9 @@
 
 class Cursor :public Sql_alloc
 {
-public:
-  typedef uint64_t Table_flags;
-
 protected:
   TableShare *table_share;   /* The table definition */
   Table *table;               /* The current open table */
-  Table_flags cached_table_flags;       /* Set on init() and open() */
 
   ha_rows estimation_rows_to_insert;
 public:
@@ -248,11 +244,6 @@
     {}
   virtual ~Cursor(void);
   virtual Cursor *clone(MEM_ROOT *mem_root);
-  /** This is called after create to allow us to set up cached variables */
-  void init()
-  {
-    cached_table_flags= table_flags();
-  }
 
   /* ha_ methods: pubilc wrappers for private virtual API */
 
@@ -265,7 +256,7 @@
 
   /* this is necessary in many places, e.g. in HANDLER command */
   int ha_index_or_rnd_end();
-  Table_flags ha_table_flags() const;
+  drizzled::plugin::StorageEngine::Table_flags ha_table_flags() const;
 
   /**
     These functions represent the public interface to *users* of the
@@ -298,10 +289,6 @@
 
   void adjust_next_insert_id_after_explicit_value(uint64_t nr);
   int update_auto_increment();
-  void print_keydup_error(uint32_t key_nr, const char *msg);
-  virtual void print_error(int error, myf errflag);
-  virtual bool get_error_message(int error, String *buf);
-  uint32_t get_dup_key(int error);
   virtual void change_table_ptr(Table *table_arg, TableShare *share);
 
   /* Estimates calculation */
@@ -623,35 +610,12 @@
    return memcmp(ref1, ref2, ref_length);
  }
 
-  /**
-    Lock table.
-
-    @param    session                     Thread handle
-    @param    lock_type               HA_LOCK_IN_SHARE_MODE     (F_RDLCK)
-                                      HA_LOCK_IN_EXCLUSIVE_MODE (F_WRLCK)
-    @param    lock_timeout            -1 default timeout
-                                      0  no wait
-                                      >0 wait timeout in milliseconds.
-
-   @note
-      lock_timeout >0 is not used by MySQL currently. If the storage
-      engine does not support NOWAIT (lock_timeout == 0) it should
-      return an error. But if it does not support WAIT X (lock_timeout
-      >0) it should treat it as lock_timeout == -1 and wait a default
-      (or even hard-coded) timeout.
-
-    @retval HA_ERR_WRONG_COMMAND      Storage engine does not support
-                                      lock_table()
-    @retval HA_ERR_UNSUPPORTED        Storage engine does not support NOWAIT
-    @retval HA_ERR_LOCK_WAIT_TIMEOUT  Lock request timed out or
-                                      lock conflict with NOWAIT option
-    @retval HA_ERR_LOCK_DEADLOCK      Deadlock detected
-  */
-  virtual int lock_table(Session *, int, int)
+  virtual bool isOrdered(void)
   {
-    return HA_ERR_WRONG_COMMAND;
+    return false;
   }
 
+
 protected:
   /* Service methods for use by storage engines. */
   void ha_statistic_increment(ulong SSV::*offset) const;
@@ -701,7 +665,6 @@
     by that statement.
   */
   virtual int reset() { return 0; }
-  virtual Table_flags table_flags(void) const= 0;
 
   /**
     Is not invoked for non-transactional temporary tables.
@@ -776,17 +739,28 @@
   */
   virtual int reset_auto_increment(uint64_t)
   { return HA_ERR_WRONG_COMMAND; }
+
   virtual int optimize(Session *, HA_CHECK_OPT *)
   { return HA_ADMIN_NOT_IMPLEMENTED; }
+
   virtual int analyze(Session *, HA_CHECK_OPT *)
   { return HA_ADMIN_NOT_IMPLEMENTED; }
 
   virtual int disable_indexes(uint32_t)
   { return HA_ERR_WRONG_COMMAND; }
+
   virtual int enable_indexes(uint32_t)
   { return HA_ERR_WRONG_COMMAND; }
+
   virtual int discard_or_import_tablespace(bool)
   { return (my_errno=HA_ERR_WRONG_COMMAND); }
+
+  /* 
+    @todo this is just for the HEAP engine, it should
+    be removed at some point in the future (and
+    no new engine should ever use it). Right
+    now HEAP does rely on it, so we cannot remove it.
+  */
   virtual void drop_table(const char *name);
 };
 

=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2009-10-01 17:56:07 +0000
+++ drizzled/db.cc	2009-11-17 02:26:31 +0000
@@ -473,7 +473,7 @@
     SELECT DATABASE() in the future). For this we free() session->db and set
     it to 0.
   */
-  if (session->db && !strcmp(session->db, db))
+  if (! session->db.empty() && ! strcmp(session->db.c_str(), db))
     mysql_change_db_impl(session, NULL);
   pthread_mutex_unlock(&LOCK_create_db);
   start_waiting_global_read_lock(session);
@@ -607,17 +607,6 @@
 
     session->set_db(NULL, 0);
   }
-  else if (my_strcasecmp(system_charset_info, new_db_name->str,
-                         INFORMATION_SCHEMA_NAME.c_str()) == 0)
-  {
-    /*
-      Here we must use Session::set_db(), because we want to copy
-      INFORMATION_SCHEMA_NAME constant.
-    */
-
-    session->set_db(INFORMATION_SCHEMA_NAME.c_str(),
-                    INFORMATION_SCHEMA_NAME.length());
-  }
   else
   {
     /*
@@ -626,10 +615,7 @@
       the previous database name, we should do it explicitly.
     */
 
-    if (session->db)
-      free(session->db);
-
-    session->reset_db(new_db_name->str, new_db_name->length);
+    session->set_db(new_db_name->str, new_db_name->length);
   }
 }
 
@@ -807,14 +793,10 @@
     }
   }
 
-  /*
-    NOTE: in mysql_change_db_impl() new_db_file_name is assigned to Session
-    attributes and will be freed in Session::~Session().
-  */
-
   db_default_cl= get_default_db_collation(new_db_file_name.str);
 
   mysql_change_db_impl(session, &new_db_file_name);
+  free(new_db_file_name.str);
 
   return false;
 }

=== modified file 'drizzled/definitions.h'
--- drizzled/definitions.h	2009-11-04 16:17:49 +0000
+++ drizzled/definitions.h	2009-11-16 20:30:19 +0000
@@ -340,15 +340,6 @@
 #define HA_NO_TRANSACTIONS     (1 << 0) /* Doesn't support transactions */
 #define HA_PARTIAL_COLUMN_READ (1 << 1) /* read may not return all columns */
 #define HA_TABLE_SCAN_ON_INDEX (1 << 2) /* No separate data/index file */
-/*
-  The following should be set if the following is not true when scanning
-  a table with rnd_next()
-  - We will see all rows (including deleted ones)
-  - Row positions are 'table->s->db_record_offset' apart
-  If this flag is not set, filesort will do a postion() call for each matched
-  row to be able to find the row later.
-*/
-#define HA_REC_NOT_IN_SEQ      (1 << 3)
 
 /*
   Reading keys in random order is as fast as reading keys in sort order

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-11-13 19:51:29 +0000
+++ drizzled/drizzled.cc	2009-11-16 18:23:07 +0000
@@ -385,7 +385,7 @@
 /** 
   Refresh value. We use to test this to find out if a refresh even has happened recently.
 */
-drizzled::atomic<uint32_t> refresh_version;  /* Increments on each reload */
+uint64_t refresh_version;  /* Increments on each reload */
 
 /* Function declarations */
 bool drizzle_rm_tmp_tables();

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2009-11-14 22:50:02 +0000
+++ drizzled/enum.h	2009-11-16 20:30:19 +0000
@@ -347,5 +347,4 @@
   DUP_UPDATE 
 };
 
-
 #endif /* DRIZZLED_ENUM_H */

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2009-11-11 19:42:27 +0000
+++ drizzled/filesort.cc	2009-11-16 20:30:19 +0000
@@ -465,7 +465,7 @@
   ref_pos= ref_buff;
   quick_select=select && select->quick;
   record=0;
-  flag= ((!indexfile && file->ha_table_flags() & HA_REC_NOT_IN_SEQ)
+  flag= ((!indexfile && ! file->isOrdered())
 	 || quick_select);
   if (indexfile || flag)
     ref_pos= &file->ref[0];
@@ -589,7 +589,7 @@
 
   if (error != HA_ERR_END_OF_FILE)
   {
-    file->print_error(error,MYF(ME_ERROR | ME_WAITTANG));
+    sort_form->print_error(error,MYF(ME_ERROR | ME_WAITTANG));
     return(HA_POS_ERROR);
   }
   if (indexpos && idx &&

=== modified file 'drizzled/function/str/database.cc'
--- drizzled/function/str/database.cc	2008-12-12 20:40:37 +0000
+++ drizzled/function/str/database.cc	2009-11-17 02:26:31 +0000
@@ -26,13 +26,13 @@
 {
   assert(fixed == 1);
   Session *session= current_session;
-  if (session->db == NULL)
+  if (session->db.empty())
   {
     null_value= 1;
     return 0;
   }
   else
-    str->copy(session->db, session->db_length, system_charset_info);
+    str->copy(session->db.c_str(), session->db.length(), system_charset_info);
   return str;
 }
 

=== modified file 'drizzled/item/sum.cc'
--- drizzled/item/sum.cc	2009-11-11 21:40:59 +0000
+++ drizzled/item/sum.cc	2009-11-16 17:06:48 +0000
@@ -2765,7 +2765,7 @@
 
   if(error)
   {
-    table->cursor->print_error(error, MYF(0));
+    table->print_error(error, MYF(0));
   }
 
   return table->cursor->stats.records;

=== modified file 'drizzled/join.cc'
--- drizzled/join.cc	2009-11-11 19:42:27 +0000
+++ drizzled/join.cc	2009-11-16 17:06:48 +0000
@@ -2811,7 +2811,7 @@
     if ((error= table->cursor->ha_update_row(table->record[1],
                                           table->record[0])))
     {
-      table->cursor->print_error(error,MYF(0));
+      table->print_error(error,MYF(0));
       return NESTED_LOOP_ERROR;
     }
     return NESTED_LOOP_OK;
@@ -2869,14 +2869,14 @@
     join->send_records++;			// New group
   else
   {
-    if ((int) table->cursor->get_dup_key(error) < 0)
+    if ((int) table->get_dup_key(error) < 0)
     {
-      table->cursor->print_error(error,MYF(0));
+      table->print_error(error,MYF(0));
       return NESTED_LOOP_ERROR;
     }
     if (table->cursor->rnd_pos(table->record[1],table->cursor->dup_ref))
     {
-      table->cursor->print_error(error,MYF(0));
+      table->print_error(error,MYF(0));
       return NESTED_LOOP_ERROR;
     }
     table->restoreRecord();
@@ -2884,7 +2884,7 @@
     if ((error= table->cursor->ha_update_row(table->record[1],
                                           table->record[0])))
     {
-      table->cursor->print_error(error,MYF(0));
+      table->print_error(error,MYF(0));
       return NESTED_LOOP_ERROR;
     }
   }
@@ -5570,7 +5570,7 @@
     error= table->cursor->info(HA_STATUS_VARIABLE | HA_STATUS_NO_LOCK);
     if (error)
     {
-        table->cursor->print_error(error, MYF(0));
+        table->print_error(error, MYF(0));
         return 1;
     }
     table->quick_keys.reset();

=== modified file 'drizzled/key.cc'
--- drizzled/key.cc	2009-11-11 19:42:27 +0000
+++ drizzled/key.cc	2009-11-16 17:06:48 +0000
@@ -323,7 +323,7 @@
      idx	Key number
 */
 
-void key_unpack(String *to,Table *table,uint32_t idx)
+void key_unpack(String *to, Table *table, uint32_t idx)
 {
   KEY_PART_INFO *key_part,*key_part_end;
   Field *field;

=== modified file 'drizzled/lock.cc'
--- drizzled/lock.cc	2009-11-11 19:42:27 +0000
+++ drizzled/lock.cc	2009-11-17 18:11:45 +0000
@@ -478,105 +478,6 @@
 }
 
 
-/**
-  Find duplicate lock in tables.
-
-  Temporary tables are ignored here like they are ignored in
-  get_lock_data(). If we allow two opens on temporary tables later,
-  both functions should be checked.
-
-  @param session                 The current thread.
-  @param needle              The table to check for duplicate lock.
-  @param haystack            The list of tables to search for the dup lock.
-
-  @note
-    This is mainly meant for MERGE tables in INSERT ... SELECT
-    situations. The 'real', underlying tables can be found only after
-    the MERGE tables are opened. This function assumes that the tables are
-    already locked.
-
-  @retval
-    NULL    No duplicate lock found.
-  @retval
-    !NULL   First table from 'haystack' that matches a lock on 'needle'.
-*/
-
-TableList *mysql_lock_have_duplicate(Session *session, TableList *needle,
-                                      TableList *haystack)
-{
-  DRIZZLE_LOCK            *mylock;
-  Table                 **lock_tables;
-  Table                 *table;
-  Table                 *table2;
-  THR_LOCK_DATA         **lock_locks;
-  THR_LOCK_DATA         **table_lock_data;
-  THR_LOCK_DATA         **end_data;
-  THR_LOCK_DATA         **lock_data2;
-  THR_LOCK_DATA         **end_data2;
-
-  /*
-    Table may not be defined for derived or view tables.
-    Table may not be part of a lock for delayed operations.
-  */
-  if (! (table= needle->table) || ! table->lock_count)
-    goto end;
-
-  /* A temporary table does not have locks. */
-  if (table->s->tmp_table == NON_TRANSACTIONAL_TMP_TABLE)
-    goto end;
-
-  /* Get command lock or LOCK TABLES lock. Maybe empty for INSERT DELAYED. */
-  if (!(mylock= session->lock))
-    goto end;
-
-  /* If we have less than two tables, we cannot have duplicates. */
-  if (mylock->table_count < 2)
-    goto end;
-
-  lock_locks=  mylock->locks;
-  lock_tables= mylock->table;
-
-  /* Prepare table related variables that don't change in loop. */
-  assert((table->lock_position < mylock->table_count) &&
-              (table == lock_tables[table->lock_position]));
-  table_lock_data= lock_locks + table->lock_data_start;
-  end_data= table_lock_data + table->lock_count;
-
-  for (; haystack; haystack= haystack->next_global)
-  {
-    if (haystack->placeholder())
-      continue;
-    table2= haystack->table;
-    if (table2->s->tmp_table == NON_TRANSACTIONAL_TMP_TABLE)
-      continue;
-
-    /* All tables in list must be in lock. */
-    assert((table2->lock_position < mylock->table_count) &&
-                (table2 == lock_tables[table2->lock_position]));
-
-    for (lock_data2=  lock_locks + table2->lock_data_start,
-           end_data2= lock_data2 + table2->lock_count;
-         lock_data2 < end_data2;
-         lock_data2++)
-    {
-      THR_LOCK_DATA **lock_data;
-      THR_LOCK *lock2= (*lock_data2)->lock;
-
-      for (lock_data= table_lock_data;
-           lock_data < end_data;
-           lock_data++)
-      {
-        if ((*lock_data)->lock == lock2)
-          return haystack;
-      }
-    }
-  }
-
- end:
-  return NULL;
-}
-
-
 /** Unlock a set of external. */
 
 static int unlock_external(Session *session, Table **table,uint32_t count)
@@ -652,8 +553,9 @@
     Table *table;
     enum thr_lock_type lock_type;
 
-    if ((table=table_ptr[i])->s->tmp_table == NON_TRANSACTIONAL_TMP_TABLE)
+    if (table_ptr[i]->s->tmp_table == NON_TRANSACTIONAL_TMP_TABLE)
       continue;
+    table= table_ptr[i];
     lock_type= table->reginfo.lock_type;
     assert (lock_type != TL_WRITE_DEFAULT);
     if (lock_type >= TL_WRITE_ALLOW_WRITE)

=== modified file 'drizzled/lock.h'
--- drizzled/lock.h	2009-08-11 03:02:59 +0000
+++ drizzled/lock.h	2009-11-17 18:11:45 +0000
@@ -41,8 +41,6 @@
 void mysql_lock_remove(Session *session, Table *table);
 void mysql_lock_abort(Session *session, Table *table);
 bool mysql_lock_abort_for_thread(Session *session, Table *table);
-TableList *mysql_lock_have_duplicate(Session *session, TableList *needle,
-                                      TableList *haystack);
 bool lock_global_read_lock(Session *session);
 void unlock_global_read_lock(Session *session);
 bool wait_if_global_read_lock(Session *session, bool abort_on_refresh,

=== modified file 'drizzled/message/table.proto'
--- drizzled/message/table.proto	2009-11-11 15:49:59 +0000
+++ drizzled/message/table.proto	2009-11-16 02:35:17 +0000
@@ -99,8 +99,6 @@
       optional string update_value = 2;
       optional bool default_null = 3 [default = false];
       optional bytes default_bin_value = 4;
-
-      optional uint32 length = 100; /* TODO: should go away */
     }
 
     message TimestampFieldOptions {
@@ -108,7 +106,7 @@
     }
 
     message FieldConstraints {
-      required bool is_nullable = 1 [default = false];
+      required bool is_nullable = 1 [default = true];
       optional bool is_unsigned = 2 [default = false];
       repeated string expression = 16; /* Reserve 0-15 for frequenty accessed attributes */
     }
@@ -195,3 +193,7 @@
   optional TableOptions options = 9;
   optional TableStats stats = 10;
 }
+
+message AlterTable {
+  repeated Table.Field added_field = 1;
+}
\ No newline at end of file

=== modified file 'drizzled/opt_sum.cc'
--- drizzled/opt_sum.cc	2009-11-11 19:42:27 +0000
+++ drizzled/opt_sum.cc	2009-11-16 17:06:48 +0000
@@ -176,7 +176,7 @@
       error= tl->table->cursor->info(HA_STATUS_VARIABLE | HA_STATUS_NO_LOCK);
       if(error)
       {
-        tl->table->cursor->print_error(error, MYF(ME_FATALERROR));
+        tl->table->print_error(error, MYF(ME_FATALERROR));
         return error;
       }
       count*= tl->table->cursor->stats.records;
@@ -337,7 +337,7 @@
 	    if (error == HA_ERR_KEY_NOT_FOUND || error == HA_ERR_END_OF_FILE)
 	      return HA_ERR_KEY_NOT_FOUND;	      // No rows matching WHERE
 	    /* HA_ERR_LOCK_DEADLOCK or some other error */
- 	    table->cursor->print_error(error, MYF(0));
+ 	    table->print_error(error, MYF(0));
             return(error);
 	  }
           removed_tables|= table->map;
@@ -424,7 +424,7 @@
 	    if (error == HA_ERR_KEY_NOT_FOUND || error == HA_ERR_END_OF_FILE)
 	      return HA_ERR_KEY_NOT_FOUND;	     // No rows matching WHERE
 	    /* HA_ERR_LOCK_DEADLOCK or some other error */
-            table->cursor->print_error(error, MYF(ME_FATALERROR));
+            table->print_error(error, MYF(ME_FATALERROR));
             return(error);
 	  }
           removed_tables|= table->map;

=== modified file 'drizzled/plugin/storage_engine.cc'
--- drizzled/plugin/storage_engine.cc	2009-11-14 22:21:51 +0000
+++ drizzled/plugin/storage_engine.cc	2009-11-18 03:04:25 +0000
@@ -49,14 +49,22 @@
 
 #include <drizzled/table_proto.h>
 
+#include <drizzled/hash.h>
+
+static bool shutdown_has_begun= false; // Once we put in the container for the vector/etc for engines this will go away.
+
 using namespace std;
 
 namespace drizzled
 {
 
-NameMap<plugin::StorageEngine *> all_engines;
-static std::vector<plugin::StorageEngine *> vector_of_engines;
-static std::vector<plugin::StorageEngine *> vector_of_transactional_engines;
+typedef hash_map<std::string, plugin::StorageEngine *> EngineMap;
+typedef std::vector<plugin::StorageEngine *> EngineVector;
+
+static EngineMap engine_map;
+static EngineVector vector_of_engines;
+static EngineVector vector_of_transactional_engines;
+
 static std::set<std::string> set_of_table_definition_ext;
 
 plugin::StorageEngine::StorageEngine(const string name_arg,
@@ -191,13 +199,23 @@
 
 bool plugin::StorageEngine::addPlugin(plugin::StorageEngine *engine)
 {
-  if (all_engines.add(engine))
+  vector<string> aliases= engine->getAliases();
+
+  string name= engine->getName();
+  transform(name.begin(), name.end(),
+            name.begin(), ::tolower);
+  engine_map.insert(make_pair(name, engine));
+
+  for (vector<string>::iterator iter= aliases.begin();
+       iter != aliases.end(); iter++)
   {
-    errmsg_printf(ERRMSG_LVL_ERROR,
-                  _("Couldn't add StorageEngine"));
-    return true;
+    string alias= *iter;
+    transform(alias.begin(), alias.end(),
+              alias.begin(), ::tolower);
+    engine_map.insert(make_pair(alias, engine));
   }
 
+
   vector_of_engines.push_back(engine);
 
   if (engine->check_flag(HTON_BIT_DOES_TRANSACTIONS))
@@ -212,11 +230,16 @@
   return false;
 }
 
-void plugin::StorageEngine::removePlugin(plugin::StorageEngine *engine)
+void plugin::StorageEngine::removePlugin(plugin::StorageEngine *)
 {
-  all_engines.remove(engine);
-  vector_of_engines.clear();
-  vector_of_transactional_engines.clear();
+  if (shutdown_has_begun == false)
+  {
+    engine_map.clear();
+    vector_of_engines.clear();
+    vector_of_transactional_engines.clear();
+
+    shutdown_has_begun= true;
+  }
 }
 
 plugin::StorageEngine *plugin::StorageEngine::findByName(string find_str)
@@ -224,10 +247,13 @@
   transform(find_str.begin(), find_str.end(),
             find_str.begin(), ::tolower);
 
-  plugin::StorageEngine *engine= all_engines.find(find_str);
-
-  if (engine && engine->is_user_selectable())
-    return engine;
+  EngineMap::iterator iter= engine_map.find(find_str);
+  if (iter != engine_map.end())
+  {
+    StorageEngine *engine= (*iter).second;
+    if (engine->is_user_selectable())
+      return engine;
+  }
 
   return NULL;
 }
@@ -242,16 +268,19 @@
   if (find_str.compare("default") == 0)
     return session.getDefaultStorageEngine();
 
-  plugin::StorageEngine *engine= all_engines.find(find_str);
-
-  if (engine && engine->is_user_selectable())
-    return engine;
+  EngineMap::iterator iter= engine_map.find(find_str);
+  if (iter != engine_map.end())
+  {
+    StorageEngine *engine= (*iter).second;
+    if (engine->is_user_selectable())
+      return engine;
+  }
 
   return NULL;
 }
 
 class StorageEngineCloseConnection
-  : public unary_function<plugin::StorageEngine *, void>
+: public unary_function<plugin::StorageEngine *, void>
 {
   Session *session;
 public:
@@ -263,7 +292,7 @@
   inline result_type operator() (argument_type engine)
   {
     if (engine->is_enabled() && 
-      session_get_ha_data(session, engine))
+        session_get_ha_data(session, engine))
     engine->close_connection(session);
   }
 };
@@ -289,10 +318,10 @@
   vector<int> results;
   
   if (commit)
-    transform(all_engines.begin(), all_engines.end(), results.begin(),
+    transform(vector_of_engines.begin(), vector_of_engines.end(), results.begin(),
               bind2nd(mem_fun(&plugin::StorageEngine::commit_by_xid),xid));
   else
-    transform(all_engines.begin(), all_engines.end(), results.begin(),
+    transform(vector_of_engines.begin(), vector_of_engines.end(), results.begin(),
               bind2nd(mem_fun(&plugin::StorageEngine::rollback_by_xid),xid));
 
   if (find_if(results.begin(), results.end(), bind2nd(equal_to<int>(),0))
@@ -330,9 +359,9 @@
 {
   if (engine == NULL)
   {
-    if (find_if(all_engines.begin(), all_engines.end(),
+    if (find_if(vector_of_engines.begin(), vector_of_engines.end(),
             mem_fun(&plugin::StorageEngine::flush_logs))
-          != all_engines.begin())
+          != vector_of_engines.begin())
       return true;
   }
   else
@@ -690,47 +719,18 @@
 
   if (error && generate_warning)
   {
-    TableShare dummy_share;
-    Table dummy_table;
-    Cursor *file= NULL;
-
-    if (engine)
-    {
-      if ((file= engine->create(dummy_share, session.mem_root)))
-        file->init();
-    }
-    memset(&dummy_table, 0, sizeof(dummy_table));
-    memset(&dummy_share, 0, sizeof(dummy_share));
-    dummy_table.s= &dummy_share;
-
     /*
-      Because file->print_error() use my_error() to generate the error message
+      Because engine->print_error() use my_error() to generate the error message
       we use an internal error Cursor to intercept it and store the text
       in a temporary buffer. Later the message will be presented to user
       as a warning.
     */
     Ha_delete_table_error_handler ha_delete_table_error_handler;
 
-    /* Fill up strucutures that print_error may need */
-    dummy_share.path.str= (char*) path;
-    dummy_share.path.length= strlen(path);
-    dummy_share.db.str= (char*) db;
-    dummy_share.db.length= strlen(db);
-    dummy_share.table_name.str= (char*) alias;
-    dummy_share.table_name.length= strlen(alias);
-    dummy_table.alias= alias;
-
-    if (file != NULL)
-    {
-      file->change_table_ptr(&dummy_table, &dummy_share);
-
-      session.push_internal_handler(&ha_delete_table_error_handler);
-      file->print_error(error, 0);
-
-      session.pop_internal_handler();
-    }
-    else
-      error= -1; /* General form of fail. maybe bad FRM */
+    session.push_internal_handler(&ha_delete_table_error_handler);
+    engine->print_error(error, 0);
+
+    session.pop_internal_handler();
 
     /*
       XXX: should we convert *all* errors to warnings here?
@@ -750,6 +750,8 @@
    0  ok
   @retval
    1  error
+
+   @todo refactor to remove goto, and absorb check_engine() logic into createTable()
 */
 int plugin::StorageEngine::createTable(Session& session, const char *path,
                                        const char *db, const char *table_name,
@@ -780,6 +782,39 @@
   if (update_create_info)
     table.updateCreateInfo(&create_info, &table_proto);
 
+  /* Check for legal operations against the Engine using the proto (if used) */
+  if (proto_used)
+  {
+    if (table_proto.type() == message::Table::TEMPORARY && 
+        share.storage_engine->check_flag(HTON_BIT_TEMPORARY_NOT_SUPPORTED) == true)
+    {
+      error= HA_ERR_UNSUPPORTED;
+      goto err2;
+    }
+    else if (table_proto.type() != message::Table::TEMPORARY && 
+             share.storage_engine->check_flag(HTON_BIT_TEMPORARY_ONLY) == true)
+    {
+      error= HA_ERR_UNSUPPORTED;
+      goto err2;
+    }
+  }
+  else // Lets see how good old create_info handles this
+  {
+    if (create_info.options & HA_LEX_CREATE_TMP_TABLE && 
+        share.storage_engine->check_flag(HTON_BIT_TEMPORARY_NOT_SUPPORTED) == true)
+    {
+      error= HA_ERR_UNSUPPORTED;
+      goto err2;
+    }
+    else if (create_info.options | HA_LEX_CREATE_TMP_TABLE &&
+             share.storage_engine->check_flag(HTON_BIT_TEMPORARY_ONLY) == true)
+    {
+      error= HA_ERR_UNSUPPORTED;
+      goto err2;
+    }
+  }
+
+
   {
     char name_buff[FN_REFLEN];
     const char *table_name_arg;
@@ -792,7 +827,9 @@
                                                create_info, table_proto);
   }
 
+err2:
   table.closefrm(false);
+
   if (error)
   {
     char name_buff[FN_REFLEN];
@@ -806,13 +843,8 @@
 
 Cursor *plugin::StorageEngine::getCursor(TableShare &share, MEM_ROOT *alloc)
 {
-  Cursor *file;
-
   assert(enabled);
-
-  if ((file= create(share, alloc)))
-    file->init();
-  return file;
+  return create(share, alloc);
 }
 
 /**
@@ -998,4 +1030,265 @@
 }
 
 
+/**
+  Print error that we got from Cursor function.
+
+  @note
+    In case of delete table it's only safe to use the following parts of
+    the 'table' structure:
+    - table->s->path
+    - table->alias
+*/
+void plugin::StorageEngine::print_error(int error, myf errflag, Table &table)
+{
+  print_error(error, errflag, &table);
+}
+
+void plugin::StorageEngine::print_error(int error, myf errflag, Table *table)
+{
+  int textno= ER_GET_ERRNO;
+  switch (error) {
+  case EACCES:
+    textno=ER_OPEN_AS_READONLY;
+    break;
+  case EAGAIN:
+    textno=ER_FILE_USED;
+    break;
+  case ENOENT:
+    textno=ER_FILE_NOT_FOUND;
+    break;
+  case HA_ERR_KEY_NOT_FOUND:
+  case HA_ERR_NO_ACTIVE_RECORD:
+  case HA_ERR_END_OF_FILE:
+    textno=ER_KEY_NOT_FOUND;
+    break;
+  case HA_ERR_WRONG_MRG_TABLE_DEF:
+    textno=ER_WRONG_MRG_TABLE;
+    break;
+  case HA_ERR_FOUND_DUPP_KEY:
+  {
+    assert(table);
+    uint32_t key_nr= table->get_dup_key(error);
+    if ((int) key_nr >= 0)
+    {
+      const char *err_msg= ER(ER_DUP_ENTRY_WITH_KEY_NAME);
+
+      if (key_nr == 0 &&
+          (table->key_info[0].key_part[0].field->flags &
+           AUTO_INCREMENT_FLAG)
+          && (current_session)->lex->sql_command == SQLCOM_ALTER_TABLE)
+      {
+        err_msg= ER(ER_DUP_ENTRY_AUTOINCREMENT_CASE);
+      }
+
+      print_keydup_error(key_nr, err_msg, *table);
+      return;
+    }
+    textno=ER_DUP_KEY;
+    break;
+  }
+  case HA_ERR_FOREIGN_DUPLICATE_KEY:
+  {
+    assert(table);
+    uint32_t key_nr= table->get_dup_key(error);
+    if ((int) key_nr >= 0)
+    {
+      uint32_t max_length;
+
+      /* Write the key in the error message */
+      char key[MAX_KEY_LENGTH];
+      String str(key,sizeof(key),system_charset_info);
+
+      /* Table is opened and defined at this point */
+      key_unpack(&str,table,(uint32_t) key_nr);
+      max_length= (DRIZZLE_ERRMSG_SIZE-
+                   (uint32_t) strlen(ER(ER_FOREIGN_DUPLICATE_KEY)));
+      if (str.length() >= max_length)
+      {
+        str.length(max_length-4);
+        str.append(STRING_WITH_LEN("..."));
+      }
+      my_error(ER_FOREIGN_DUPLICATE_KEY, MYF(0), table->s->table_name.str,
+        str.c_ptr(), key_nr+1);
+      return;
+    }
+    textno= ER_DUP_KEY;
+    break;
+  }
+  case HA_ERR_FOUND_DUPP_UNIQUE:
+    textno=ER_DUP_UNIQUE;
+    break;
+  case HA_ERR_RECORD_CHANGED:
+    textno=ER_CHECKREAD;
+    break;
+  case HA_ERR_CRASHED:
+    textno=ER_NOT_KEYFILE;
+    break;
+  case HA_ERR_WRONG_IN_RECORD:
+    textno= ER_CRASHED_ON_USAGE;
+    break;
+  case HA_ERR_CRASHED_ON_USAGE:
+    textno=ER_CRASHED_ON_USAGE;
+    break;
+  case HA_ERR_NOT_A_TABLE:
+    textno= error;
+    break;
+  case HA_ERR_CRASHED_ON_REPAIR:
+    textno=ER_CRASHED_ON_REPAIR;
+    break;
+  case HA_ERR_OUT_OF_MEM:
+    textno=ER_OUT_OF_RESOURCES;
+    break;
+  case HA_ERR_WRONG_COMMAND:
+    textno=ER_ILLEGAL_HA;
+    break;
+  case HA_ERR_OLD_FILE:
+    textno=ER_OLD_KEYFILE;
+    break;
+  case HA_ERR_UNSUPPORTED:
+    textno=ER_UNSUPPORTED_EXTENSION;
+    break;
+  case HA_ERR_RECORD_FILE_FULL:
+  case HA_ERR_INDEX_FILE_FULL:
+    textno=ER_RECORD_FILE_FULL;
+    break;
+  case HA_ERR_LOCK_WAIT_TIMEOUT:
+    textno=ER_LOCK_WAIT_TIMEOUT;
+    break;
+  case HA_ERR_LOCK_TABLE_FULL:
+    textno=ER_LOCK_TABLE_FULL;
+    break;
+  case HA_ERR_LOCK_DEADLOCK:
+    textno=ER_LOCK_DEADLOCK;
+    break;
+  case HA_ERR_READ_ONLY_TRANSACTION:
+    textno=ER_READ_ONLY_TRANSACTION;
+    break;
+  case HA_ERR_CANNOT_ADD_FOREIGN:
+    textno=ER_CANNOT_ADD_FOREIGN;
+    break;
+  case HA_ERR_ROW_IS_REFERENCED:
+  {
+    String str;
+    get_error_message(error, &str);
+    my_error(ER_ROW_IS_REFERENCED_2, MYF(0), str.c_ptr_safe());
+    return;
+  }
+  case HA_ERR_NO_REFERENCED_ROW:
+  {
+    String str;
+    get_error_message(error, &str);
+    my_error(ER_NO_REFERENCED_ROW_2, MYF(0), str.c_ptr_safe());
+    return;
+  }
+  case HA_ERR_TABLE_DEF_CHANGED:
+    textno=ER_TABLE_DEF_CHANGED;
+    break;
+  case HA_ERR_NO_SUCH_TABLE:
+    assert(table);
+    my_error(ER_NO_SUCH_TABLE, MYF(0), table->s->db.str,
+             table->s->table_name.str);
+    return;
+  case HA_ERR_RBR_LOGGING_FAILED:
+    textno= ER_BINLOG_ROW_LOGGING_FAILED;
+    break;
+  case HA_ERR_DROP_INDEX_FK:
+  {
+    assert(table);
+    const char *ptr= "???";
+    uint32_t key_nr= table->get_dup_key(error);
+    if ((int) key_nr >= 0)
+      ptr= table->key_info[key_nr].name;
+    my_error(ER_DROP_INDEX_FK, MYF(0), ptr);
+    return;
+  }
+  case HA_ERR_TABLE_NEEDS_UPGRADE:
+    textno=ER_TABLE_NEEDS_UPGRADE;
+    break;
+  case HA_ERR_TABLE_READONLY:
+    textno= ER_OPEN_AS_READONLY;
+    break;
+  case HA_ERR_AUTOINC_READ_FAILED:
+    textno= ER_AUTOINC_READ_FAILED;
+    break;
+  case HA_ERR_AUTOINC_ERANGE:
+    textno= ER_WARN_DATA_OUT_OF_RANGE;
+    break;
+  case HA_ERR_LOCK_OR_ACTIVE_TRANSACTION:
+    my_message(ER_LOCK_OR_ACTIVE_TRANSACTION,
+               ER(ER_LOCK_OR_ACTIVE_TRANSACTION), MYF(0));
+    return;
+  default:
+    {
+      /* 
+        The error was "unknown" to this function.
+        Ask Cursor if it has got a message for this error 
+      */
+      bool temporary= false;
+      String str;
+      temporary= get_error_message(error, &str);
+      if (!str.is_empty())
+      {
+        const char* engine_name= getName().c_str();
+        if (temporary)
+          my_error(ER_GET_TEMPORARY_ERRMSG, MYF(0), error, str.ptr(),
+                   engine_name);
+        else
+          my_error(ER_GET_ERRMSG, MYF(0), error, str.ptr(), engine_name);
+      }
+      else
+      {
+	      my_error(ER_GET_ERRNO,errflag,error);
+      }
+      return;
+    }
+  }
+  my_error(textno, errflag, table->s->table_name.str, error);
+}
+
+
+/**
+  Return an error message specific to this Cursor.
+
+  @param error  error code previously returned by Cursor
+  @param buf    pointer to String where to add error message
+
+  @return
+    Returns true if this is a temporary error
+*/
+bool plugin::StorageEngine::get_error_message(int , String* )
+{
+  return false;
+}
+
+
+void plugin::StorageEngine::print_keydup_error(uint32_t key_nr, const char *msg, Table &table)
+{
+  /* Write the duplicated key in the error message */
+  char key[MAX_KEY_LENGTH];
+  String str(key,sizeof(key),system_charset_info);
+
+  if (key_nr == MAX_KEY)
+  {
+    /* Key is unknown */
+    str.copy("", 0, system_charset_info);
+    my_printf_error(ER_DUP_ENTRY, msg, MYF(0), str.c_ptr(), "*UNKNOWN*");
+  }
+  else
+  {
+    /* Table is opened and defined at this point */
+    key_unpack(&str, &table, (uint32_t) key_nr);
+    uint32_t max_length=DRIZZLE_ERRMSG_SIZE-(uint32_t) strlen(msg);
+    if (str.length() >= max_length)
+    {
+      str.length(max_length-4);
+      str.append(STRING_WITH_LEN("..."));
+    }
+    my_printf_error(ER_DUP_ENTRY, msg,
+		    MYF(0), str.c_ptr(), table.key_info[key_nr].name);
+  }
+}
+
+
+
 } /* namespace drizzled */

=== modified file 'drizzled/plugin/storage_engine.h'
--- drizzled/plugin/storage_engine.h	2009-11-12 16:49:34 +0000
+++ drizzled/plugin/storage_engine.h	2009-11-16 23:57:07 +0000
@@ -49,7 +49,6 @@
 enum engine_flag_bits {
   HTON_BIT_ALTER_NOT_SUPPORTED,       // Engine does not support alter
   HTON_BIT_HIDDEN,                    // Engine does not appear in lists
-  HTON_BIT_FLUSH_AFTER_RENAME,
   HTON_BIT_NOT_USER_SELECTABLE,
   HTON_BIT_TEMPORARY_NOT_SUPPORTED,   // Having temporary tables not supported
   HTON_BIT_TEMPORARY_ONLY,
@@ -62,7 +61,6 @@
 static const std::bitset<HTON_BIT_SIZE> HTON_NO_FLAGS(0);
 static const std::bitset<HTON_BIT_SIZE> HTON_ALTER_NOT_SUPPORTED(1 << HTON_BIT_ALTER_NOT_SUPPORTED);
 static const std::bitset<HTON_BIT_SIZE> HTON_HIDDEN(1 << HTON_BIT_HIDDEN);
-static const std::bitset<HTON_BIT_SIZE> HTON_FLUSH_AFTER_RENAME(1 << HTON_BIT_FLUSH_AFTER_RENAME);
 static const std::bitset<HTON_BIT_SIZE> HTON_NOT_USER_SELECTABLE(1 << HTON_BIT_NOT_USER_SELECTABLE);
 static const std::bitset<HTON_BIT_SIZE> HTON_TEMPORARY_NOT_SUPPORTED(1 << HTON_BIT_TEMPORARY_NOT_SUPPORTED);
 static const std::bitset<HTON_BIT_SIZE> HTON_TEMPORARY_ONLY(1 << HTON_BIT_TEMPORARY_ONLY);
@@ -94,6 +92,11 @@
 */
 class StorageEngine : public Plugin
 {
+public:
+  typedef uint64_t Table_flags;
+
+private:
+
   /*
     Name used for storage engine.
   */
@@ -170,6 +173,14 @@
     return ENOENT;
   }
 
+  /* Old style cursor errors */
+protected:
+  void print_keydup_error(uint32_t key_nr, const char *msg, Table &table);
+  void print_error(int error, myf errflag, Table *table= NULL);
+  virtual bool get_error_message(int error, String *buf);
+public:
+  virtual void print_error(int error, myf errflag, Table& table);
+
   /*
     each storage engine has it's own memory area (actually a pointer)
     in the session, for storing per-connection information.
@@ -181,6 +192,8 @@
   */
   uint32_t slot;
 
+  virtual Table_flags table_flags(void) const= 0;
+
   inline uint32_t getSlot (void) { return slot; }
   inline void setSlot (uint32_t value) { slot= value; }
 

=== modified file 'drizzled/records.cc'
--- drizzled/records.cc	2009-11-11 19:42:27 +0000
+++ drizzled/records.cc	2009-11-16 17:06:48 +0000
@@ -47,7 +47,7 @@
   @param info         READ_RECORD structure to initialize.
   @param session          Thread handle
   @param table        Table to be accessed
-  @param print_error  If true, call table->cursor->print_error() if an error
+  @param print_error  If true, call table->print_error() if an error
                       occurs (except for end-of-records error)
   @param idx          index to scan
 */
@@ -142,8 +142,8 @@
 void init_read_record(READ_RECORD *info,
                       Session *session, 
                       Table *table,
-		                  SQL_SELECT *select,
-		                  int use_record_cache, 
+                      SQL_SELECT *select,
+                      int use_record_cache, 
                       bool print_error)
 {
   IO_CACHE *tempfile;
@@ -166,7 +166,7 @@
     info->ref_length= table->cursor->ref_length;
   }
   info->select=select;
-  info->print_error=print_error;
+  info->print_error= print_error;
   info->ignore_not_found_rows= 0;
   table->status=0;			/* And it's always found */
 
@@ -262,7 +262,7 @@
   else
   {
     if (info->print_error)
-      info->table->cursor->print_error(error, MYF(0));
+      info->table->print_error(error, MYF(0));
     if (error < 0)                            // Fix negative BDB errno
       error= 1;
   }
@@ -502,7 +502,7 @@
       {
         shortget(error,info->cache_pos);
         if (info->print_error)
-          info->table->cursor->print_error(error,MYF(0));
+          info->table->print_error(error,MYF(0));
       }
       else
       {

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2009-11-11 19:42:27 +0000
+++ drizzled/session.cc	2009-11-17 02:26:31 +0000
@@ -180,7 +180,6 @@
   Open_tables_state(refresh_version),
   mem_root(&main_mem_root),
   lex(&main_lex),
-  db(NULL),
   client(client_arg),
   scheduler(NULL),
   scheduler_arg(NULL),
@@ -411,11 +410,6 @@
   plugin::StorageEngine::closeConnection(this);
   plugin_sessionvar_cleanup(this);
 
-  if (db)
-  {
-    free(db);
-    db= NULL;
-  }
   free_root(&warn_root,MYF(0));
   free_root(&transaction.mem_root,MYF(0));
   mysys_var=0;					// Safety (shouldn't be needed)
@@ -661,14 +655,6 @@
   LEX_STRING db_str= { (char *) in_db, in_db ? strlen(in_db) : 0 };
   bool is_authenticated;
 
-  /*
-    Clear session->db as it points to something, that will be freed when
-    connection is closed. We don't want to accidentally free a wrong
-    pointer if connect failed. Also in case of 'CHANGE USER' failure,
-    current database will be switched to 'no database selected'.
-  */
-  reset_db(NULL, 0);
-
   if (passwd_len != 0 && passwd_len != SCRAMBLE_LENGTH)
   {
     my_error(ER_HANDSHAKE_ERROR, MYF(0), security_ctx.ip.c_str());
@@ -753,7 +739,7 @@
 
   /* We must allocate some extra memory for the cached query string */
   query_length= 0; /* Extra safety: Avoid races */
-  query= (char*) memdup_w_gap((unsigned char*) in_packet, in_packet_length, db_length + 1);
+  query= (char*) memdup_w_gap((unsigned char*) in_packet, in_packet_length, db.length() + 1);
   if (! query)
     return false;
 
@@ -1126,8 +1112,8 @@
   if (!dirname_length(exchange->file_name))
   {
     strcpy(path, drizzle_real_data_home);
-    if (session->db)
-      strncat(path, session->db, FN_REFLEN-strlen(drizzle_real_data_home)-1);
+    if (! session->db.empty())
+      strncat(path, session->db.c_str(), FN_REFLEN-strlen(drizzle_real_data_home)-1);
     (void) fn_format(path, exchange->file_name, path, "", option);
   }
   else
@@ -1631,13 +1617,13 @@
 
 bool Session::copy_db_to(char **p_db, size_t *p_db_length)
 {
-  if (db == NULL)
+  if (db.empty())
   {
     my_message(ER_NO_DB_ERROR, ER(ER_NO_DB_ERROR), MYF(0));
     return true;
   }
-  *p_db= strmake(db, db_length);
-  *p_db_length= db_length;
+  *p_db= strmake(db.c_str(), db.length());
+  *p_db_length= db.length();
   return false;
 }
 
@@ -1712,29 +1698,15 @@
 }
 
 
-bool Session::set_db(const char *new_db, size_t new_db_len)
+bool Session::set_db(const char *new_db, size_t length)
 {
   /* Do not reallocate memory if current chunk is big enough. */
-  if (db && new_db && db_length >= new_db_len)
-    memcpy(db, new_db, new_db_len+1);
+  if (length)
+    db= new_db;
   else
-  {
-    if (db)
-      free(db);
-    if (new_db)
-    {
-      db= (char *)malloc(new_db_len + 1);
-      if (db != NULL)
-      {
-        memcpy(db, new_db, new_db_len);
-        db[new_db_len]= 0;
-      }
-    }
-    else
-      db= NULL;
-  }
-  db_length= db ? new_db_len : 0;
-  return new_db && !db;
+    db.clear();
+
+  return false;
 }
 
 
@@ -1821,7 +1793,7 @@
 
       errmsg_printf(ERRMSG_LVL_WARN, ER(ER_NEW_ABORTING_CONNECTION)
                   , thread_id
-                  , (db ? db : "unconnected")
+                  , (db.empty() ? "unconnected" : db.c_str())
                   , sctx->user.empty() == false ? sctx->user.c_str() : "unauthenticated"
                   , sctx->ip.c_str()
                   , (main_da.is_error() ? main_da.message() : ER(ER_UNKNOWN_ERROR)));
@@ -1890,7 +1862,7 @@
   for (table= temporary_tables; table; table= tmp_next)
   {
     tmp_next= table->next;
-    close_temporary(table, true, true);
+    close_temporary(table);
   }
   temporary_tables= NULL;
 }
@@ -1899,8 +1871,8 @@
   unlink from session->temporary tables and close temporary table
 */
 
-void Session::close_temporary_table(Table *table,
-                                    bool free_share, bool delete_table)
+void Session::close_temporary_table(Table *table)
+                         
 {
   if (table->prev)
   {
@@ -1921,7 +1893,7 @@
     if (temporary_tables)
       table->next->prev= NULL;
   }
-  close_temporary(table, free_share, delete_table);
+  close_temporary(table);
 }
 
 /*
@@ -1932,22 +1904,18 @@
   If this is needed, use close_temporary_table()
 */
 
-void Session::close_temporary(Table *table, bool free_share, bool delete_table)
+void Session::close_temporary(Table *table)
 {
   plugin::StorageEngine *table_type= table->s->db_type();
 
   table->free_io_cache();
   table->closefrm(false);
 
-  if (delete_table)
-    rm_temporary_table(table_type, table->s->path.str);
+  rm_temporary_table(table_type, table->s->path.str);
 
-  if (free_share)
-  {
-    table->s->free_table_share();
-    /* This makes me sad, but we're allocating it via malloc */
-    free(table);
-  }
+  table->s->free_table_share();
+  /* This makes me sad, but we're allocating it via malloc */
+  free(table);
 }
 
 /** Clear most status variables. */

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2009-11-14 22:21:51 +0000
+++ drizzled/session.h	2009-11-17 02:26:31 +0000
@@ -438,8 +438,7 @@
     the Session of that thread); that thread is (and must remain, for now) the
     only responsible for freeing this member.
   */
-  char *db;
-  uint32_t db_length; /**< Length of current schema name */
+  std::string db;
 
   /**
     Constant for Session::where initialization in the beginning of every query.
@@ -1151,22 +1150,6 @@
   */
   bool set_db(const char *new_db, size_t new_db_len);
 
-  /**
-    Set the current database; use shallow copy of C-string.
-
-    @param new_db     a pointer to the new database name.
-    @param new_db_len length of the new database name.
-
-    @note This operation just sets {db, db_length}. Switching the current
-    database usually involves other actions, like switching other database
-    attributes including security context. In the future, this operation
-    will be made private and more convenient interface will be provided.
-  */
-  void reset_db(char *new_db, size_t new_db_len)
-  {
-    db= new_db;
-    db_length= new_db_len;
-  }
   /*
     Copy the current database to the argument. Use the current arena to
     allocate memory for a deep copy: current database may be freed after
@@ -1460,13 +1443,19 @@
   /* Work with temporary tables */
   Table *find_temporary_table(TableList *table_list);
   Table *find_temporary_table(const char *db, const char *table_name);
+
   void close_temporary_tables();
-  void close_temporary_table(Table *table, bool free_share, bool delete_table);
-  void close_temporary(Table *table, bool free_share, bool delete_table);
+  void close_temporary_table(Table *table);
+  // The method below just handles the de-allocation of the table. In
+  // a better memory type world, this would not be needed. 
+private:
+  void close_temporary(Table *table);
+public:
+
   int drop_temporary_table(TableList *table_list);
   bool rm_temporary_table(drizzled::plugin::StorageEngine *base, char *path);
   Table *open_temporary_table(const char *path, const char *db,
-                              const char *table_name, bool link_in_list);
+                              const char *table_name, bool link_in_list= true);
   
   /* Reopen operations */
   bool reopen_tables(bool get_locks, bool mark_share_as_old);

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-11-11 21:40:59 +0000
+++ drizzled/show.cc	2009-11-17 02:26:31 +0000
@@ -764,7 +764,7 @@
         session_info->thread_id=tmp->thread_id;
         session_info->user= session->strdup(tmp_sctx->user.c_str() ? tmp_sctx->user.c_str() : "unauthenticated user");
         session_info->host= session->strdup(tmp_sctx->ip.c_str());
-        if ((session_info->db=tmp->db))             // Safe test
+        if ((session_info->db= tmp->db.c_str()))             // Safe test
           session_info->db=session->strdup(session_info->db);
         session_info->command=(int) tmp->command;
         if ((mysys_var= tmp->mysys_var))

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2009-11-14 22:21:51 +0000
+++ drizzled/sql_base.cc	2009-11-17 18:11:45 +0000
@@ -601,7 +601,7 @@
 0 if table is unique
 */
 
-TableList* unique_table(Session *session, TableList *table, TableList *table_list,
+TableList* unique_table(TableList *table, TableList *table_list,
                         bool check_alias)
 {
   TableList *res;
@@ -635,8 +635,7 @@
 
   for (;;)
   {
-    if (((! (res= find_table_in_global_list(table_list, d_name, t_name))) &&
-         (! (res= mysql_lock_have_duplicate(session, table, table_list)))) ||
+    if ((! (res= find_table_in_global_list(table_list, d_name, t_name))) ||
         ((!res->table || res->table != table->table) &&
          (!check_alias || !(my_strcasecmp(files_charset_info, t_alias, res->alias))) &&
          res->select_lex && !res->select_lex->exclude_from_table_unique_test))
@@ -715,7 +714,7 @@
     return -1;
   }
 
-  close_temporary_table(table, true, true);
+  close_temporary_table(table);
 
   return 0;
 }
@@ -809,7 +808,7 @@
                               const char *table_name)
 {
   if (table->s->tmp_table)
-    close_temporary_table(table, true, true);
+    close_temporary_table(table);
   else
   {
     pthread_mutex_lock(&LOCK_open); /* Close and drop a table (AUX routine) */
@@ -1094,7 +1093,7 @@
 
 Table *Session::openTable(TableList *table_list, bool *refresh, uint32_t flags)
 {
-  register Table *table;
+  Table *table;
   char key[MAX_DBKEY_LENGTH];
   unsigned int key_length;
   const char *alias= table_list->alias;
@@ -2362,7 +2361,7 @@
   path_length= strlen(path);
 
   if (!(new_tmp_table= (Table*) malloc(sizeof(*new_tmp_table) + sizeof(*share) +
-                                   path_length + 1 + key_length)))
+                                       path_length + 1 + key_length)))
     return NULL;
 
   share= (TableShare*) (new_tmp_table+1);

=== modified file 'drizzled/sql_base.h'
--- drizzled/sql_base.h	2009-10-19 06:38:33 +0000
+++ drizzled/sql_base.h	2009-11-17 18:11:45 +0000
@@ -129,8 +129,8 @@
                                TableList *TableList::*link,
                                const char *db_name,
                                const char *table_name);
-TableList *unique_table(Session *session, TableList *table, TableList *table_list,
-                         bool check_alias);
+TableList *unique_table(TableList *table, TableList *table_list,
+                        bool check_alias= false);
 void remove_db_from_cache(const char *db);
 
 /* bits for last argument to remove_table_from_cache() */

=== modified file 'drizzled/sql_delete.cc'
--- drizzled/sql_delete.cc	2009-11-11 19:42:27 +0000
+++ drizzled/sql_delete.cc	2009-11-17 18:11:45 +0000
@@ -140,7 +140,7 @@
     }
     if (error != HA_ERR_WRONG_COMMAND)
     {
-      table->cursor->print_error(error,MYF(0));
+      table->print_error(error,MYF(0));
       error=0;
       goto cleanup;
     }
@@ -270,7 +270,7 @@
       }
       else
       {
-	table->cursor->print_error(error,MYF(0));
+	table->print_error(error,MYF(0));
 	/*
 	  In < 4.0.14 we set the error number to 0 here, but that
 	  was not sensible, because then MySQL would not roll back the
@@ -292,7 +292,7 @@
   if (will_batch && (loc_error= table->cursor->end_bulk_delete()))
   {
     if (error != 1)
-      table->cursor->print_error(loc_error,MYF(0));
+      table->print_error(loc_error,MYF(0));
     error=1;
   }
   session->set_proc_info("end");
@@ -312,7 +312,7 @@
 
     if (error2 && (error2 != HA_ERR_WRONG_COMMAND))
     {
-      table->cursor->print_error(error2, MYF(0));
+      table->print_error(error2, MYF(0));
       error= 1;
     }
   }
@@ -379,7 +379,7 @@
     return(true);
   {
     TableList *duplicate;
-    if ((duplicate= unique_table(session, table_list, table_list->next_global, 0)))
+    if ((duplicate= unique_table(table_list, table_list->next_global)))
     {
       my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->alias);
       return(true);

=== modified file 'drizzled/sql_insert.cc'
--- drizzled/sql_insert.cc	2009-11-11 19:42:27 +0000
+++ drizzled/sql_insert.cc	2009-11-17 18:11:45 +0000
@@ -406,7 +406,7 @@
     table->cursor->ha_release_auto_increment();
     if (table->cursor->ha_end_bulk_insert() && !error)
     {
-      table->cursor->print_error(my_errno,MYF(0));
+      table->print_error(my_errno,MYF(0));
       error=1;
     }
     if (duplic != DUP_ERROR || ignore)
@@ -655,7 +655,7 @@
   if (!select_insert)
   {
     TableList *duplicate;
-    if ((duplicate= unique_table(session, table_list, table_list->next_global, 1)))
+    if ((duplicate= unique_table(table_list, table_list->next_global, true)))
     {
       my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->alias);
 
@@ -751,7 +751,7 @@
           goto gok_or_after_err; /* Ignoring a not fatal error, return 0 */
         goto err;
       }
-      if ((int) (key_nr = table->cursor->get_dup_key(error)) < 0)
+      if ((int) (key_nr = table->get_dup_key(error)) < 0)
       {
 	error= HA_ERR_FOUND_DUPP_KEY;         /* Database can't find key */
 	goto err;
@@ -936,7 +936,7 @@
   /* current_select is NULL if this is a delayed insert */
   if (session->lex->current_select)
     session->lex->current_select->no_error= 0;        // Give error
-  table->cursor->print_error(error,MYF(0));
+  table->print_error(error,MYF(0));
 
 before_err:
   table->cursor->restore_auto_increment(prev_insert_id);
@@ -1154,7 +1154,7 @@
     Is table which we are changing used somewhere in other parts of
     query
   */
-  if (unique_table(session, table_list, table_list->next_global, 0))
+  if (unique_table(table_list, table_list->next_global))
   {
     /* Using same table for INSERT and SELECT */
     lex->current_select->options|= OPTION_BUFFER_RESULT;
@@ -1338,7 +1338,7 @@
 
   if (error)
   {
-    table->cursor->print_error(error,MYF(0));
+    table->print_error(error,MYF(0));
     DRIZZLE_INSERT_SELECT_DONE(error, 0);
     return 1;
   }

=== modified file 'drizzled/sql_load.cc'
--- drizzled/sql_load.cc	2009-11-11 19:42:27 +0000
+++ drizzled/sql_load.cc	2009-11-17 18:11:45 +0000
@@ -128,7 +128,7 @@
     If this is not set, we will use the directory where the table to be
     loaded is located
   */
-  char *tdb= session->db ? session->db : db;		// Result is never null
+  const char *tdb= session->db.empty() ? db  : session->db.c_str();		// Result is never null
   assert(tdb);
   uint32_t skip_lines= ex->skip_lines;
   bool transactional_table;
@@ -158,7 +158,7 @@
     table is marked to be 'used for insert' in which case we should never
     mark this table as 'const table' (ie, one that has only one row).
   */
-  if (unique_table(session, table_list, table_list->next_global, 0))
+  if (unique_table(table_list, table_list->next_global))
   {
     my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->table_name);
     return(true);
@@ -301,7 +301,7 @@
   info.escape_char=escaped->length() ? (*escaped)[0] : INT_MAX;
 
   READ_INFO read_info(file, tot_length,
-                      ex->cs ? ex->cs : get_default_db_collation(session->db),
+                      ex->cs ? ex->cs : get_default_db_collation(session->db.c_str()),
 		      *field_term,*ex->line_start, *ex->line_term, *enclosed,
 		      info.escape_char, is_fifo);
   if (read_info.error)
@@ -357,7 +357,7 @@
 			    *enclosed, skip_lines, ignore);
     if (table->cursor->ha_end_bulk_insert() && !error)
     {
-      table->cursor->print_error(my_errno, MYF(0));
+      table->print_error(my_errno, MYF(0));
       error= 1;
     }
     table->cursor->extra(HA_EXTRA_NO_IGNORE_DUP_KEY);

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2009-11-11 22:03:15 +0000
+++ drizzled/sql_select.cc	2009-11-16 17:06:48 +0000
@@ -3380,7 +3380,7 @@
       new_errno= tmp;
     }
     if (new_errno)
-      table->cursor->print_error(new_errno,MYF(0));
+      table->print_error(new_errno,MYF(0));
   }
   return(join->session->is_error() ? -1 : rc);
 }
@@ -5403,7 +5403,7 @@
 err:
   cursor->extra(HA_EXTRA_NO_CACHE);
   if (error)
-    cursor->print_error(error,MYF(0));
+    table->print_error(error,MYF(0));
   return(1);
 }
 
@@ -5514,7 +5514,7 @@
   cursor->extra(HA_EXTRA_NO_CACHE);
   (void) cursor->ha_rnd_end();
   if (error)
-    cursor->print_error(error,MYF(0));
+    table->print_error(error,MYF(0));
   return(1);
 }
 

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2009-11-14 22:21:51 +0000
+++ drizzled/sql_table.cc	2009-11-18 03:04:25 +0000
@@ -345,7 +345,7 @@
     built_query.append("DROP TABLE ");
 
   built_query.append("`");
-  if (session->db == NULL || strcmp(db_name ,session->db) != 0)
+  if (session->db.empty() || strcmp(db_name, session->db.c_str()) != 0)
   {
     built_query.append(db_name);
     built_query.append("`.`");
@@ -1276,15 +1276,18 @@
       key_info->comment.str= key->key_create_info.comment.str;
     }
 
+    message::Table::Field *protofield= NULL;
+
     List_iterator<Key_part_spec> cols(key->columns), cols2(key->columns);
     for (uint32_t column_nr=0 ; (column=cols++) ; column_nr++)
     {
       uint32_t length;
       Key_part_spec *dup_column;
+      int proto_field_nr= 0;
 
       it.rewind();
       field=0;
-      while ((sql_field=it++) &&
+      while ((sql_field=it++) && ++proto_field_nr &&
 	     my_strcasecmp(system_charset_info,
 			   column->field_name.str,
 			   sql_field->field_name))
@@ -1306,6 +1309,10 @@
 	}
       }
       cols2.rewind();
+
+      if (create_proto->field_size() > 0)
+        protofield= create_proto->mutable_field(proto_field_nr - 1);
+
       {
         column->length*= sql_field->charset->mbmaxlen;
 
@@ -1329,6 +1336,14 @@
             /* Implicitly set primary key fields to NOT NULL for ISO conf. */
             sql_field->flags|= NOT_NULL_FLAG;
             null_fields--;
+
+            if (protofield)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+              constraints= protofield->mutable_constraints();
+              constraints->set_is_nullable(false);
+            }
+
           }
           else
           {
@@ -1760,7 +1775,7 @@
   if (create_info->options & HA_LEX_CREATE_TMP_TABLE)
   {
     /* Open table and put in temporary table list */
-    if (!(session->open_temporary_table(path, db, table_name, 1)))
+    if (!(session->open_temporary_table(path, db, table_name)))
     {
       (void) session->rm_temporary_table(create_info->db_type, path);
       goto unlock_and_end;
@@ -2447,6 +2462,20 @@
 
     if (src_table->schema_table)
     {
+      /* 
+        If engine was not specified and we are reading from the I_S, then we need to 
+        toss an error. This should go away later on when we straighten out the 
+        I_S engine.
+      */
+      if (! (create_info->used_fields & HA_CREATE_USED_ENGINE))
+      {
+        pthread_mutex_unlock(&LOCK_open);
+        my_error(ER_ILLEGAL_HA_CREATE_OPTION, MYF(0),
+                 "INFORMATION_ENGINE",
+                 "TEMPORARY");
+        goto err;
+      }
+
       if (create_like_schema_frm(session, src_table, create_info, &src_proto))
       {
         pthread_mutex_unlock(&LOCK_open);
@@ -2473,20 +2502,22 @@
       protoengine->set_name(create_table_proto.engine().name());
     }
 
-    string dst_proto_path(dst_path);
-    string file_ext = ".dfe";
-
-    dst_proto_path.append(file_ext);
-
     if (protoerr == EEXIST)
     {
       plugin::StorageEngine* engine= plugin::StorageEngine::findByName(*session,
                                                                        new_proto.engine().name());
 
       if (engine->check_flag(HTON_BIT_HAS_DATA_DICTIONARY) == false)
+      {
+        string dst_proto_path(dst_path);
+        dst_proto_path.append(".dfe");
+
         protoerr= drizzle_write_proto_file(dst_proto_path.c_str(), &new_proto);
+      }
       else
+      {
         protoerr= 0;
+      }
     }
 
     if (protoerr)
@@ -2511,7 +2542,7 @@
 
   if (create_info->options & HA_LEX_CREATE_TMP_TABLE)
   {
-    if (err || !session->open_temporary_table(dst_path, db, table_name, true))
+    if (err || !session->open_temporary_table(dst_path, db, table_name))
     {
       (void) session->rm_temporary_table(create_info->db_type, dst_path);
       goto err;

=== modified file 'drizzled/sql_union.cc'
--- drizzled/sql_union.cc	2009-11-11 19:42:27 +0000
+++ drizzled/sql_union.cc	2009-11-16 17:06:48 +0000
@@ -83,7 +83,7 @@
   int error;
   if ((error=table->cursor->extra(HA_EXTRA_NO_CACHE)))
   {
-    table->cursor->print_error(error, MYF(0));
+    table->print_error(error, MYF(0));
     return 1;
   }
   return 0;
@@ -480,9 +480,9 @@
       }
       /* Needed for the following test and for records_at_start in next loop */
       int error= table->cursor->info(HA_STATUS_VARIABLE);
-      if(error)
+      if (error)
       {
-        table->cursor->print_error(error, MYF(0));
+        table->print_error(error, MYF(0));
         return(1);
       }
       if (found_rows_for_union && !sl->braces &&

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2009-11-11 19:42:27 +0000
+++ drizzled/sql_update.cc	2009-11-17 18:11:45 +0000
@@ -63,7 +63,7 @@
     Get the number of the offended index.
     We will see MAX_KEY if the engine cannot determine the affected index.
   */
-  if ((keynr= table->cursor->get_dup_key(error)) >= MAX_KEY)
+  if ((keynr= table->get_dup_key(error)) >= MAX_KEY)
     return;
 
   /* Create unique_map with all fields used by that index. */
@@ -516,7 +516,7 @@
             flags|= ME_FATALERROR; /* Other handler errors are fatal */
 
           prepare_record_for_error_message(error, table);
-	  table->cursor->print_error(error,MYF(flags));
+	  table->print_error(error,MYF(flags));
 	  error= 1;
 	  break;
 	}
@@ -544,7 +544,7 @@
               are ignored. This is a requirement on batching handlers.
             */
             prepare_record_for_error_message(error, table);
-            table->cursor->print_error(error,MYF(0));
+            table->print_error(error,MYF(0));
             error= 1;
             break;
           }
@@ -593,7 +593,7 @@
     */
   {
     prepare_record_for_error_message(loc_error, table);
-    table->cursor->print_error(loc_error,MYF(ME_FATALERROR));
+    table->print_error(loc_error,MYF(ME_FATALERROR));
     error= 1;
   }
   else
@@ -701,7 +701,7 @@
   /* Check that we are not using table that we are updating in a sub select */
   {
     TableList *duplicate;
-    if ((duplicate= unique_table(session, table_list, table_list->next_global, 0)))
+    if ((duplicate= unique_table(table_list, table_list->next_global)))
     {
       my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->table_name);
       return true;

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2009-11-13 00:56:59 +0000
+++ drizzled/sql_yacc.yy	2009-11-17 07:21:27 +0000
@@ -1542,11 +1542,19 @@
             statement->comment= null_lex_str;
             lex->charset=NULL;
             statement->column_format= COLUMN_FORMAT_TYPE_DEFAULT;
+
+            message::AlterTable &alter_proto=
+              ((statement::CreateTable *)Lex->statement)->alter_info.alter_proto;
+            statement->current_proto_field= alter_proto.add_added_field();
           }
           field_def
           {
             LEX *lex=Lex;
             statement::CreateTable *statement= (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_name($1.str);
+
             if (add_field_to_list(lex->session, &$1, (enum enum_field_types) $3,
                                   lex->length,lex->dec,lex->type,
                                   statement->column_format,
@@ -1554,6 +1562,8 @@
                                   &statement->comment,
                                   statement->change, &lex->interval_list, lex->charset))
               DRIZZLE_YYABORT;
+
+            statement->current_proto_field= NULL;
           }
         ;
 field_def:
@@ -1562,63 +1572,213 @@
 
 type:
         int_type
-        { 
-          $$=$1; 
+        {
+          $$=$1;
           Lex->length=(char*) 0; /* use default length */
-        }
-        | real_type opt_precision { $$=$1; }
-        | char '(' NUM ')'
-          {
-            Lex->length=$3.str;
-            $$=DRIZZLE_TYPE_VARCHAR;
-          }
-        | char
-          {
-            Lex->length=(char*) "1";
-            $$=DRIZZLE_TYPE_VARCHAR;
-          }
-        | varchar '(' NUM ')'
-          {
-            Lex->length=$3.str;
-            $$= DRIZZLE_TYPE_VARCHAR;
-          }
-        | VARBINARY '(' NUM ')'
-          {
-            Lex->length=$3.str;
-            Lex->charset=&my_charset_bin;
-            $$= DRIZZLE_TYPE_VARCHAR;
-          }
-        | DATE_SYM
-          { $$=DRIZZLE_TYPE_DATE; }
-        | TIMESTAMP_SYM
-          { $$=DRIZZLE_TYPE_TIMESTAMP; }
-        | DATETIME_SYM
-          { $$=DRIZZLE_TYPE_DATETIME; }
-        | BLOB_SYM
-          {
-            Lex->charset=&my_charset_bin;
-            $$=DRIZZLE_TYPE_BLOB;
-            Lex->length=(char*) 0; /* use default length */
-          }
-        | TEXT_SYM
-          { 
-            $$=DRIZZLE_TYPE_BLOB; 
-            Lex->length=(char*) 0; /* use default length */
-          }
-        | DECIMAL_SYM float_options
-          { $$=DRIZZLE_TYPE_DECIMAL;}
-        | NUMERIC_SYM float_options
-          { $$=DRIZZLE_TYPE_DECIMAL;}
-        | FIXED_SYM float_options
-          { $$=DRIZZLE_TYPE_DECIMAL;}
-        | ENUM_SYM
-          {Lex->interval_list.empty();}
-          '(' string_list ')'
-          { $$=DRIZZLE_TYPE_ENUM; }
+          statement::CreateTable *statement=
+            (statement::CreateTable *)Lex->statement;
+
+          if (statement->current_proto_field)
+          {
+            if ($1 == DRIZZLE_TYPE_LONG)
+              statement->current_proto_field->set_type(message::Table::Field::INTEGER);
+            else if ($1 == DRIZZLE_TYPE_LONGLONG)
+              statement->current_proto_field->set_type(message::Table::Field::BIGINT);
+            else
+              abort();
+          }
+          }
+        | real_type opt_precision
+          {
+            $$=$1;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+            {
+              assert ($1 == DRIZZLE_TYPE_DOUBLE);
+              statement->current_proto_field->set_type(message::Table::Field::DOUBLE);
+            }
+          }
+          | char '(' NUM ')'
+            {
+              Lex->length=$3.str;
+              $$=DRIZZLE_TYPE_VARCHAR;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+            {
+              statement->current_proto_field->set_type(message::Table::Field::VARCHAR);
+              message::Table::Field::StringFieldOptions *string_field_options;
+
+              string_field_options= statement->current_proto_field->mutable_string_options();
+
+              string_field_options->set_length(atoi($3.str));
+            }
+            }
+          | char
+            {
+              Lex->length=(char*) "1";
+              $$=DRIZZLE_TYPE_VARCHAR;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::VARCHAR);
+            }
+          | varchar '(' NUM ')'
+            {
+              Lex->length=$3.str;
+              $$= DRIZZLE_TYPE_VARCHAR;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+	    {
+              statement->current_proto_field->set_type(message::Table::Field::VARCHAR);
+
+              message::Table::Field::StringFieldOptions *string_field_options;
+
+              string_field_options= statement->current_proto_field->mutable_string_options();
+
+              string_field_options->set_length(atoi($3.str));
+            }
+            }
+          | VARBINARY '(' NUM ')'
+            {
+              Lex->length=$3.str;
+              Lex->charset=&my_charset_bin;
+              $$= DRIZZLE_TYPE_VARCHAR;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+	    {
+              statement->current_proto_field->set_type(message::Table::Field::VARCHAR);
+              message::Table::Field::StringFieldOptions *string_field_options;
+
+              string_field_options= statement->current_proto_field->mutable_string_options();
+
+              string_field_options->set_length(atoi($3.str));
+              string_field_options->set_collation_id(my_charset_bin.number);
+              string_field_options->set_collation(my_charset_bin.name);
+            }
+            }
+          | DATE_SYM
+          {
+            $$=DRIZZLE_TYPE_DATE;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::DATE);
+          }
+          | TIMESTAMP_SYM
+          {
+            $$=DRIZZLE_TYPE_TIMESTAMP;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::TIMESTAMP);
+          }
+          | DATETIME_SYM
+          {
+            $$=DRIZZLE_TYPE_DATETIME;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::DATETIME);
+          }
+          | BLOB_SYM
+            {
+              Lex->charset=&my_charset_bin;
+              $$=DRIZZLE_TYPE_BLOB;
+              Lex->length=(char*) 0; /* use default length */
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::BLOB);
+            }
+          | TEXT_SYM
+            {
+              $$=DRIZZLE_TYPE_BLOB;
+              Lex->length=(char*) 0; /* use default length */
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::BLOB);
+            }
+          | DECIMAL_SYM float_options
+          {
+            $$=DRIZZLE_TYPE_DECIMAL;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::DECIMAL);
+          }
+          | NUMERIC_SYM float_options
+          {
+            $$=DRIZZLE_TYPE_DECIMAL;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::DECIMAL);
+          }
+          | FIXED_SYM float_options
+          {
+            $$=DRIZZLE_TYPE_DECIMAL;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::DECIMAL);
+          }
+          | ENUM_SYM
+            {Lex->interval_list.empty();}
+            '(' string_list ')'
+          {
+            $$=DRIZZLE_TYPE_ENUM;
+
+            statement::CreateTable *statement=
+              (statement::CreateTable *)Lex->statement;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_type(message::Table::Field::ENUM);
+          }
         | SERIAL_SYM
           {
             $$=DRIZZLE_TYPE_LONGLONG;
             Lex->type|= (AUTO_INCREMENT_FLAG | NOT_NULL_FLAG | UNIQUE_FLAG);
+
+            statement::CreateTable *statement= (statement::CreateTable *)Lex->statement;
+            if (statement->current_proto_field)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+              constraints= statement->current_proto_field->mutable_constraints();
+              constraints->set_is_nullable(false);
+
+              statement->current_proto_field->set_type(message::Table::Field::BIGINT);
+            }
           }
         ;
 
@@ -1686,7 +1846,18 @@
         ;
 
 attribute:
-          NULL_SYM { Lex->type&= ~ NOT_NULL_FLAG; }
+          NULL_SYM
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+            Lex->type&= ~ NOT_NULL_FLAG;
+
+            if (statement->current_proto_field)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+              constraints= statement->current_proto_field->mutable_constraints();
+              constraints->set_is_nullable(true);
+            }
+          }
         | COLUMN_FORMAT_SYM column_format_types
           {
             statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
@@ -1694,7 +1865,18 @@
             statement->column_format= $2;
             statement->alter_info.flags.set(ALTER_COLUMN_FORMAT);
           }
-        | not NULL_SYM { Lex->type|= NOT_NULL_FLAG; }
+        | not NULL_SYM
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+            Lex->type|= NOT_NULL_FLAG;
+
+            if (statement->current_proto_field)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+              constraints= statement->current_proto_field->mutable_constraints();
+              constraints->set_is_nullable(false);
+            }
+          }
         | DEFAULT now_or_signed_literal 
           { 
             statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
@@ -1704,7 +1886,19 @@
           }
         | ON UPDATE_SYM NOW_SYM optional_braces 
           { ((statement::AlterTable *)Lex->statement)->on_update_value= new Item_func_now_local(); }
-        | AUTO_INC { Lex->type|= AUTO_INCREMENT_FLAG | NOT_NULL_FLAG; }
+        | AUTO_INC
+          {
+            Lex->type|= AUTO_INCREMENT_FLAG | NOT_NULL_FLAG;
+
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+            if (statement->current_proto_field)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+
+              constraints= statement->current_proto_field->mutable_constraints();
+              constraints->set_is_nullable(false);
+            }
+          }
         | SERIAL_SYM DEFAULT VALUE_SYM
           { 
             LEX *lex=Lex;
@@ -1712,6 +1906,13 @@
 
             lex->type|= AUTO_INCREMENT_FLAG | NOT_NULL_FLAG | UNIQUE_FLAG;
             statement->alter_info.flags.set(ALTER_ADD_INDEX);
+
+            if (statement->current_proto_field)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+              constraints= statement->current_proto_field->mutable_constraints();
+              constraints->set_is_nullable(false);
+            }
           }
         | opt_primary KEY_SYM
           {
@@ -1720,6 +1921,13 @@
 
             lex->type|= PRI_KEY_FLAG | NOT_NULL_FLAG;
             statement->alter_info.flags.set(ALTER_ADD_INDEX);
+
+            if (statement->current_proto_field)
+            {
+              message::Table::Field::FieldConstraints *constraints;
+              constraints= statement->current_proto_field->mutable_constraints();
+              constraints->set_is_nullable(false);
+            }
           }
         | UNIQUE_SYM
           {
@@ -1737,7 +1945,14 @@
             lex->type|= UNIQUE_KEY_FLAG; 
             statement->alter_info.flags.set(ALTER_ADD_INDEX);
           }
-        | COMMENT_SYM TEXT_STRING_sys { ((statement::AlterTable *)Lex->statement)->comment= $2; }
+        | COMMENT_SYM TEXT_STRING_sys
+          {
+            statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
+            statement->comment= $2;
+
+            if (statement->current_proto_field)
+              statement->current_proto_field->set_comment($2.str);
+          }
         | COLLATE_SYM collation_name
           {
             if (Lex->charset && !my_charset_same(Lex->charset,$2))
@@ -2169,6 +2384,8 @@
             lex->charset= NULL;
             statement->alter_info.flags.set(ALTER_CHANGE_COLUMN);
             statement->column_format= COLUMN_FORMAT_TYPE_DEFAULT;
+
+            statement->current_proto_field= NULL;
           }
           field_def
           {

=== modified file 'drizzled/statement/alter_table.cc'
--- drizzled/statement/alter_table.cc	2009-11-14 22:21:51 +0000
+++ drizzled/statement/alter_table.cc	2009-11-16 23:57:07 +0000
@@ -564,7 +564,7 @@
     return 0;
   }
 
-  table->cursor->print_error(error, MYF(0));
+  table->print_error(error, MYF(0));
 
   return -1;
 }
@@ -606,7 +606,7 @@
                         table->s->table_name.str);
     error= 0;
   } else if (error)
-    table->cursor->print_error(error, MYF(0));
+    table->print_error(error, MYF(0));
 
   return(error);
 }
@@ -933,7 +933,7 @@
     }
     else if (error > 0)
     {
-      table->cursor->print_error(error, MYF(0));
+      table->print_error(error, MYF(0));
       error= -1;
     }
 
@@ -1040,7 +1040,7 @@
     }
 
     /* Remove link to old table and rename the new one */
-    session->close_temporary_table(table, true, true);
+    session->close_temporary_table(table);
 
     /* Should pass the 'new_name' as we store table name in the cache */
     if (new_table->rename_temporary_table(new_db, new_name))
@@ -1139,28 +1139,6 @@
   session->set_proc_info("end");
 
   write_bin_log(session, session->query, session->query_length);
-
-  if (old_db_type->check_flag(HTON_BIT_FLUSH_AFTER_RENAME))
-  {
-    /*
-      For the alter table to be properly flushed to the logs, we
-      have to open the new table.  If not, we get a problem on server
-      shutdown. But we do not need to attach MERGE children.
-    */
-    char table_path[FN_REFLEN];
-    Table *t_table;
-    build_table_filename(table_path, sizeof(table_path), new_db, table_name, false);
-    t_table= session->open_temporary_table(table_path, new_db, tmp_name, false);
-    if (t_table)
-    {
-      t_table->intern_close_table();
-      free(t_table);
-    }
-    else
-      errmsg_printf(ERRMSG_LVL_WARN, _("Could not open table %s.%s after rename\n"), new_db, table_name);
-
-    plugin::StorageEngine::flushLogs(old_db_type);
-  }
   table_list->table= NULL;
 
 end_temporary:
@@ -1188,7 +1166,7 @@
   if (new_table)
   {
     /* close_temporary_table() frees the new_table pointer. */
-    session->close_temporary_table(new_table, true, true);
+    session->close_temporary_table(new_table);
   }
   else
     quick_rm_table(*session, new_db, tmp_name, true);
@@ -1248,12 +1226,12 @@
 /* alter_table */
 
 static int
-copy_data_between_tables(Table *from,Table *to,
-			 List<CreateField> &create,
+copy_data_between_tables(Table *from, Table *to,
+                         List<CreateField> &create,
                          bool ignore,
-			 uint32_t order_num, order_st *order,
-			 ha_rows *copied,
-			 ha_rows *deleted,
+                         uint32_t order_num, order_st *order,
+                         ha_rows *copied,
+                         ha_rows *deleted,
                          enum enum_enable_or_disable keys_onoff,
                          bool error_if_not_empty)
 {
@@ -1387,26 +1365,11 @@
     error= to->cursor->ha_write_row(to->record[0]);
     to->auto_increment_field_not_null= false;
     if (error)
-    {
+    { 
       if (!ignore ||
           to->cursor->is_fatal_error(error, HA_CHECK_DUP))
-      {
-        if (!to->cursor->is_fatal_error(error, HA_CHECK_DUP))
-        {
-          uint32_t key_nr= to->cursor->get_dup_key(error);
-          if ((int) key_nr >= 0)
-          {
-            const char *err_msg= ER(ER_DUP_ENTRY_WITH_KEY_NAME);
-            if (key_nr == 0 &&
-                (to->key_info[0].key_part[0].field->flags &
-                 AUTO_INCREMENT_FLAG))
-              err_msg= ER(ER_DUP_ENTRY_AUTOINCREMENT_CASE);
-            to->cursor->print_keydup_error(key_nr, err_msg);
-            break;
-          }
-        }
-
-        to->cursor->print_error(error,MYF(0));
+      { 
+        to->print_error(error, MYF(0));
         break;
       }
       to->cursor->restore_auto_increment(prev_insert_id);
@@ -1421,7 +1384,7 @@
 
   if (to->cursor->ha_end_bulk_insert() && error <= 0)
   {
-    to->cursor->print_error(my_errno,MYF(0));
+    to->print_error(my_errno, MYF(0));
     error=1;
   }
   to->cursor->extra(HA_EXTRA_NO_IGNORE_DUP_KEY);

=== modified file 'drizzled/statement/create_index.cc'
--- drizzled/statement/create_index.cc	2009-09-27 23:05:13 +0000
+++ drizzled/statement/create_index.cc	2009-11-17 02:26:31 +0000
@@ -47,7 +47,7 @@
   }
 
   create_info.row_type= ROW_TYPE_NOT_USED;
-  create_info.default_table_charset= get_default_db_collation(session->db);
+  create_info.default_table_charset= get_default_db_collation(session->db.c_str());
 
   bool res= alter_table(session, 
                         first_table->db, 

=== modified file 'drizzled/statement/create_table.cc'
--- drizzled/statement/create_table.cc	2009-10-30 15:58:06 +0000
+++ drizzled/statement/create_table.cc	2009-11-18 03:04:25 +0000
@@ -58,7 +58,6 @@
   }
 
 
-
   /* 
     Now we set the name in our Table proto so that it will match 
     create_info.db_type.
@@ -136,7 +135,7 @@
       {
         TableList *duplicate= NULL;
         create_table= session->lex->unlink_first_table(&link_to_local);
-        if ((duplicate= unique_table(session, create_table, select_tables, 0)))
+        if ((duplicate= unique_table(create_table, select_tables)))
         {
           my_error(ER_UPDATE_TABLE_USED, MYF(0), create_table->alias);
           /*
@@ -189,6 +188,14 @@
     }
     else
     {
+
+      for (int32_t x= 0; x < alter_info.alter_proto.added_field_size(); x++)
+      {
+        message::Table::Field *field= create_table_proto.add_field();
+
+        *field= alter_info.alter_proto.added_field(x);
+      }
+
       res= mysql_create_table(session, 
                               create_table->db,
                               create_table->table_name, 

=== modified file 'drizzled/statement/create_table.h'
--- drizzled/statement/create_table.h	2009-09-23 21:37:46 +0000
+++ drizzled/statement/create_table.h	2009-11-16 02:35:00 +0000
@@ -42,6 +42,7 @@
 
   bool execute();
   drizzled::message::Table create_table_proto;
+  drizzled::message::Table::Field *current_proto_field;
   HA_CREATE_INFO create_info;
   AlterInfo alter_info;
   KEY_CREATE_INFO key_create_info;

=== modified file 'drizzled/statement/drop_index.cc'
--- drizzled/statement/drop_index.cc	2009-09-27 23:05:13 +0000
+++ drizzled/statement/drop_index.cc	2009-11-17 02:26:31 +0000
@@ -51,7 +51,7 @@
   memset(&create_info, 0, sizeof(create_info));
   create_info.db_type= 0;
   create_info.row_type= ROW_TYPE_NOT_USED;
-  create_info.default_table_charset= get_default_db_collation(session->db);
+  create_info.default_table_charset= get_default_db_collation(session->db.c_str());
 
   bool res= alter_table(session, 
                         first_table->db, 

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2009-11-14 22:21:51 +0000
+++ drizzled/table.cc	2009-11-17 17:45:47 +0000
@@ -38,6 +38,7 @@
 #include <drizzled/item/decimal.h>
 #include <drizzled/item/float.h>
 #include <drizzled/item/null.h>
+#include <drizzled/temporal.h>
 
 #include "drizzled/table_proto.h"
 
@@ -485,7 +486,7 @@
   for (unsigned int fieldnr= 0; fieldnr < share->fields; fieldnr++)
   {
     message::Table::Field pfield= table.field(fieldnr);
-    if (pfield.has_constraints() && pfield.constraints().is_nullable())
+    if (pfield.constraints().is_nullable())
       null_fields++;
 
     enum_field_types drizzle_field_type=
@@ -820,10 +821,24 @@
     temp_table.s->db_low_byte_first= 1; //Cursor->low_byte_first();
     temp_table.s->blob_ptr_size= portable_sizeof_char_ptr;
 
-    uint32_t field_length;
+    uint32_t field_length= 0; //Assignment is for compiler complaint.
 
     switch (field_type)
     {
+    case DRIZZLE_TYPE_BLOB:
+    case DRIZZLE_TYPE_VARCHAR:
+    {
+      message::Table::Field::StringFieldOptions field_options= pfield.string_options();
+
+      charset= get_charset(field_options.has_collation_id() ?
+                           field_options.collation_id() : 0);
+
+      if (! charset)
+      	charset= default_charset_info;
+
+      field_length= field_options.length() * charset->mbmaxlen;
+    }
+      break;
     case DRIZZLE_TYPE_DOUBLE:
     {
       message::Table::Field::NumericFieldOptions fo= pfield.numeric_options();
@@ -852,16 +867,48 @@
                                                    false);
       break;
     }
-    default:
-      field_length= pfield.options().length();
-      break;
+    case DRIZZLE_TYPE_TIMESTAMP:
+    case DRIZZLE_TYPE_DATETIME:
+      field_length= drizzled::DateTime::MAX_STRING_LENGTH;
+      break;
+    case DRIZZLE_TYPE_DATE:
+      field_length= drizzled::Date::MAX_STRING_LENGTH;
+      break;
+    case DRIZZLE_TYPE_ENUM:
+    {
+      field_length= 0;
+
+      message::Table::Field::SetFieldOptions fo= pfield.set_options();
+
+      for(int valnr= 0; valnr < fo.field_value_size(); valnr++)
+      {
+        if (fo.field_value(valnr).length() > field_length)
+          field_length= charset->cset->numchars(charset,
+                                                fo.field_value(valnr).c_str(),
+                                                fo.field_value(valnr).c_str()
+                                                + fo.field_value(valnr).length())
+            * charset->mbmaxlen;
+      }
+    }
+      break;
+    case DRIZZLE_TYPE_LONG:
+      {
+        uint32_t sign_len= pfield.constraints().is_unsigned() ? 0 : 1;
+          field_length= MAX_INT_WIDTH+sign_len;
+      }
+      break;
+    case DRIZZLE_TYPE_LONGLONG:
+      field_length= MAX_BIGINT_WIDTH;
+      break;
+    case DRIZZLE_TYPE_NULL:
+      abort(); // Programming error
     }
 
     Field* f= make_field(share,
                          &share->mem_root,
                          record + field_offsets[fieldnr] + data_offset,
                          field_length,
-                         pfield.has_constraints() && pfield.constraints().is_nullable() ? true : false,
+                         pfield.constraints().is_nullable(),
                          null_pos,
                          null_bit_pos,
                          decimals,
@@ -1472,7 +1519,7 @@
 	  my_errno= EMFILE;
           break;
         default:
-          outparam->cursor->print_error(ha_err, MYF(0));
+          outparam->print_error(ha_err, MYF(0));
           error_reported= true;
           if (ha_err == HA_ERR_TABLE_DEF_CHANGED)
             error= 7;
@@ -3085,7 +3132,7 @@
   if ((error=cursor->ha_open(this, s->table_name.str,O_RDWR,
                                   HA_OPEN_TMP_TABLE | HA_OPEN_INTERNAL_TABLE)))
   {
-    cursor->print_error(error,MYF(0));
+    print_error(error, MYF(0));
     db_stat= 0;
     return true;
   }
@@ -3218,7 +3265,7 @@
 		       &create_info,
 		       HA_CREATE_TMP_TABLE)))
   {
-    cursor->print_error(error,MYF(0));
+    print_error(error, MYF(0));
     db_stat= 0;
     goto err;
   }
@@ -3278,7 +3325,7 @@
   if (table->s->db_type() != heap_engine ||
       error != HA_ERR_RECORD_FILE_FULL)
   {
-    table->cursor->print_error(error,MYF(0));
+    table->print_error(error, MYF(0));
     return true;
   }
 
@@ -3354,7 +3401,7 @@
   return false;
 
  err:
-  table->cursor->print_error(write_err, MYF(0));
+  table->print_error(write_err, MYF(0));
   (void) table->cursor->ha_rnd_end();
   (void) new_table.cursor->close();
  err1:
@@ -3514,7 +3561,7 @@
   if (error != HA_ERR_LOCK_DEADLOCK && error != HA_ERR_LOCK_WAIT_TIMEOUT)
     errmsg_printf(ERRMSG_LVL_ERROR, _("Got error %d when reading table '%s'"),
 		    error, s->path.str);
-  cursor->print_error(error,MYF(0));
+  print_error(error, MYF(0));
 
   return 1;
 }

=== modified file 'drizzled/table.h'
--- drizzled/table.h	2009-11-11 19:42:27 +0000
+++ drizzled/table.h	2009-11-16 18:23:07 +0000
@@ -47,7 +47,7 @@
 class Field_timestamp;
 class Field_blob;
 
-extern drizzled::atomic<uint32_t> refresh_version;
+extern uint64_t refresh_version;
 
 typedef enum enum_table_category TABLE_CATEGORY;
 
@@ -583,6 +583,26 @@
   void free_io_cache();
   void filesort_free_buffers(bool full= false);
   void intern_close_table();
+
+  void print_error(int error, myf errflag)
+  {
+    s->storage_engine->print_error(error, errflag, *this);
+  }
+
+  /**
+    @return
+    key if error because of duplicated keys
+  */
+  uint32_t get_dup_key(int error)
+  {
+    cursor->errkey  = (uint32_t) -1;
+    if (error == HA_ERR_FOUND_DUPP_KEY || error == HA_ERR_FOREIGN_DUPLICATE_KEY ||
+        error == HA_ERR_FOUND_DUPP_UNIQUE ||
+        error == HA_ERR_DROP_INDEX_FK)
+      cursor->info(HA_STATUS_ERRKEY | HA_STATUS_NO_LOCK);
+
+    return(cursor->errkey);
+  }
 };
 
 Table *create_virtual_tmp_table(Session *session, List<CreateField> &field_list);

=== modified file 'drizzled/table_proto_write.cc'
--- drizzled/table_proto_write.cc	2009-11-11 15:49:59 +0000
+++ drizzled/table_proto_write.cc	2009-11-16 02:35:23 +0000
@@ -54,21 +54,40 @@
 
   assert(strcmp(table_proto->name().c_str(),table_name)==0);
 
+  int field_number= 0;
+  bool use_existing_fields= table_proto->field_size() > 0;
   while ((field_arg= it++))
   {
     message::Table::Field *attribute;
 
-    attribute= table_proto->add_field();
-    attribute->set_name(field_arg->field_name);
+    /* some (one) code path for CREATE TABLE fills the proto
+       out more than the others, so we already have partially
+       filled out Field messages */
 
-    if(! (field_arg->flags & NOT_NULL_FLAG))
+    if (use_existing_fields)
+      attribute= table_proto->mutable_field(field_number++);
+    else
     {
-      message::Table::Field::FieldConstraints *constraints;
-
-      constraints= attribute->mutable_constraints();
-      constraints->set_is_nullable(true);
+      /* Other code paths still have to fill out the proto */
+      attribute= table_proto->add_field();
+
+      if(field_arg->flags & NOT_NULL_FLAG)
+      {
+        message::Table::Field::FieldConstraints *constraints;
+
+        constraints= attribute->mutable_constraints();
+        constraints->set_is_nullable(false);
+      }
+
+      attribute->set_name(field_arg->field_name);
     }
 
+    assert((!(field_arg->flags & NOT_NULL_FLAG)) == attribute->constraints().is_nullable());
+    assert(strcmp(attribute->name().c_str(), field_arg->field_name)==0);
+
+
+    message::Table::Field::FieldType parser_type= attribute->type();
+
     switch (field_arg->sql_type) {
     case DRIZZLE_TYPE_LONG:
       attribute->set_type(message::Table::Field::INTEGER);
@@ -112,11 +131,17 @@
 
         string_field_options= attribute->mutable_string_options();
         attribute->set_type(message::Table::Field::VARCHAR);
-        string_field_options->set_length(field_arg->length
-					 / field_arg->charset->mbmaxlen);
-        string_field_options->set_collation_id(field_arg->charset->number);
-        string_field_options->set_collation(field_arg->charset->name);
+        if (! use_existing_fields || string_field_options->length()==0)
+          string_field_options->set_length(field_arg->length
+                                           / field_arg->charset->mbmaxlen);
+        else
+          assert((uint32_t)string_field_options->length() == (uint32_t)(field_arg->length / field_arg->charset->mbmaxlen));
 
+        if (! string_field_options->has_collation())
+        {
+          string_field_options->set_collation_id(field_arg->charset->number);
+          string_field_options->set_collation(field_arg->charset->name);
+        }
         break;
       }
     case DRIZZLE_TYPE_DECIMAL:
@@ -166,6 +191,8 @@
       assert(0); /* Tell us, since this shouldn't happend */
     }
 
+    assert (!use_existing_fields || parser_type == attribute->type());
+
 #ifdef NOTDONE
     field_constraints= attribute->mutable_constraints();
     constraints->set_is_nullable(field_arg->def->null_value);
@@ -205,7 +232,10 @@
 	return(1);
       }
 
-      attribute->set_comment(field_arg->comment.str);
+      if (! use_existing_fields)
+        attribute->set_comment(field_arg->comment.str);
+
+      assert(strcmp(attribute->comment().c_str(), field_arg->comment.str)==0);
     }
 
     if(field_arg->unireg_check == Field::NEXT_NUMBER)
@@ -273,15 +303,6 @@
       }
     }
 
-    if (field_arg->sql_type != DRIZZLE_TYPE_DECIMAL
-        && field_arg->sql_type != DRIZZLE_TYPE_DOUBLE)
-    {
-      message::Table::Field::FieldOptions *field_options;
-      field_options= attribute->mutable_options();
-
-      field_options->set_length(field_arg->length);
-    }
-
     assert(field_arg->unireg_check == Field::NONE
 	   || field_arg->unireg_check == Field::NEXT_NUMBER
 	   || field_arg->unireg_check == Field::TIMESTAMP_DN_FIELD
@@ -290,6 +311,8 @@
 
   }
 
+  assert(! use_existing_fields || (field_number == table_proto->field_size()));
+
   switch(create_info->row_type)
   {
   case ROW_TYPE_DEFAULT:

=== modified file 'extra/cpplint.py'
--- extra/cpplint.py	2009-10-15 18:16:17 +0000
+++ extra/cpplint.py	2009-11-23 20:04:18 +0000
@@ -10,7 +10,7 @@
 #
 # b) the "Artistic License".
 
-# Modified for Drizzle by Monty Taylor
+# Modified for Drizzle by Monty Taylor & Robert Collins
 
 # Here are some issues that I've had people identify in my code during reviews,
 # that I think are possible to flag automatically in a lint tool.  If these were
@@ -75,7 +75,7 @@
 
 
 _USAGE = """
-Syntax: cpplint.py [--verbose=#] [--output=vs7] [--filter=-x,+y,...]
+Syntax: cpplint.py [--verbose=#] [--output=vs7] [--deps=path] [--filter=-x,+y,...]
         <file> [file] ...
 
   The style guidelines this tries to follow are those in
@@ -100,6 +100,13 @@
     verbose=#
       Specify a number 0-5 to restrict errors to certain verbosity levels.
 
+    deps=path
+      write out a Makefile that can be included listing the files included
+      during the lint process as make style dependencies with null-build rules
+      (so that deleted headers will not cause failures). This can be useful to
+      perform on-demand linting. The top level target will be the name of the
+      lint file itself.
+
     filter=-x,+y,...
       Specify a comma-separated list of category-filters to apply: only
       error messages whose category names pass the filters will be printed.
@@ -378,6 +385,31 @@
     # "vs7" - format that Microsoft Visual Studio 7 can parse
     self.output_format = 'emacs'
 
+    # deps
+    self.depfilename = None
+    self.seen_fnames = set()
+
+  def finished(self):
+    """Complete things that wait for the end of the lint operation."""
+    if self.depfilename is not None:
+      if self.error_count:
+          # Don't alter dependency data
+          return
+      depfile = file(self.depfilename, 'w+')
+      # depend on what we read
+      depfile.write("%s: " % self.depfilename)
+      names = sorted(self.seen_fnames)
+      depfile.write(' '.join(names))
+      depfile.write('\n')
+      # anything we read shouldn't cause an error if its missing - so claim
+      # it can be made.
+      for name in names:
+        depfile.write('%s:\n' % name)
+      depfile.close()
+
+  def seen_file(self, fname):
+    self.seen_fnames.add(fname)
+
   def SetOutputFormat(self, output_format):
     """Sets the output format for errors."""
     self.output_format = output_format
@@ -2582,6 +2614,7 @@
     headerfile = io.open(filename, 'r', 'utf8', 'replace')
   except IOError:
     return False
+  _cpplint_state.seen_file(filename)
   linenum = 0
   for line in headerfile:
     linenum += 1
@@ -2788,6 +2821,7 @@
                                         codecs.getwriter('utf8'),
                                         'replace').read().split('\n')
     else:
+      _cpplint_state.seen_file(filename)
       lines = codecs.open(filename, 'r', 'utf8', 'replace').read().split('\n')
 
     carriage_return_found = False
@@ -2856,8 +2890,8 @@
     The list of filenames to lint.
   """
   try:
-    (opts, filenames) = getopt.getopt(args, '', ['help', 'output=', 'verbose=',
-                                                 'filter='])
+    (opts, filenames) = getopt.getopt(args, '',
+        ['help', 'output=', 'verbose=', 'deps=', 'filter='])
   except getopt.GetoptError:
     PrintUsage('Invalid arguments.')
 
@@ -2878,6 +2912,8 @@
       filters = val
       if not filters:
         PrintCategories()
+    elif opt == '--deps':
+        _cpplint_state.depfilename = val
 
   if not filenames:
     PrintUsage('No files were specified.')
@@ -2902,6 +2938,7 @@
   _cpplint_state.ResetErrorCount()
   for filename in filenames:
     ProcessFile(filename, _cpplint_state.verbose_level)
+  _cpplint_state.finished()
   sys.stderr.write('Total errors found: %d\n' % _cpplint_state.error_count)
   sys.exit(_cpplint_state.error_count > 0)
 

=== modified file 'extra/run_cpplint.sh'
--- extra/run_cpplint.sh	2009-11-11 04:20:32 +0000
+++ extra/run_cpplint.sh	2009-11-23 20:04:18 +0000
@@ -22,7 +22,8 @@
 
 command="python extra/cpplint.py  --filter=-whitespace,-runtime,-readability,+legal,-build,+build/header_guard,+build/include_config,+build/namespaces"
 if test "x$1" = "x" ; then
-  find plugin drizzled client -type f | egrep '\.(cc|h)$' | grep -v innobase | grep -v gnulib | grep -v '\.pb\.'| grep -v bak-header | grep -v '^intl' | grep -v '^config' | grep -v '\.am$' | grep -v '\.ac$' | grep -v m4 | grep -v sql_yacc.yy | grep -v '.gperf$' | grep -v 'drizzled/probes.h' | grep -v 'drizzled/function_hash.h' | grep -v 'drizzled/symbol_hash.h' | grep -v 'util/dummy.cc' | grep -v 'drizzled/sql_yacc.h' | grep -v 'drizzled/configmake.h' | xargs $command
+  echo "ERROR: supply a filename to line."
+  exit 1
 else
   $command $1
 fi

=== added file 'm4/pandora_have_libevent.m4'
--- m4/pandora_have_libevent.m4	1970-01-01 00:00:00 +0000
+++ m4/pandora_have_libevent.m4	2009-11-24 18:25:09 +0000
@@ -0,0 +1,57 @@
+dnl  Copyright (C) 2009 Sun Microsystems
+dnl This file is free software; Sun Microsystems
+dnl gives unlimited permission to copy and/or distribute it,
+dnl with or without modifications, as long as this notice is preserved.
+
+#--------------------------------------------------------------------
+# Check for libevent
+#--------------------------------------------------------------------
+
+
+AC_DEFUN([_PANDORA_SEARCH_LIBEVENT],[
+  AC_REQUIRE([AC_LIB_PREFIX])
+
+  AC_LIB_HAVE_LINKFLAGS(event,,
+  [
+    #include <sys/types.h>
+    #include <sys/time.h>
+    #include <stdlib.h>
+    #include <event.h>
+  ],[
+    struct bufferevent bev;
+    bufferevent_settimeout(&bev, 1, 1);
+    event_init();
+    event_loop(EVLOOP_ONCE);
+  ]) 
+
+  AM_CONDITIONAL(HAVE_LIBEVENT, [test "x${ac_cv_libevent}" = "xyes"])
+])
+
+AC_DEFUN([_PANDORA_HAVE_LIBEVENT],[
+
+  AC_ARG_ENABLE([libevent],
+    [AS_HELP_STRING([--disable-libevent],
+      [Build with libevent support @<:@default=on@:>@])],
+    [ac_enable_libevent="$enableval"],
+    [ac_enable_libevent="yes"])
+
+  _PANDORA_SEARCH_LIBEVENT
+])
+
+
+AC_DEFUN([PANDORA_HAVE_LIBEVENT],[
+  AC_REQUIRE([_PANDORA_HAVE_LIBEVENT])
+])
+
+AC_DEFUN([_PANDORA_REQUIRE_LIBEVENT],[
+  ac_enable_libevent="yes"
+  _PANDORA_SEARCH_LIBEVENT
+
+  AS_IF([test x$ac_cv_libevent = xno],[
+    AC_MSG_ERROR([libevent is required for ${PACKAGE}. On Debian this can be found in libevent-dev. On RedHat this can be found in libevent-devel.])
+  ])
+])
+
+AC_DEFUN([PANDORA_REQUIRE_LIBEVENT],[
+  AC_REQUIRE([_PANDORA_REQUIRE_LIBEVENT])
+])

=== added file 'm4/pandora_run_cpplint.m4'
--- m4/pandora_run_cpplint.m4	1970-01-01 00:00:00 +0000
+++ m4/pandora_run_cpplint.m4	2009-11-24 18:09:52 +0000
@@ -0,0 +1,8 @@
+dnl  Copyright (C) 2009 Sun Microsystems
+dnl This file is free software; Sun Microsystems
+dnl gives unlimited permission to copy and/or distribute it,
+dnl with or without modifications, as long as this notice is preserved.
+
+AC_DEFUN([PANDORA_RUN_CPPLINT],[
+  m4_syscmd([python config/make-lint.py])
+])

=== modified file 'plugin/archive/ha_archive.cc'
--- plugin/archive/ha_archive.cc	2009-11-13 18:55:43 +0000
+++ plugin/archive/ha_archive.cc	2009-11-16 23:04:22 +0000
@@ -145,6 +145,13 @@
     table_definition_ext= ARZ;
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NO_TRANSACTIONS |
+            HA_STATS_RECORDS_IS_EXACT |
+            HA_HAS_RECORDS);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {
@@ -230,7 +237,7 @@
                                         drizzled::message::Table *table_proto)
 {
   struct stat stat_info;
-  int error= 0;
+  int error= ENOENT;
   string proto_path;
 
   proto_path.reserve(FN_REFLEN);
@@ -240,12 +247,14 @@
 
   if (stat(proto_path.c_str(),&stat_info))
     return errno;
+  else
+    error= EEXIST;
 
   if (table_proto)
   {
     azio_stream proto_stream;
     char* proto_string;
-    if(azopen(&proto_stream, proto_path.c_str(), O_RDONLY, AZ_METHOD_BLOCK) == 0)
+    if (azopen(&proto_stream, proto_path.c_str(), O_RDONLY, AZ_METHOD_BLOCK) == 0)
       return HA_ERR_CRASHED_ON_USAGE;
 
     proto_string= (char*)malloc(sizeof(char) * proto_stream.frm_length);
@@ -257,14 +266,14 @@
 
     azread_frm(&proto_stream, proto_string);
 
-    if(table_proto->ParseFromArray(proto_string, proto_stream.frm_length) == false)
+    if (table_proto->ParseFromArray(proto_string, proto_stream.frm_length) == false)
       error= HA_ERR_CRASHED_ON_USAGE;
 
     azclose(&proto_stream);
     free(proto_string);
   }
 
-  return EEXIST;
+  return error;
 }
 
 static ArchiveEngine *archive_engine= NULL;
@@ -1042,7 +1051,7 @@
 
 
 /*
-  Thanks to the table flag HA_REC_NOT_IN_SEQ this will be called after
+  Thanks to the table bool is_ordered this will be called after
   each call to ha_archive::rnd_next() if an ordering of the rows is
   needed.
 */

=== modified file 'plugin/archive/ha_archive.h'
--- plugin/archive/ha_archive.h	2009-11-11 19:42:27 +0000
+++ plugin/archive/ha_archive.h	2009-11-16 20:30:19 +0000
@@ -96,12 +96,6 @@
 
   const char *index_type(uint32_t)
   { return "NONE"; }
-  uint64_t table_flags() const
-  {
-    return (HA_NO_TRANSACTIONS | HA_REC_NOT_IN_SEQ |
-            HA_STATS_RECORDS_IS_EXACT |
-            HA_HAS_RECORDS);
-  }
   uint32_t index_flags(uint32_t, uint32_t, bool) const
   {
     return HA_ONLY_WHOLE_INDEX;

=== modified file 'plugin/blackhole/ha_blackhole.cc'
--- plugin/blackhole/ha_blackhole.cc	2009-11-13 18:55:43 +0000
+++ plugin/blackhole/ha_blackhole.cc	2009-11-16 23:04:22 +0000
@@ -46,6 +46,11 @@
     table_definition_ext= BLACKHOLE_EXT;
   }
 
+  uint64_t table_flags() const
+  {
+    return(HA_NULL_IN_KEY | HA_CAN_INDEX_BLOBS | HA_AUTO_PART_KEY);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {
@@ -197,20 +202,20 @@
 
   if (fd == -1)
   {
-    return -1;
+    return errno;
   }
 
   google::protobuf::io::ZeroCopyInputStream* input=
     new google::protobuf::io::FileInputStream(fd);
 
   if (! input)
-    return -1;
+    return HA_ERR_CRASHED_ON_USAGE;
 
   if (table_proto && ! table_proto->ParseFromZeroCopyStream(input))
   {
     close(fd);
     delete input;
-    return -1;
+    return HA_ERR_CRASHED_ON_USAGE;
   }
 
   delete input;

=== modified file 'plugin/blackhole/ha_blackhole.h'
--- plugin/blackhole/ha_blackhole.h	2009-11-11 19:42:27 +0000
+++ plugin/blackhole/ha_blackhole.h	2009-11-16 20:30:19 +0000
@@ -53,10 +53,6 @@
     don't implement this method unless you really have indexes
   */
   const char *index_type(uint32_t key_number);
-  uint64_t table_flags() const
-  {
-    return(HA_NULL_IN_KEY | HA_CAN_INDEX_BLOBS | HA_AUTO_PART_KEY);
-  }
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   /* The following defines can be increased if necessary */
   uint32_t max_supported_keys()          const { return BLACKHOLE_MAX_KEY; }

=== modified file 'plugin/csv/ha_tina.cc'
--- plugin/csv/ha_tina.cc	2009-11-13 18:55:43 +0000
+++ plugin/csv/ha_tina.cc	2009-11-16 23:04:22 +0000
@@ -129,6 +129,11 @@
     return new (mem_root) ha_tina(*this, table);
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NO_TRANSACTIONS | HA_NO_AUTO_INCREMENT);
+  }
+
   const char **bas_ext() const {
     return ha_tina_exts;
   }
@@ -189,7 +194,7 @@
                                const bool,
                                drizzled::message::Table *table_proto)
 {
-  int error= 1;
+  int error= ENOENT;
   ProtoCache::iterator iter;
 
   pthread_mutex_lock(&proto_cache_mutex);

=== modified file 'plugin/csv/ha_tina.h'
--- plugin/csv/ha_tina.h	2009-11-11 19:42:27 +0000
+++ plugin/csv/ha_tina.h	2009-11-16 20:30:19 +0000
@@ -101,10 +101,6 @@
   const char *table_type(void) const { return "CSV"; }
   const char *index_type(uint32_t)
   { return "NONE"; }
-  uint64_t table_flags() const
-  {
-    return (HA_NO_TRANSACTIONS | HA_REC_NOT_IN_SEQ | HA_NO_AUTO_INCREMENT);
-  }
   uint32_t index_flags(uint32_t, uint32_t, bool) const
   {
     /*

=== modified file 'plugin/heap/ha_heap.cc'
--- plugin/heap/ha_heap.cc	2009-11-13 18:55:43 +0000
+++ plugin/heap/ha_heap.cc	2009-11-16 23:04:22 +0000
@@ -46,6 +46,15 @@
     addAlias("HEAP");
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_FAST_KEY_READ |
+            HA_NO_BLOBS | HA_NULL_IN_KEY |
+            HA_NO_TRANSACTIONS |
+            HA_HAS_RECORDS |
+            HA_STATS_RECORDS_IS_EXACT);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {
@@ -95,7 +104,7 @@
                                      const bool,
                                      drizzled::message::Table *table_proto)
 {
-  int error= 1;
+  int error= ENOENT;
   ProtoCache::iterator iter;
 
   pthread_mutex_lock(&proto_cache_mutex);

=== modified file 'plugin/heap/ha_heap.h'
--- plugin/heap/ha_heap.h	2009-11-11 19:42:27 +0000
+++ plugin/heap/ha_heap.h	2009-11-16 20:30:19 +0000
@@ -42,12 +42,6 @@
 
   const char *index_type(uint32_t inx);
   enum row_type get_row_type() const;
-  uint64_t table_flags() const
-  {
-    return (HA_FAST_KEY_READ | HA_NO_BLOBS | HA_NULL_IN_KEY |
-            HA_REC_NOT_IN_SEQ | HA_NO_TRANSACTIONS |
-            HA_HAS_RECORDS | HA_STATS_RECORDS_IS_EXACT);
-  }
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   const key_map *keys_to_use_for_scanning() { return &btree_keys; }
   uint32_t max_supported_keys()          const { return MAX_KEY; }

=== added file 'plugin/info_schema/character_set.cc'
--- plugin/info_schema/character_set.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/character_set.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,197 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Character Set I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "character_set.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the character set I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the character set I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * character set I_S table.
+ */
+static plugin::InfoSchemaTable *char_set_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return false on success; true on failure.
+ */
+vector<const plugin::ColumnInfo *> *CharacterSetIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  /*
+   * Create each column for the CHARACTER_SET table.
+   */
+  columns->push_back(new plugin::ColumnInfo("CHARACTER_SET_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Charset",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DEFAULT_COLLATE_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Default collation",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DESCRIPTION",
+                                            60,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Description",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MAXLEN",
+                                            3,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Maxlen",
+                                            SKIP_OPEN_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *CharacterSetIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new CharSetISMethods();
+  }
+
+  if (char_set_table == NULL)
+  {
+    char_set_table= new plugin::InfoSchemaTable("CHARACTER_SETS",
+                                                *columns,
+                                                -1, -1, false, false, 0,
+                                                methods);
+  }
+
+  return char_set_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void CharacterSetIS::cleanup()
+{
+  clearColumns(*columns);
+  delete char_set_table;
+  delete methods;
+  delete columns;
+}
+
+int CharSetISMethods::fillTable(Session *session, TableList *tables)
+{
+  CHARSET_INFO **cs;
+  const char *wild= session->lex->wild ? session->lex->wild->ptr() : NULL;
+  Table *table= tables->table;
+  const CHARSET_INFO * const scs= system_charset_info;
+
+  for (cs= all_charsets ; cs < all_charsets+255 ; cs++)
+  {
+    const CHARSET_INFO * const tmp_cs= cs[0];
+    if (tmp_cs && (tmp_cs->state & MY_CS_PRIMARY) &&
+        (tmp_cs->state & MY_CS_AVAILABLE) &&
+        !(tmp_cs->state & MY_CS_HIDDEN) &&
+        !(wild && wild[0] &&
+          wild_case_compare(scs, tmp_cs->csname,wild)))
+    {
+      const char *comment;
+      table->restoreRecordAsDefault();
+      table->field[0]->store(tmp_cs->csname, strlen(tmp_cs->csname), scs);
+      table->field[1]->store(tmp_cs->name, strlen(tmp_cs->name), scs);
+      comment= tmp_cs->comment ? tmp_cs->comment : "";
+      table->field[2]->store(comment, strlen(comment), scs);
+      table->field[3]->store((int64_t) tmp_cs->mbmaxlen, true);
+      if (schema_table_store_record(session, table))
+        return 1;
+    }
+  }
+  return 0;
+}
+
+int CharSetISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
+  const
+{
+  int fields_arr[]= {0, 2, 1, 3, -1};
+  int *field_num= fields_arr;
+  const drizzled::plugin::InfoSchemaTable::Columns tab_columns= schema_table->getColumns();
+  const drizzled::plugin::ColumnInfo *column= NULL;
+  Name_resolution_context *context= &session->lex->select_lex.context;
+
+  for (; *field_num >= 0; field_num++)
+  {
+    column= tab_columns[*field_num];
+    Item_field *field= new Item_field(context,
+                                      NULL, NULL, column->getName().c_str());
+    if (field)
+    {
+      field->set_name(column->getOldName().c_str(),
+                      column->getOldName().length(),
+                      system_charset_info);
+      if (session->add_item_to_list(field))
+        return 1;
+    }
+  }
+  return 0;
+}

=== added file 'plugin/info_schema/character_set.h'
--- plugin/info_schema/character_set.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/character_set.h	2009-11-16 03:00:14 +0000
@@ -0,0 +1,59 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_CHARACTER_SET_H
+#define PLUGIN_INFO_SCHEMA_CHARACTER_SET_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class CharSetISMethods
+ * @brief
+ *   Class which implements any methods that the
+ *   CHARACTER_SET I_S table needs besides the default
+ *   methods.
+ */
+class CharSetISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+  virtual int oldFormat(Session *session,
+                        drizzled::plugin::InfoSchemaTable *schema_table) const;
+};
+
+class CharacterSetIS 
+{
+  /**
+   * Create the various columns for the CHARACTER_SET I_S table and add them
+   * to the std::vector of columns for the CHARACTER_SET table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_CHARACTER_SET_H */
+
+

=== added file 'plugin/info_schema/collation.cc'
--- plugin/info_schema/collation.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/collation.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,206 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Character Set I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "collation.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the collation I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the collation I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * collation I_S table.
+ */
+static plugin::InfoSchemaTable *coll_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *CollationIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  /*
+   * Create each column for the COLLATION table.
+   */
+  columns->push_back(new plugin::ColumnInfo("COLLATION_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Collation",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CHARACTER_SET_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Default collation",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DESCRIPTION",
+                                            60,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Charset",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("ID",
+                                            MY_INT32_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Id",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("IS_DEFAULT",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Default",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("IS_COMPILED",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Compiled",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("SORTLEN",
+                                            3,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Sortlen",
+                                            SKIP_OPEN_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *CollationIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new CollationISMethods();
+  }
+
+  if (coll_table == NULL)
+  {
+    coll_table= new(nothrow) plugin::InfoSchemaTable("COLLATIONS",
+                                                     *columns,
+                                                     -1, -1, false, false, 0,
+                                                     methods);
+  }
+
+  return coll_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void CollationIS::cleanup()
+{
+  clearColumns(*columns);
+  delete coll_table;
+  delete methods;
+  delete columns;
+}
+
+int CollationISMethods::fillTable(Session *session, TableList *tables)
+{
+  CHARSET_INFO **cs;
+  const char *wild= session->lex->wild ? session->lex->wild->ptr() : NULL;
+  Table *table= tables->table;
+  const CHARSET_INFO * const scs= system_charset_info;
+  for (cs= all_charsets ; cs < all_charsets+255 ; cs++ )
+  {
+    CHARSET_INFO **cl;
+    const CHARSET_INFO *tmp_cs= cs[0];
+    if (! tmp_cs || ! (tmp_cs->state & MY_CS_AVAILABLE) ||
+         (tmp_cs->state & MY_CS_HIDDEN) ||
+        !(tmp_cs->state & MY_CS_PRIMARY))
+      continue;
+    for (cl= all_charsets; cl < all_charsets+255 ;cl ++)
+    {
+      const CHARSET_INFO *tmp_cl= cl[0];
+      if (! tmp_cl || ! (tmp_cl->state & MY_CS_AVAILABLE) ||
+          !my_charset_same(tmp_cs, tmp_cl))
+        continue;
+      if (! (wild && wild[0] &&
+          wild_case_compare(scs, tmp_cl->name,wild)))
+      {
+        const char *tmp_buff;
+        table->restoreRecordAsDefault();
+        table->field[0]->store(tmp_cl->name, strlen(tmp_cl->name), scs);
+        table->field[1]->store(tmp_cl->csname , strlen(tmp_cl->csname), scs);
+        table->field[2]->store((int64_t) tmp_cl->number, true);
+        tmp_buff= (tmp_cl->state & MY_CS_PRIMARY) ? "Yes" : "";
+        table->field[3]->store(tmp_buff, strlen(tmp_buff), scs);
+        tmp_buff= (tmp_cl->state & MY_CS_COMPILED)? "Yes" : "";
+        table->field[4]->store(tmp_buff, strlen(tmp_buff), scs);
+        table->field[5]->store((int64_t) tmp_cl->strxfrm_multiply, true);
+        if (schema_table_store_record(session, table))
+          return 1;
+      }
+    }
+  }
+  return 0;
+}

=== added file 'plugin/info_schema/collation.h'
--- plugin/info_schema/collation.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/collation.h	2009-11-16 03:25:39 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_COLLATION_H
+#define PLUGIN_INFO_SCHEMA_COLLATION_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class CollationISMethods
+ * @brief
+ *   Class which implements any methods that the Collations
+ *   I_S table needs besides the default methods
+ */
+class CollationISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+};
+
+class CollationIS
+{
+  /**
+   * Create the various columns for the COLLATION I_S table and add them
+   * to the std::vector of columns for the COLLATION table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_COLLATION_H */

=== added file 'plugin/info_schema/collation_char_set.cc'
--- plugin/info_schema/collation_char_set.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/collation_char_set.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,154 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Collation Character Set I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "collation_char_set.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the collation char set I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the collation char set I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * collation char set I_S table.
+ */
+static plugin::InfoSchemaTable *coll_cs_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *CollationCharSetIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  /*
+   * Create each column for the COLLATION_CHAR_SET table.
+   */
+  columns->push_back(new plugin::ColumnInfo("COLLATION_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CHARACTER_SET_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *CollationCharSetIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new CollCharISMethods();
+  }
+
+  if (coll_cs_table == NULL)
+  {
+    coll_cs_table= new plugin::InfoSchemaTable("COLLATION_CHARACTER_SET_APPLICABILITY",
+                                               *columns,
+                                               -1, -1, false, false, 0,
+                                               methods);
+  }
+
+  return coll_cs_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void CollationCharSetIS::cleanup()
+{
+  clearColumns(*columns);
+  delete coll_cs_table;
+  delete methods;
+  delete columns;
+}
+
+int CollCharISMethods::fillTable(Session *session, TableList *tables)
+{
+  CHARSET_INFO **cs;
+  Table *table= tables->table;
+  const CHARSET_INFO * const scs= system_charset_info;
+  for (cs= all_charsets ; cs < all_charsets+255 ; cs++ )
+  {
+    CHARSET_INFO **cl;
+    const CHARSET_INFO *tmp_cs= cs[0];
+    if (! tmp_cs || ! (tmp_cs->state & MY_CS_AVAILABLE) ||
+        ! (tmp_cs->state & MY_CS_PRIMARY))
+      continue;
+    for (cl= all_charsets; cl < all_charsets+255 ;cl ++)
+    {
+      const CHARSET_INFO *tmp_cl= cl[0];
+      if (! tmp_cl || ! (tmp_cl->state & MY_CS_AVAILABLE) ||
+          ! my_charset_same(tmp_cs,tmp_cl))
+        continue;
+      table->restoreRecordAsDefault();
+      table->field[0]->store(tmp_cl->name, strlen(tmp_cl->name), scs);
+      table->field[1]->store(tmp_cl->csname , strlen(tmp_cl->csname), scs);
+      if (schema_table_store_record(session, table))
+        return 1;
+    }
+  }
+  return 0;
+}
+

=== added file 'plugin/info_schema/collation_char_set.h'
--- plugin/info_schema/collation_char_set.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/collation_char_set.h	2009-11-16 03:47:07 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_COLLATION_CHAR_SET_H
+#define PLUGIN_INFO_SCHEMA_COLLATION_CHAR_SET_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class CollCharISMethods
+ * @brief
+ *   Class which implements any methods that the collation char set
+ *   I_S table needs besides the default methods
+ */
+class CollCharISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+};
+
+class CollationCharSetIS
+{
+  /**
+   * Create the various columns for the COLLATION_CHAR_SET I_S table and add them
+   * to the std::vector of columns for the COLLATION_CHAR_SET table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_COLLATION_CHAR_SET_H */

=== added file 'plugin/info_schema/columns.cc'
--- plugin/info_schema/columns.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/columns.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,311 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Character Set I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "columns.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the columns I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the columns I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * columns I_S table.
+ */
+static plugin::InfoSchemaTable *cols_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *ColumnsIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  /*
+   * Create each column for the COLUMNS table.
+   */
+  columns->push_back(new plugin::ColumnInfo("TABLE_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Field",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("ORDINAL_POSITION",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            MY_I_S_UNSIGNED,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_DEFAULT",
+                                            MAX_FIELD_VARCHARLENGTH,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Default",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("IS_NULLABLE",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Null",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("DATA_TYPE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("CHARACTER_MAXIMUM_LENGTH",
+                                           MY_INT64_NUM_DECIMAL_DIGITS,
+                                           DRIZZLE_TYPE_LONGLONG,
+                                           0,
+                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                           "",
+                                           OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("CHARACTER_OCTET_LENGTH",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("NUMERIC_PRECISION",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("NUMERIC_SCALE",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("CHARACTER_SET_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLLATION_NAME",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Collation",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_TYPE",
+                                            65535,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Type",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_KEY",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Key",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("EXTRA",
+                                            27,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Extra",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("PRIVILEGES",
+                                            80,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Privileges",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_COMMENT",
+                                            COLUMN_COMMENT_MAXLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Comment",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("STORAGE",
+                                            8,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Storage",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("FORMAT",
+                                            8,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Format",
+                                            OPEN_FRM_ONLY));
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *ColumnsIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new ColumnsISMethods();
+  }
+
+  if (cols_table == NULL)
+  {
+    cols_table= new plugin::InfoSchemaTable("COLUMNS",
+                                            *columns,
+                                            1, 2, false, true,
+                                            OPTIMIZE_I_S_TABLE,
+                                            methods);
+  }
+
+  return cols_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void ColumnsIS::cleanup()
+{
+  clearColumns(*columns);
+  delete cols_table;
+  delete methods;
+  delete columns;
+}
+
+int ColumnsISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
+  const
+{
+  int fields_arr[]= {3, 14, 13, 6, 15, 5, 16, 17, 18, -1};
+  int *field_num= fields_arr;
+  const drizzled::plugin::InfoSchemaTable::Columns tab_columns= schema_table->getColumns();
+  const drizzled::plugin::ColumnInfo *column;
+  Name_resolution_context *context= &session->lex->select_lex.context;
+
+  for (; *field_num >= 0; field_num++)
+  {
+    column= tab_columns[*field_num];
+    if (! session->lex->verbose && (*field_num == 13 ||
+                                    *field_num == 17 ||
+                                    *field_num == 18))
+    {
+      continue;
+    }
+    Item_field *field= new Item_field(context,
+                                      NULL, NULL, column->getName().c_str());
+    if (field)
+    {
+      field->set_name(column->getOldName().c_str(),
+                      column->getOldName().length(),
+                      system_charset_info);
+      if (session->add_item_to_list(field))
+      {
+        return 1;
+      }
+    }
+  }
+  return 0;
+}

=== added file 'plugin/info_schema/columns.h'
--- plugin/info_schema/columns.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/columns.h	2009-11-16 04:22:39 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_COLUMNS_H
+#define PLUGIN_INFO_SCHEMA_COLUMNS_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class ColumnsISMethods
+ * @brief
+ *   Class which implements any methods that the COLUMNS
+ *   I_S table needs besides the default methods
+ */
+class ColumnsISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int oldFormat(Session *session,
+                        drizzled::plugin::InfoSchemaTable *schema_table) const;
+};
+
+class ColumnsIS
+{
+  /**
+   * Create the various columns for the COLUMNS I_S table and add them
+   * to the std::vector of columns for the COLUMNS table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_COLUMNS_H */

=== renamed file 'plugin/info_schema/info_schema_methods.cc' => 'plugin/info_schema/helper_methods.cc'
--- plugin/info_schema/info_schema_methods.cc	2009-11-11 19:42:27 +0000
+++ plugin/info_schema/helper_methods.cc	2009-11-18 21:40:45 +0000
@@ -20,18 +20,17 @@
 
 /**
  * @file 
- *   Implementation of the methods for I_S tables.
+ *   Implementation of helper methods for I_S tables.
  */
 
-#include <drizzled/server_includes.h>
-#include <drizzled/session.h>
-#include <drizzled/show.h>
-#include <drizzled/tztime.h>
-#include <drizzled/sql_base.h>
-#include <drizzled/plugin/client.h>
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/sql_base.h"
+#include "drizzled/plugin/client.h"
 #include "drizzled/join_table.h"
 
-#include "info_schema_methods.h"
+#include "helper_methods.h"
 
 #include <vector>
 #include <string>
@@ -44,12 +43,13 @@
     *buf= my_toupper(system_charset_info, *buf);
 }
 
-static bool show_status_array(Session *session, const char *wild,
-                              SHOW_VAR *variables,
-                              enum enum_var_type value_type,
-                              struct system_status_var *status_var,
-                              const char *prefix, Table *table,
-                              bool ucase_names)
+bool show_status_array(Session *session, 
+                       const char *wild,
+                       SHOW_VAR *variables,
+                       enum enum_var_type value_type,
+                       struct system_status_var *status_var,
+                       const char *prefix, Table *table,
+                       bool ucase_names)
 {
   MY_ALIGNED_BYTE_ARRAY(buff_data, SHOW_VAR_FUNC_BUFF_SIZE, int64_t);
   char * const buff= (char *) &buff_data;
@@ -203,169 +203,15 @@
   return false;
 }
 
-int CharSetISMethods::fillTable(Session *session, TableList *tables)
-{
-  CHARSET_INFO **cs;
-  const char *wild= session->lex->wild ? session->lex->wild->ptr() : NULL;
-  Table *table= tables->table;
-  const CHARSET_INFO * const scs= system_charset_info;
-
-  for (cs= all_charsets ; cs < all_charsets+255 ; cs++)
-  {
-    const CHARSET_INFO * const tmp_cs= cs[0];
-    if (tmp_cs && (tmp_cs->state & MY_CS_PRIMARY) &&
-        (tmp_cs->state & MY_CS_AVAILABLE) &&
-        !(tmp_cs->state & MY_CS_HIDDEN) &&
-        !(wild && wild[0] &&
-          wild_case_compare(scs, tmp_cs->csname,wild)))
-    {
-      const char *comment;
-      table->restoreRecordAsDefault();
-      table->field[0]->store(tmp_cs->csname, strlen(tmp_cs->csname), scs);
-      table->field[1]->store(tmp_cs->name, strlen(tmp_cs->name), scs);
-      comment= tmp_cs->comment ? tmp_cs->comment : "";
-      table->field[2]->store(comment, strlen(comment), scs);
-      table->field[3]->store((int64_t) tmp_cs->mbmaxlen, true);
-      if (schema_table_store_record(session, table))
-        return 1;
-    }
-  }
-  return 0;
-}
-
-int CharSetISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
-  const
-{
-  int fields_arr[]= {0, 2, 1, 3, -1};
-  int *field_num= fields_arr;
-  const drizzled::plugin::InfoSchemaTable::Columns columns= schema_table->getColumns();
-  const drizzled::plugin::ColumnInfo *column;
-  Name_resolution_context *context= &session->lex->select_lex.context;
-
-  for (; *field_num >= 0; field_num++)
-  {
-    column= columns[*field_num];
-    Item_field *field= new Item_field(context,
-                                      NULL, NULL, column->getName().c_str());
-    if (field)
-    {
-      field->set_name(column->getOldName().c_str(),
-                      column->getOldName().length(),
-                      system_charset_info);
-      if (session->add_item_to_list(field))
-        return 1;
-    }
-  }
-  return 0;
-}
-
-int CollationISMethods::fillTable(Session *session, TableList *tables)
-{
-  CHARSET_INFO **cs;
-  const char *wild= session->lex->wild ? session->lex->wild->ptr() : NULL;
-  Table *table= tables->table;
-  const CHARSET_INFO * const scs= system_charset_info;
-  for (cs= all_charsets ; cs < all_charsets+255 ; cs++ )
-  {
-    CHARSET_INFO **cl;
-    const CHARSET_INFO *tmp_cs= cs[0];
-    if (! tmp_cs || ! (tmp_cs->state & MY_CS_AVAILABLE) ||
-         (tmp_cs->state & MY_CS_HIDDEN) ||
-        !(tmp_cs->state & MY_CS_PRIMARY))
-      continue;
-    for (cl= all_charsets; cl < all_charsets+255 ;cl ++)
-    {
-      const CHARSET_INFO *tmp_cl= cl[0];
-      if (! tmp_cl || ! (tmp_cl->state & MY_CS_AVAILABLE) ||
-          !my_charset_same(tmp_cs, tmp_cl))
-        continue;
-      if (! (wild && wild[0] &&
-          wild_case_compare(scs, tmp_cl->name,wild)))
-      {
-        const char *tmp_buff;
-        table->restoreRecordAsDefault();
-        table->field[0]->store(tmp_cl->name, strlen(tmp_cl->name), scs);
-        table->field[1]->store(tmp_cl->csname , strlen(tmp_cl->csname), scs);
-        table->field[2]->store((int64_t) tmp_cl->number, true);
-        tmp_buff= (tmp_cl->state & MY_CS_PRIMARY) ? "Yes" : "";
-        table->field[3]->store(tmp_buff, strlen(tmp_buff), scs);
-        tmp_buff= (tmp_cl->state & MY_CS_COMPILED)? "Yes" : "";
-        table->field[4]->store(tmp_buff, strlen(tmp_buff), scs);
-        table->field[5]->store((int64_t) tmp_cl->strxfrm_multiply, true);
-        if (schema_table_store_record(session, table))
-          return 1;
-      }
-    }
-  }
-  return 0;
-}
-
-int CollCharISMethods::fillTable(Session *session, TableList *tables)
-{
-  CHARSET_INFO **cs;
-  Table *table= tables->table;
-  const CHARSET_INFO * const scs= system_charset_info;
-  for (cs= all_charsets ; cs < all_charsets+255 ; cs++ )
-  {
-    CHARSET_INFO **cl;
-    const CHARSET_INFO *tmp_cs= cs[0];
-    if (! tmp_cs || ! (tmp_cs->state & MY_CS_AVAILABLE) ||
-        ! (tmp_cs->state & MY_CS_PRIMARY))
-      continue;
-    for (cl= all_charsets; cl < all_charsets+255 ;cl ++)
-    {
-      const CHARSET_INFO *tmp_cl= cl[0];
-      if (! tmp_cl || ! (tmp_cl->state & MY_CS_AVAILABLE) ||
-          ! my_charset_same(tmp_cs,tmp_cl))
-        continue;
-      table->restoreRecordAsDefault();
-      table->field[0]->store(tmp_cl->name, strlen(tmp_cl->name), scs);
-      table->field[1]->store(tmp_cl->csname , strlen(tmp_cl->csname), scs);
-      if (schema_table_store_record(session, table))
-        return 1;
-    }
-  }
-  return 0;
-}
-
-int ColumnsISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
-  const
-{
-  int fields_arr[]= {3, 14, 13, 6, 15, 5, 16, 17, 18, -1};
-  int *field_num= fields_arr;
-  const drizzled::plugin::InfoSchemaTable::Columns columns= schema_table->getColumns();
-  const drizzled::plugin::ColumnInfo *column;
-  Name_resolution_context *context= &session->lex->select_lex.context;
-
-  for (; *field_num >= 0; field_num++)
-  {
-    column= columns[*field_num];
-    if (! session->lex->verbose && (*field_num == 13 ||
-                                    *field_num == 17 ||
-                                    *field_num == 18))
-    {
-      continue;
-    }
-    Item_field *field= new Item_field(context,
-                                      NULL, NULL, column->getName().c_str());
-    if (field)
-    {
-      field->set_name(column->getOldName().c_str(),
-                      column->getOldName().length(),
-                      system_charset_info);
-      if (session->add_item_to_list(field))
-      {
-        return 1;
-      }
-    }
-  }
-  return 0;
-}
-
-static void store_key_column_usage(Table *table, LEX_STRING *db_name,
-                                   LEX_STRING *table_name, const char *key_name,
-                                   uint32_t key_len, const char *con_type, uint32_t con_len,
-                                   int64_t idx)
+
+void store_key_column_usage(Table *table, 
+                            LEX_STRING *db_name,
+                            LEX_STRING *table_name, 
+                            const char *key_name,
+                            uint32_t key_len, 
+                            const char *con_type, 
+                            uint32_t con_len,
+                            int64_t idx)
 {
   const CHARSET_INFO * const cs= system_charset_info;
   table->field[1]->store(db_name->str, db_name->length, cs);
@@ -376,999 +222,22 @@
   table->field[7]->store((int64_t) idx, true);
 }
 
-int KeyColUsageISMethods::processTable(Session *session,
-                                              TableList *tables,
-                                              Table *table, bool res,
-                                              LEX_STRING *db_name,
-                                              LEX_STRING *table_name) const
-{
-  if (res)
-  {
-    if (session->is_error())
-      push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
-                   session->main_da.sql_errno(), session->main_da.message());
-    session->clear_error();
-    return(0);
-  }
-  else
-  {
-    List<FOREIGN_KEY_INFO> f_key_list;
-    Table *show_table= tables->table;
-    KEY *key_info=show_table->key_info;
-    uint32_t primary_key= show_table->s->primary_key;
-    show_table->cursor->info(HA_STATUS_VARIABLE |
-                             HA_STATUS_NO_LOCK |
-                             HA_STATUS_TIME);
-    for (uint32_t i=0 ; i < show_table->s->keys ; i++, key_info++)
-    {
-      if (i != primary_key && !(key_info->flags & HA_NOSAME))
-      {
-        continue;
-      }
-      uint32_t f_idx= 0;
-      KEY_PART_INFO *key_part= key_info->key_part;
-      for (uint32_t j=0 ; j < key_info->key_parts ; j++,key_part++)
-      {
-        if (key_part->field)
-        {
-          f_idx++;
-          table->restoreRecordAsDefault();
-          store_key_column_usage(table, db_name, table_name,
-                                 key_info->name,
-                                 strlen(key_info->name),
-                                 key_part->field->field_name,
-                                 strlen(key_part->field->field_name),
-                                 (int64_t) f_idx);
-          if (schema_table_store_record(session, table))
-          {
-            return (1);
-          }
-        }
-      }
-    }
-
-    show_table->cursor->get_foreign_key_list(session, &f_key_list);
-    FOREIGN_KEY_INFO *f_key_info;
-    List_iterator_fast<FOREIGN_KEY_INFO> fkey_it(f_key_list);
-    while ((f_key_info= fkey_it++))
-    {
-      LEX_STRING *f_info;
-      LEX_STRING *r_info;
-      List_iterator_fast<LEX_STRING> it(f_key_info->foreign_fields),
-        it1(f_key_info->referenced_fields);
-      uint32_t f_idx= 0;
-      while ((f_info= it++))
-      {
-        r_info= it1++;
-        f_idx++;
-        table->restoreRecordAsDefault();
-        store_key_column_usage(table, db_name, table_name,
-                               f_key_info->forein_id->str,
-                               f_key_info->forein_id->length,
-                               f_info->str, f_info->length,
-                               (int64_t) f_idx);
-        table->field[8]->store((int64_t) f_idx, true);
-        table->field[8]->set_notnull();
-        table->field[9]->store(f_key_info->referenced_db->str,
-                               f_key_info->referenced_db->length,
-                               system_charset_info);
-        table->field[9]->set_notnull();
-        table->field[10]->store(f_key_info->referenced_table->str,
-                                f_key_info->referenced_table->length,
-                                system_charset_info);
-        table->field[10]->set_notnull();
-        table->field[11]->store(r_info->str, r_info->length,
-                                system_charset_info);
-        table->field[11]->set_notnull();
-        if (schema_table_store_record(session, table))
-        {
-          return (1);
-        }
-      }
-    }
-  }
-  return (res);
-}
-
-inline bool open_list_store(Table *table, open_table_list_st& open_list);
-inline bool open_list_store(Table *table, open_table_list_st& open_list)
-{
-  table->restoreRecordAsDefault();
-  table->field[0]->store(open_list.db.c_str(), open_list.db.length(), system_charset_info);
-  table->field[1]->store(open_list.table.c_str(), open_list.table.length(), system_charset_info);
-  table->field[2]->store((int64_t) open_list.in_use, true);
-  table->field[3]->store((int64_t) open_list.locked, true);
-  if (schema_table_store_record(table->in_use, table))
-    return true;
-
-  return false;
-}
-
-int OpenTablesISMethods::fillTable(Session *session, TableList *tables)
-{
-  const char *wild= session->lex->wild ? session->lex->wild->ptr() : NULL;
-
-  if ((list_open_tables(session->lex->select_lex.db, wild, open_list_store, tables->table) == true) && session->is_fatal_error)
-    return 1;
-
-  return 0;
-}
-
-class ShowModules : public unary_function<drizzled::plugin::Module *, bool>
-{
-  Session *session;
-  Table *table;
-public:
-  ShowModules(Session *session_arg, Table *table_arg)
-    : session(session_arg), table(table_arg) {}
-
-  result_type operator() (argument_type module)
-  {
-    const drizzled::plugin::Manifest &manifest= module->getManifest();
-    const CHARSET_INFO * const cs= system_charset_info;
-
-    table->restoreRecordAsDefault();
-
-    table->field[0]->store(module->getName().c_str(),
-                           module->getName().size(), cs);
-
-    if (manifest.version)
-    {
-      table->field[1]->store(manifest.version, strlen(manifest.version), cs);
-      table->field[1]->set_notnull();
-    }
-    else
-      table->field[1]->set_null();
-
-    if (manifest.author)
-    {
-      table->field[2]->store(manifest.author, strlen(manifest.author), cs);
-      table->field[2]->set_notnull();
-    }
-    else
-    {
-      table->field[2]->set_null();
-    }
-
-    if (module->plugin_dl == NULL)
-    {
-      table->field[3]->store(STRING_WITH_LEN("YES"),cs);
-      table->field[4]->set_null();
-    }
-    else
-    {
-      table->field[3]->store(STRING_WITH_LEN("NO"),cs);
-      table->field[4]->store(module->plugin_dl->dl.str,
-                             module->plugin_dl->dl.length, cs);
-    }
-
-    if (manifest.descr)
-    {
-      table->field[5]->store(manifest.descr, strlen(manifest.descr), cs);
-      table->field[5]->set_notnull();
-    }
-    else
-    {
-      table->field[5]->set_null();
-    }
-
-    switch (manifest.license) {
-    case PLUGIN_LICENSE_GPL:
-      table->field[6]->store(drizzled::plugin::LICENSE_GPL_STRING.c_str(),
-                             drizzled::plugin::LICENSE_GPL_STRING.size(), cs);
-      break;
-    case PLUGIN_LICENSE_BSD:
-      table->field[6]->store(drizzled::plugin::LICENSE_BSD_STRING.c_str(),
-                             drizzled::plugin::LICENSE_BSD_STRING.size(), cs);
-      break;
-    case PLUGIN_LICENSE_LGPL:
-      table->field[6]->store(drizzled::plugin::LICENSE_LGPL_STRING.c_str(),
-                             drizzled::plugin::LICENSE_LGPL_STRING.size(), cs);
-      break;
-    default:
-      table->field[6]->store(drizzled::plugin::LICENSE_PROPRIETARY_STRING.c_str(),
-                             drizzled::plugin::LICENSE_PROPRIETARY_STRING.size(),
-                             cs);
-      break;
-    }
-    table->field[6]->set_notnull();
-
-    return schema_table_store_record(session, table);
-  }
-};
-
-int ModulesISMethods::fillTable(Session *session, TableList *tables)
-{
-  Table *table= tables->table;
-
-  drizzled::plugin::Registry &registry= drizzled::plugin::Registry::singleton();
-  vector<drizzled::plugin::Module *> modules= registry.getList(true);
-  vector<drizzled::plugin::Module *>::iterator iter=
-    find_if(modules.begin(), modules.end(), ShowModules(session, table));
-  if (iter != modules.end())
-  {
-    return 1;
-  }
-  return 0;
-}
-
-class ShowPlugins
- : public unary_function<pair<const string, const drizzled::plugin::Plugin *>, bool>
-{
-  Session *session;
-  Table *table;
-public:
-  ShowPlugins(Session *session_arg, Table *table_arg)
-    : session(session_arg), table(table_arg) {}
-
-  result_type operator() (argument_type plugin)
-  {
-    const CHARSET_INFO * const cs= system_charset_info;
-
-    table->restoreRecordAsDefault();
-
-    table->field[0]->store(plugin.first.c_str(),
-                           plugin.first.size(), cs);
-
-    table->field[1]->store(plugin.second->getTypeName().c_str(),
-                           plugin.second->getTypeName().size(), cs);
-
-    if (plugin.second->isActive())
-    {
-      table->field[2]->store(STRING_WITH_LEN("YES"),cs);
-    }
-    else
-    {
-      table->field[2]->store(STRING_WITH_LEN("NO"), cs);
-    }
-
-    table->field[3]->store(plugin.second->getModuleName().c_str(),
-                           plugin.second->getModuleName().size(), cs);
-
-    return schema_table_store_record(session, table);
-  }
-};
-
-int PluginsISMethods::fillTable(Session *session, TableList *tables)
-{
-  Table *table= tables->table;
-
-  drizzled::plugin::Registry &registry= drizzled::plugin::Registry::singleton();
-  const map<string, const drizzled::plugin::Plugin *> &plugin_map=
-    registry.getPluginsMap();
-  map<string, const drizzled::plugin::Plugin *>::const_iterator iter=
-    find_if(plugin_map.begin(), plugin_map.end(), ShowPlugins(session, table));
-  if (iter != plugin_map.end())
-  {
-    return 1;
-  }
-  return (0);
-}
-
-int ProcessListISMethods::fillTable(Session* session, TableList* tables)
-{
-  Table *table= tables->table;
-  const CHARSET_INFO * const cs= system_charset_info;
-  time_t now= time(NULL);
-  size_t length;
-
-  if (now == (time_t)-1)
-    return 1;
-
-  pthread_mutex_lock(&LOCK_thread_count);
-
-  if (! session->killed)
-  {
-    Session* tmp;
-
-    for (vector<Session*>::iterator it= session_list.begin(); it != session_list.end(); ++it)
-    {
-      tmp= *it;
-      Security_context *tmp_sctx= &tmp->security_ctx;
-      struct st_my_thread_var *mysys_var;
-      const char *val;
-
-      if (! tmp->client->isConnected())
-        continue;
-
-      table->restoreRecordAsDefault();
-      /* ID */
-      table->field[0]->store((int64_t) tmp->thread_id, true);
-      /* USER */
-      val= tmp_sctx->user.c_str() ? tmp_sctx->user.c_str() : "unauthenticated user";
-      table->field[1]->store(val, strlen(val), cs);
-      /* HOST */
-      table->field[2]->store(tmp_sctx->ip.c_str(), strlen(tmp_sctx->ip.c_str()), cs);
-      /* DB */
-      if (tmp->db)
-      {
-        table->field[3]->store(tmp->db, strlen(tmp->db), cs);
-        table->field[3]->set_notnull();
-      }
-
-      if ((mysys_var= tmp->mysys_var))
-        pthread_mutex_lock(&mysys_var->mutex);
-      /* COMMAND */
-      if ((val= (char *) (tmp->killed == Session::KILL_CONNECTION? "Killed" : 0)))
-        table->field[4]->store(val, strlen(val), cs);
-      else
-        table->field[4]->store(command_name[tmp->command].str,
-                               command_name[tmp->command].length, cs);
-      /* DRIZZLE_TIME */
-      table->field[5]->store((uint32_t)(tmp->start_time ?
-                                      now - tmp->start_time : 0), true);
-      /* STATE */
-      val= (char*) (tmp->client->isWriting() ?
-                    "Writing to net" :
-                    tmp->client->isReading() ?
-                    (tmp->command == COM_SLEEP ?
-                     NULL : "Reading from net") :
-                    tmp->get_proc_info() ? tmp->get_proc_info() :
-                    tmp->mysys_var &&
-                    tmp->mysys_var->current_cond ?
-                    "Waiting on cond" : NULL);
-      if (val)
-      {
-        table->field[6]->store(val, strlen(val), cs);
-        table->field[6]->set_notnull();
-      }
-
-      if (mysys_var)
-        pthread_mutex_unlock(&mysys_var->mutex);
-
-      length= strlen(tmp->process_list_info);
-
-      if (length)
-      {
-        table->field[7]->store(tmp->process_list_info, length, cs);
-        table->field[7]->set_notnull();
-      }
-
-      if (schema_table_store_record(session, table))
-      {
-        pthread_mutex_unlock(&LOCK_thread_count);
-        return(1);
-      }
-    }
-  }
-
-  pthread_mutex_unlock(&LOCK_thread_count);
-  return(0);
-}
-
-int
-RefConstraintsISMethods::processTable(Session *session, TableList *tables,
-                                      Table *table, bool res,
-                                      LEX_STRING *db_name, LEX_STRING *table_name)
-  const
-{
-  const CHARSET_INFO * const cs= system_charset_info;
-
-  if (res)
-  {
-    if (session->is_error())
-      push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
-                   session->main_da.sql_errno(), session->main_da.message());
-    session->clear_error();
-    return(0);
-  }
-
-  {
-    List<FOREIGN_KEY_INFO> f_key_list;
-    Table *show_table= tables->table;
-    show_table->cursor->info(HA_STATUS_VARIABLE |
-                           HA_STATUS_NO_LOCK |
-                           HA_STATUS_TIME);
-
-    show_table->cursor->get_foreign_key_list(session, &f_key_list);
-    FOREIGN_KEY_INFO *f_key_info;
-    List_iterator_fast<FOREIGN_KEY_INFO> it(f_key_list);
-    while ((f_key_info= it++))
-    {
-      table->restoreRecordAsDefault();
-      table->field[1]->store(db_name->str, db_name->length, cs);
-      table->field[9]->store(table_name->str, table_name->length, cs);
-      table->field[2]->store(f_key_info->forein_id->str,
-                             f_key_info->forein_id->length, cs);
-      table->field[4]->store(f_key_info->referenced_db->str,
-                             f_key_info->referenced_db->length, cs);
-      table->field[10]->store(f_key_info->referenced_table->str,
-                             f_key_info->referenced_table->length, cs);
-      if (f_key_info->referenced_key_name)
-      {
-        table->field[5]->store(f_key_info->referenced_key_name->str,
-                               f_key_info->referenced_key_name->length, cs);
-        table->field[5]->set_notnull();
-      }
-      else
-      {
-        table->field[5]->set_null();
-      }
-      table->field[6]->store(STRING_WITH_LEN("NONE"), cs);
-      table->field[7]->store(f_key_info->update_method->str,
-                             f_key_info->update_method->length, cs);
-      table->field[8]->store(f_key_info->delete_method->str,
-                             f_key_info->delete_method->length, cs);
-      if (schema_table_store_record(session, table))
-      {
-        return (1);
-      }
-    }
-  }
-  return (0);
-}
-
-static bool store_schema_schemata(Session* session, Table *table, LEX_STRING *db_name,
-                                  const CHARSET_INFO * const cs)
-{
-  table->restoreRecordAsDefault();
-  table->field[1]->store(db_name->str, db_name->length, system_charset_info);
-  table->field[2]->store(cs->csname, strlen(cs->csname), system_charset_info);
-  table->field[3]->store(cs->name, strlen(cs->name), system_charset_info);
-  return schema_table_store_record(session, table);
-}
-
-int SchemataISMethods::fillTable(Session *session, TableList *tables)
-{
-  /*
-    TODO: fill_schema_shemata() is called when new client is connected.
-    Returning error status in this case leads to client hangup.
-  */
-
-  LOOKUP_FIELD_VALUES lookup_field_vals;
-  vector<LEX_STRING*> db_names;
-  bool with_i_schema;
-  Table *table= tables->table;
-  /* the WHERE condition */
-  COND *cond= table->reginfo.join_tab->select_cond;
-
-  if (get_lookup_field_values(session, cond, tables, &lookup_field_vals))
-    return(0);
-  if (make_db_list(session, db_names, &lookup_field_vals,
-                   &with_i_schema))
-    return(1);
-
-  /*
-    If we have lookup db value we should check that the database exists
-  */
-  if(lookup_field_vals.db_value.str && !lookup_field_vals.wild_db_value &&
-     !with_i_schema)
-  {
-    char path[FN_REFLEN+16];
-    uint32_t path_len;
-    struct stat stat_info;
-    if (!lookup_field_vals.db_value.str[0])
-      return(0);
-    path_len= build_table_filename(path, sizeof(path),
-                                   lookup_field_vals.db_value.str, "", false);
-    path[path_len-1]= 0;
-    if (stat(path,&stat_info))
-      return(0);
-  }
-
-  vector<LEX_STRING*>::iterator db_name= db_names.begin();
-  while (db_name != db_names.end())
-  {
-    if (with_i_schema)       // information schema name is always first in list
-    {
-      if (store_schema_schemata(session, table, *db_name, system_charset_info))
-        return(1);
-
-      with_i_schema= 0;
-    }
-    else
-    {
-      const CHARSET_INFO *cs= get_default_db_collation((*db_name)->str);
-
-      if (store_schema_schemata(session, table, *db_name, cs))
-        return(1);
-    }
-
-    ++db_name;
-  }
-  return(0);
-}
-
-int SchemataISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
-  const
-{
-  char tmp[128];
-  LEX *lex= session->lex;
-  Select_Lex *sel= lex->current_select;
-  Name_resolution_context *context= &sel->context;
-  const drizzled::plugin::InfoSchemaTable::Columns columns= schema_table->getColumns();
-
-  if (!sel->item_list.elements)
-  {
-    const drizzled::plugin::ColumnInfo *column= columns[1];
-    String buffer(tmp,sizeof(tmp), system_charset_info);
-    Item_field *field= new Item_field(context,
-                                      NULL, NULL, column->getName().c_str());
-    if (!field || session->add_item_to_list(field))
-      return 1;
-    buffer.length(0);
-    buffer.append(column->getOldName().c_str());
-    if (lex->wild && lex->wild->ptr())
-    {
-      buffer.append(STRING_WITH_LEN(" ("));
-      buffer.append(lex->wild->ptr());
-      buffer.append(')');
-    }
-    field->set_name(buffer.ptr(), buffer.length(), system_charset_info);
-  }
-  return 0;
-}
-
-int StatsISMethods::processTable(Session *session, TableList *tables,
-                                  Table *table, bool res,
-                                  LEX_STRING *db_name,
-                                  LEX_STRING *table_name) const
-{
-  const CHARSET_INFO * const cs= system_charset_info;
-  if (res)
-  {
-    if (session->lex->sql_command != SQLCOM_SHOW_KEYS)
-    {
-      /*
-        I.e. we are in SELECT FROM INFORMATION_SCHEMA.STATISTICS
-        rather than in SHOW KEYS
-      */
-      if (session->is_error())
-      {
-        push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
-                     session->main_da.sql_errno(), session->main_da.message());
-      }
-      session->clear_error();
-      res= 0;
-    }
-    return (res);
-  }
-  else
-  {
-    Table *show_table= tables->table;
-    KEY *key_info=show_table->s->key_info;
-    if (show_table->cursor)
-    {
-      show_table->cursor->info(HA_STATUS_VARIABLE |
-                               HA_STATUS_NO_LOCK |
-                               HA_STATUS_TIME);
-    }
-    for (uint32_t i=0 ; i < show_table->s->keys ; i++,key_info++)
-    {
-      KEY_PART_INFO *key_part= key_info->key_part;
-      const char *str;
-      for (uint32_t j=0 ; j < key_info->key_parts ; j++,key_part++)
-      {
-        table->restoreRecordAsDefault();
-        table->field[1]->store(db_name->str, db_name->length, cs);
-        table->field[2]->store(table_name->str, table_name->length, cs);
-        table->field[3]->store((int64_t) ((key_info->flags &
-                                            HA_NOSAME) ? 0 : 1), true);
-        table->field[4]->store(db_name->str, db_name->length, cs);
-        table->field[5]->store(key_info->name, strlen(key_info->name), cs);
-        table->field[6]->store((int64_t) (j+1), true);
-        str=(key_part->field ? key_part->field->field_name :
-             "?unknown field?");
-        table->field[7]->store(str, strlen(str), cs);
-        if (show_table->cursor)
-        {
-          if (show_table->cursor->index_flags(i, j, 0) & HA_READ_ORDER)
-          {
-            table->field[8]->store(((key_part->key_part_flag &
-                                     HA_REVERSE_SORT) ?
-                                    "D" : "A"), 1, cs);
-            table->field[8]->set_notnull();
-          }
-          KEY *key=show_table->key_info+i;
-          if (key->rec_per_key[j])
-          {
-            ha_rows records=(show_table->cursor->stats.records /
-                             key->rec_per_key[j]);
-            table->field[9]->store((int64_t) records, true);
-            table->field[9]->set_notnull();
-          }
-          str= show_table->cursor->index_type(i);
-          table->field[13]->store(str, strlen(str), cs);
-        }
-        if ((key_part->field &&
-             key_part->length !=
-             show_table->s->field[key_part->fieldnr-1]->key_length()))
-        {
-          table->field[10]->store((int64_t) key_part->length /
-                                  key_part->field->charset()->mbmaxlen, true);
-          table->field[10]->set_notnull();
-        }
-        uint32_t flags= key_part->field ? key_part->field->flags : 0;
-        const char *pos=(char*) ((flags & NOT_NULL_FLAG) ? "" : "YES");
-        table->field[12]->store(pos, strlen(pos), cs);
-        if (!show_table->s->keys_in_use.test(i))
-        {
-          table->field[14]->store(STRING_WITH_LEN("disabled"), cs);
-        }
-        else
-        {
-          table->field[14]->store("", 0, cs);
-        }
-        table->field[14]->set_notnull();
-        assert(test(key_info->flags & HA_USES_COMMENT) ==
-                   (key_info->comment.length > 0));
-        if (key_info->flags & HA_USES_COMMENT)
-        {
-          table->field[15]->store(key_info->comment.str,
-                                  key_info->comment.length, cs);
-        }
-        if (schema_table_store_record(session, table))
-        {
-          return (1);
-        }
-      }
-    }
-  }
-  return(res);
-}
-
-int StatusISMethods::fillTable(Session *session, TableList *tables)
-{
-  LEX *lex= session->lex;
-  const char *wild= lex->wild ? lex->wild->ptr() : NULL;
-  int res= 0;
-  STATUS_VAR *tmp1, tmp;
-  const string schema_table_name= tables->schema_table->getTableName();
-  enum enum_var_type option_type;
-  bool upper_case_names= (schema_table_name.compare("STATUS") != 0);
-
-  if (schema_table_name.compare("STATUS") == 0)
-  {
-    option_type= lex->option_type;
-    if (option_type == OPT_GLOBAL)
-      tmp1= &tmp;
-    else
-      tmp1= session->initial_status_var;
-  }
-  else if (schema_table_name.compare("GLOBAL_STATUS") == 0)
-  {
-    option_type= OPT_GLOBAL;
-    tmp1= &tmp;
-  }
-  else
-  {
-    option_type= OPT_SESSION;
-    tmp1= &session->status_var;
-  }
-
-  pthread_mutex_lock(&LOCK_status);
-  if (option_type == OPT_GLOBAL)
-    calc_sum_of_all_status(&tmp);
-  res= show_status_array(session, wild,
-                         getFrontOfStatusVars(),
-                         option_type, tmp1, "", tables->table,
-                         upper_case_names);
-  pthread_mutex_unlock(&LOCK_status);
-  return(res);
-}
-
-static bool store_constraints(Session *session, Table *table, LEX_STRING *db_name,
-                              LEX_STRING *table_name, const char *key_name,
-                              uint32_t key_len, const char *con_type, uint32_t con_len)
-{
-  const CHARSET_INFO * const cs= system_charset_info;
-  table->restoreRecordAsDefault();
-  table->field[1]->store(db_name->str, db_name->length, cs);
-  table->field[2]->store(key_name, key_len, cs);
-  table->field[3]->store(db_name->str, db_name->length, cs);
-  table->field[4]->store(table_name->str, table_name->length, cs);
-  table->field[5]->store(con_type, con_len, cs);
-  return schema_table_store_record(session, table);
-}
-
-int TabConstraintsISMethods::processTable(Session *session, TableList *tables,
-                                          Table *table, bool res,
-                                          LEX_STRING *db_name,
-                                          LEX_STRING *table_name) const
-{
-  if (res)
-  {
-    if (session->is_error())
-    {
-      push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
-                   session->main_da.sql_errno(), session->main_da.message());
-    }
-    session->clear_error();
-    return (0);
-  }
-  else
-  {
-    List<FOREIGN_KEY_INFO> f_key_list;
-    Table *show_table= tables->table;
-    KEY *key_info=show_table->key_info;
-    uint32_t primary_key= show_table->s->primary_key;
-    show_table->cursor->info(HA_STATUS_VARIABLE |
-                             HA_STATUS_NO_LOCK |
-                             HA_STATUS_TIME);
-    for (uint32_t i=0 ; i < show_table->s->keys ; i++, key_info++)
-    {
-      if (i != primary_key && !(key_info->flags & HA_NOSAME))
-      {
-        continue;
-      }
-
-      if (i == primary_key && is_primary_key(key_info))
-      {
-        if (store_constraints(session, table, db_name, table_name, key_info->name,
-                              strlen(key_info->name),
-                              STRING_WITH_LEN("PRIMARY KEY")))
-        {
-          return (1);
-        }
-      }
-      else if (key_info->flags & HA_NOSAME)
-      {
-        if (store_constraints(session, table, db_name, table_name, key_info->name,
-                              strlen(key_info->name),
-                              STRING_WITH_LEN("UNIQUE")))
-        {
-          return (1);
-        }
-      }
-    }
-
-    show_table->cursor->get_foreign_key_list(session, &f_key_list);
-    FOREIGN_KEY_INFO *f_key_info;
-    List_iterator_fast<FOREIGN_KEY_INFO> it(f_key_list);
-    while ((f_key_info=it++))
-    {
-      if (store_constraints(session, table, db_name, table_name,
-                            f_key_info->forein_id->str,
-                            strlen(f_key_info->forein_id->str),
-                            "FOREIGN KEY", 11))
-      {
-        return (1);
-      }
-    }
-  }
-  return (res);
-}
-
-int TablesISMethods::processTable(Session *session, TableList *tables,
-                                    Table *table, bool res,
-                                    LEX_STRING *db_name,
-                                    LEX_STRING *table_name) const
-{
-  const char *tmp_buff;
-  DRIZZLE_TIME time;
-  const CHARSET_INFO * const cs= system_charset_info;
-
-  table->restoreRecordAsDefault();
-  table->field[1]->store(db_name->str, db_name->length, cs);
-  table->field[2]->store(table_name->str, table_name->length, cs);
-  if (res)
-  {
-    /*
-      there was errors during opening tables
-    */
-    const char *error= session->is_error() ? session->main_da.message() : "";
-    if (tables->schema_table)
-    {
-      table->field[3]->store(STRING_WITH_LEN("SYSTEM VIEW"), cs);
-    }
-    else
-    {
-      table->field[3]->store(STRING_WITH_LEN("BASE Table"), cs);
-    }
-    table->field[20]->store(error, strlen(error), cs);
-    session->clear_error();
-  }
-  else
-  {
-    char option_buff[400],*ptr;
-    Table *show_table= tables->table;
-    TableShare *share= show_table->s;
-    Cursor *cursor= show_table->cursor;
-    drizzled::plugin::StorageEngine *tmp_db_type= share->db_type();
-
-    if (share->tmp_table == SYSTEM_TMP_TABLE)
-    {
-      table->field[3]->store(STRING_WITH_LEN("SYSTEM VIEW"), cs);
-    }
-    else if (share->tmp_table)
-    {
-      table->field[3]->store(STRING_WITH_LEN("LOCAL TEMPORARY"), cs);
-    }
-    else
-    {
-      table->field[3]->store(STRING_WITH_LEN("BASE Table"), cs);
-    }
-
-    for (int i= 4; i < 20; i++)
-    {
-      if (i == 7 || (i > 12 && i < 17) || i == 18)
-      {
-        continue;
-      }
-      table->field[i]->set_notnull();
-    }
-    const string &engine_name= drizzled::plugin::StorageEngine::resolveName(tmp_db_type);
-    table->field[4]->store(engine_name.c_str(), engine_name.size(), cs);
-    table->field[5]->store((int64_t) 0, true);
-
-    ptr=option_buff;
-
-    if (share->db_create_options & HA_OPTION_PACK_KEYS)
-    {
-      ptr= strcpy(ptr," pack_keys=1")+12;
-    }
-    if (share->db_create_options & HA_OPTION_NO_PACK_KEYS)
-    {
-      ptr= strcpy(ptr," pack_keys=0")+12;
-    }
-    if (share->row_type != ROW_TYPE_DEFAULT)
-    {
-      ptr+= sprintf(ptr, " row_format=%s", ha_row_type[(uint32_t)share->row_type]);
-    }
-    if (share->block_size)
-    {
-      ptr= strcpy(ptr, " block_size=")+12;
-      ptr= int64_t10_to_str(share->block_size, ptr, 10);
-    }
-
-    table->field[19]->store(option_buff+1,
-                            (ptr == option_buff ? 0 :
-                             (uint32_t) (ptr-option_buff)-1), cs);
-
-    tmp_buff= (share->table_charset ?
-               share->table_charset->name : "default");
-    table->field[17]->store(tmp_buff, strlen(tmp_buff), cs);
-
-    if (share->hasComment())
-      table->field[20]->store(share->getComment(),
-                              share->getCommentLength(), cs);
-
-    if (cursor)
-    {
-      cursor->info(HA_STATUS_VARIABLE | HA_STATUS_TIME | HA_STATUS_AUTO |
-                   HA_STATUS_NO_LOCK);
-      enum row_type row_type = cursor->get_row_type();
-      switch (row_type) {
-      case ROW_TYPE_NOT_USED:
-      case ROW_TYPE_DEFAULT:
-        tmp_buff= ((share->db_options_in_use &
-                    HA_OPTION_COMPRESS_RECORD) ? "Compressed" :
-                   (share->db_options_in_use & HA_OPTION_PACK_RECORD) ?
-                   "Dynamic" : "Fixed");
-        break;
-      case ROW_TYPE_FIXED:
-        tmp_buff= "Fixed";
-        break;
-      case ROW_TYPE_DYNAMIC:
-        tmp_buff= "Dynamic";
-        break;
-      case ROW_TYPE_COMPRESSED:
-        tmp_buff= "Compressed";
-        break;
-      case ROW_TYPE_REDUNDANT:
-        tmp_buff= "Redundant";
-        break;
-      case ROW_TYPE_COMPACT:
-        tmp_buff= "Compact";
-        break;
-      case ROW_TYPE_PAGE:
-        tmp_buff= "Paged";
-        break;
-      }
-      table->field[6]->store(tmp_buff, strlen(tmp_buff), cs);
-      if (! tables->schema_table)
-      {
-        table->field[7]->store((int64_t) cursor->stats.records, true);
-        table->field[7]->set_notnull();
-      }
-      table->field[8]->store((int64_t) cursor->stats.mean_rec_length, true);
-      table->field[9]->store((int64_t) cursor->stats.data_file_length, true);
-      if (cursor->stats.max_data_file_length)
-      {
-        table->field[10]->store((int64_t) cursor->stats.max_data_file_length,
-                                true);
-      }
-      table->field[11]->store((int64_t) cursor->stats.index_file_length, true);
-      table->field[12]->store((int64_t) cursor->stats.delete_length, true);
-      if (show_table->found_next_number_field)
-      {
-        table->field[13]->store((int64_t) cursor->stats.auto_increment_value, true);
-        table->field[13]->set_notnull();
-      }
-      if (cursor->stats.create_time)
-      {
-        session->variables.time_zone->gmt_sec_to_TIME(&time,
-                                                  (time_t) cursor->stats.create_time);
-        table->field[14]->store_time(&time, DRIZZLE_TIMESTAMP_DATETIME);
-        table->field[14]->set_notnull();
-      }
-      if (cursor->stats.update_time)
-      {
-        session->variables.time_zone->gmt_sec_to_TIME(&time,
-                                                  (time_t) cursor->stats.update_time);
-        table->field[15]->store_time(&time, DRIZZLE_TIMESTAMP_DATETIME);
-        table->field[15]->set_notnull();
-      }
-      if (cursor->stats.check_time)
-      {
-        session->variables.time_zone->gmt_sec_to_TIME(&time,
-                                                  (time_t) cursor->stats.check_time);
-        table->field[16]->store_time(&time, DRIZZLE_TIMESTAMP_DATETIME);
-        table->field[16]->set_notnull();
-      }
-      if (cursor->ha_table_flags() & (ulong) HA_HAS_CHECKSUM)
-      {
-        table->field[18]->store((int64_t) cursor->checksum(), true);
-        table->field[18]->set_notnull();
-      }
-    }
-  }
-  return (schema_table_store_record(session, table));
-}
-
-
-int TabNamesISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
-  const
-{
-  char tmp[128];
-  String buffer(tmp,sizeof(tmp), session->charset());
-  LEX *lex= session->lex;
-  Name_resolution_context *context= &lex->select_lex.context;
-  const drizzled::plugin::InfoSchemaTable::Columns columns= schema_table->getColumns();
-
-  const drizzled::plugin::ColumnInfo *column= columns[2];
-  buffer.length(0);
-  buffer.append(column->getOldName().c_str());
-  buffer.append(lex->select_lex.db);
-  if (lex->wild && lex->wild->ptr())
-  {
-    buffer.append(STRING_WITH_LEN(" ("));
-    buffer.append(lex->wild->ptr());
-    buffer.append(')');
-  }
-  Item_field *field= new Item_field(context,
-                                    NULL, NULL, column->getName().c_str());
-  if (session->add_item_to_list(field))
-  {
-    return 1;
-  }
-  field->set_name(buffer.ptr(), buffer.length(), system_charset_info);
-  if (session->lex->verbose)
-  {
-    field->set_name(buffer.ptr(), buffer.length(), system_charset_info);
-    column= columns[3];
-    field= new Item_field(context, NULL, NULL, column->getName().c_str());
-    if (session->add_item_to_list(field))
-    {
-      return 1;
-    }
-    field->set_name(column->getOldName().c_str(),
-                    column->getOldName().length(),
-                    system_charset_info);
-  }
-  return 0;
-}
-
-int VariablesISMethods::fillTable(Session *session, TableList *tables)
-{
-  int res= 0;
-  LEX *lex= session->lex;
-  const char *wild= lex->wild ? lex->wild->ptr() : NULL;
-  const string schema_table_name= tables->schema_table->getTableName();
-  enum enum_var_type option_type= OPT_SESSION;
-  bool upper_case_names= (schema_table_name.compare("VARIABLES") != 0);
-  bool sorted_vars= (schema_table_name.compare("VARIABLES") == 0);
-
-  if (lex->option_type == OPT_GLOBAL ||
-      schema_table_name.compare("GLOBAL_VARIABLES") == 0)
-  {
-    option_type= OPT_GLOBAL;
-  }
-
-  pthread_rwlock_rdlock(&LOCK_system_variables_hash);
-  res= show_status_array(session, wild, enumerate_sys_vars(session, sorted_vars),
-                         option_type, NULL, "", tables->table, upper_case_names);
-  pthread_rwlock_unlock(&LOCK_system_variables_hash);
-  return(res);
-}
-
+/*
+ * Function object used for deleting the memory allocated
+ * for the columns contained with the vector of columns.
+ */
+class DeleteColumns
+{
+public:
+  template<typename T>
+  inline void operator()(const T *ptr) const
+  {
+    delete ptr;
+  }
+};
+
+void clearColumns(vector<const drizzled::plugin::ColumnInfo *> &cols)
+{
+  for_each(cols.begin(), cols.end(), DeleteColumns());
+  cols.clear();
+}

=== renamed file 'plugin/info_schema/info_schema_methods.h' => 'plugin/info_schema/helper_methods.h'
--- plugin/info_schema/info_schema_methods.h	2009-11-03 21:20:29 +0000
+++ plugin/info_schema/helper_methods.h	2009-11-18 21:40:45 +0000
@@ -18,253 +18,34 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef PLUGIN_INFO_SCHEMA_INFO_SCHEMA_METHODS_H
-#define PLUGIN_INFO_SCHEMA_INFO_SCHEMA_METHODS_H
+#ifndef PLUGIN_INFO_SCHEMA_HELPER_METHODS_H
+#define PLUGIN_INFO_SCHEMA_HELPER_METHODS_H
 
 #include "drizzled/plugin/info_schema_table.h"
 
-/**
- * @class CharSetISMethods
- * @brief
- *   Class which implements any methods that the 
- *   CHARACTER_SET I_S table needs besides the default
- *   methods.
- */
-class CharSetISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session, 
-                        TableList *tables);
-  virtual int oldFormat(Session *session,
-                        drizzled::plugin::InfoSchemaTable *schema_table) const;
-};
-
-/**
- * @class CollationISMethods
- * @brief
- *   Class which implements any methods that the Collations
- *   I_S table needs besides the default methods
- */
-class CollationISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-};
-
-/**
- * @class CollCharISMethods
- * @brief
- *   Class which implements any methods that the collation char set
- *   I_S table needs besides the default methods
- */
-class CollCharISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-};
-
-/**
- * @class ColumnsISMethods
- * @brief
- *   Class which implements any methods that the COLUMNS
- *   I_S table needs besides the default methods
- */
-class ColumnsISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int oldFormat(Session *session,
-                        drizzled::plugin::InfoSchemaTable *schema_table) const;
-};
-
-/**
- * @class KeyColUsageISMethods
- * @brief
- *   Class which implements any methods that the key column usage
- *   I_S table needs besides the default methods
- */
-class KeyColUsageISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int processTable(Session *session, TableList *tables,
-                           Table *table, bool res, LEX_STRING *db_name,
-                           LEX_STRING *table_name) const;
-};
-
-/**
- * @class OpenTablesISMethods
- * @brief
- *   Class which implements any methods that the OPEN_TABLES
- *   I_S table needs besides the default methods
- */
-class OpenTablesISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-};
-
-/**
- * @class ModulesISMethods
- * @brief
- *   Class which implements any methods that the MODULES
- *   I_S table needs besides the default methods
- */
-class ModulesISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-};
-
-/**
- * @class PluginsISMethods
- * @brief
- *   Class which implements any methods that the PLUGINS
- *   I_S table needs besides the default methods
- */
-class PluginsISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-};
-
-/**
- * @class ProcessListISMethods
- * @brief
- *   Class which implements any methods that the PROCESSLIST
- *   I_S table needs besides the default methods
- */
-class ProcessListISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-};
-
-/**
- * @class RefConstraintISMethods
- * @brief
- *   Class which implements any methods that the REFERENTIAL_CONSTRAINTS
- *   I_S table needs besides the default methods
- */
-class RefConstraintsISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  /**
-   * Fill and store records into I_S.referential_constraints table
-   *
-   * @param[in] session   thread handle
-   * @param[in] tables    table list struct(processed table)
-   * @param[in] table     I_S table
-   * @param[in] res       1 means the error during opening of the processed table
-   *                      0 means processed table is opened without error
-   * @param[in] base_name db name
-   * @param[in] file_name table name
-   *
-   * @return 0 on success; 1 on failure
-   */
-  virtual int processTable(Session *session, TableList *tables,
-                           Table *table, bool res, LEX_STRING *db_name,
-                           LEX_STRING *table_name) const;
-};
-
-/**
- * @class SchemataISMethods
- * @brief
- *   Class which implements any methods that the SCHEMATA
- *   I_S table needs besides the default methods
- */
-class SchemataISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session,
-                        TableList *tables);
-  virtual int oldFormat(Session *session,
-                        drizzled::plugin::InfoSchemaTable *schema_table) const;
-};
-
-/**
- * @class StatsISMethods
- * @brief
- *   Class which implements any methods that the SCHEMATA
- *   I_S table needs besides the default methods
- */
-class StatsISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int processTable(Session *session, TableList *tables,
-                           Table *table, bool res, LEX_STRING *db_name,
-                           LEX_STRING *table_name) const;
-};
-
-/**
- * @class StatusISMethods
- * @brief
- *   Class which implements any methods that the STATUS
- *   I_S table needs besides the default methods
- */
-class StatusISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session, 
-                        TableList *tables);
-};
-
-/**
- * @class TabConstraintsISMethods
- * @brief
- *   Class which implements any methods that the TABLE_CONSTRAINTS
- *   I_S table needs besides the default methods
- */
-class TabConstraintsISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int processTable(Session *session, TableList *tables,
-                           Table *table, bool res, LEX_STRING *db_name,
-                           LEX_STRING *table_name) const;
-};
-
-/**
- * @class TablesISMethods
- * @brief
- *   Class which implements any methods that the TABLE_NAMES
- *   I_S table needs besides the default methods
- */
-class TablesISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int processTable(Session *session, TableList *tables,
-                           Table *table, bool res, LEX_STRING *db_name,
-                           LEX_STRING *table_name) const;
-};
-
-/**
- * @class TabNamesISMethods
- * @brief
- *   Class which implements any methods that the TABLE_NAMES
- *   I_S table needs besides the default methods
- */
-class TabNamesISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int oldFormat(Session *session,
-                        drizzled::plugin::InfoSchemaTable *schema_table) const;
-};
-
-/**
- * @class VariablesISMethods
- * @brief
- *   Class which implements any methods that the VARIABLES
- *   I_S table needs besides the default methods
- */
-class VariablesISMethods : public drizzled::plugin::InfoSchemaMethods
-{
-public:
-  virtual int fillTable(Session *session, 
-                        TableList *tables);
-};
-
-#endif /* PLUGIN_INFO_SCHEMA_INFO_SCHEMA_METHODS_H */
+bool show_status_array(Session *session, 
+                       const char *wild,
+                       SHOW_VAR *variables,
+                       enum enum_var_type value_type,
+                       struct system_status_var *status_var,
+                       const char *prefix, Table *table,
+                       bool ucase_names);
+
+void store_key_column_usage(Table *table, 
+                            LEX_STRING *db_name,
+                            LEX_STRING *table_name, 
+                            const char *key_name,
+                            uint32_t key_len, 
+                            const char *con_type, 
+                            uint32_t con_len,
+                            int64_t idx);
+
+/**
+ * Iterate through the given vector of columns and delete the memory that
+ * has been allocated for them.
+ *
+ * @param[out] cols vector to clear and de-allocate memory from
+ */
+void clearColumns(std::vector<const drizzled::plugin::ColumnInfo *> &cols);
+
+#endif /* PLUGIN_INFO_SCHEMA_HELPER_METHODS_H */

=== modified file 'plugin/info_schema/info_schema.cc'
--- plugin/info_schema/info_schema.cc	2009-11-10 20:50:03 +0000
+++ plugin/info_schema/info_schema.cc	2009-11-18 16:39:59 +0000
@@ -23,552 +23,34 @@
  *   I_S plugin implementation.
  */
 
-#include <drizzled/server_includes.h>
-#include <drizzled/session.h>
-#include <drizzled/show.h>
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
 
-#include "info_schema_methods.h"
-#include "info_schema_columns.h"
+#include "helper_methods.h"
+#include "character_set.h"
+#include "collation.h"
+#include "collation_char_set.h"
+#include "columns.h"
+#include "key_column_usage.h"
+#include "modules.h"
+#include "open_tables.h"
+#include "plugins.h"
+#include "processlist.h"
+#include "referential_constraints.h"
+#include "schemata.h"
+#include "table_constraints.h"
+#include "tables.h"
+#include "table_names.h"
+#include "statistics.h"
+#include "status.h"
+#include "variables.h"
 
 #include <vector>
 
 using namespace drizzled;
 using namespace std;
 
-/*
- * Vectors of columns for various I_S tables.
- */
-static vector<const plugin::ColumnInfo *> char_set_columns;
-static vector<const plugin::ColumnInfo *> collation_columns;
-static vector<const plugin::ColumnInfo *> coll_char_columns;
-static vector<const plugin::ColumnInfo *> col_columns;
-static vector<const plugin::ColumnInfo *> key_col_usage_columns;
-static vector<const plugin::ColumnInfo *> open_tab_columns;
-static vector<const plugin::ColumnInfo *> modules_columns;
-static vector<const plugin::ColumnInfo *> plugins_columns;
-static vector<const plugin::ColumnInfo *> processlist_columns;
-static vector<const plugin::ColumnInfo *> ref_constraint_columns;
-static vector<const plugin::ColumnInfo *> schemata_columns;
-static vector<const plugin::ColumnInfo *> stats_columns;
-static vector<const plugin::ColumnInfo *> status_columns;
-static vector<const plugin::ColumnInfo *> tab_constraints_columns;
-static vector<const plugin::ColumnInfo *> tables_columns;
-static vector<const plugin::ColumnInfo *> tab_names_columns;
-
-/*
- * Methods for various I_S tables.
- */
-static plugin::InfoSchemaMethods *char_set_methods= NULL;
-static plugin::InfoSchemaMethods *collation_methods= NULL;
-static plugin::InfoSchemaMethods *coll_char_methods= NULL;
-static plugin::InfoSchemaMethods *columns_methods= NULL;
-static plugin::InfoSchemaMethods *key_col_usage_methods= NULL;
-static plugin::InfoSchemaMethods *open_tab_methods= NULL;
-static plugin::InfoSchemaMethods *modules_methods= NULL;
-static plugin::InfoSchemaMethods *plugins_methods= NULL;
-static plugin::InfoSchemaMethods *processlist_methods= NULL;
-static plugin::InfoSchemaMethods *ref_constraint_methods= NULL;
-static plugin::InfoSchemaMethods *schemata_methods= NULL;
-static plugin::InfoSchemaMethods *stats_methods= NULL;
-static plugin::InfoSchemaMethods *status_methods= NULL;
-static plugin::InfoSchemaMethods *tab_constraints_methods= NULL;
-static plugin::InfoSchemaMethods *tables_methods= NULL;
-static plugin::InfoSchemaMethods *tab_names_methods= NULL;
-static plugin::InfoSchemaMethods *variables_methods= NULL;
-
-/*
- * I_S tables.
- */
-static plugin::InfoSchemaTable *char_set_table= NULL;
-static plugin::InfoSchemaTable *collation_table= NULL;
-static plugin::InfoSchemaTable *coll_char_set_table= NULL;
-static plugin::InfoSchemaTable *columns_table= NULL;
-static plugin::InfoSchemaTable *key_col_usage_table= NULL;
-static plugin::InfoSchemaTable *global_stat_table= NULL;
-static plugin::InfoSchemaTable *global_var_table= NULL;
-static plugin::InfoSchemaTable *open_tab_table= NULL;
-static plugin::InfoSchemaTable *modules_table= NULL;
-static plugin::InfoSchemaTable *plugins_table= NULL;
-static plugin::InfoSchemaTable *processlist_table= NULL;
-static plugin::InfoSchemaTable *ref_constraint_table= NULL;
-static plugin::InfoSchemaTable *schemata_table= NULL;
-static plugin::InfoSchemaTable *sess_stat_table= NULL;
-static plugin::InfoSchemaTable *sess_var_table= NULL;
-static plugin::InfoSchemaTable *stats_table= NULL;
-static plugin::InfoSchemaTable *status_table= NULL;
-static plugin::InfoSchemaTable *tab_constraints_table= NULL;
-static plugin::InfoSchemaTable *tables_table= NULL;
-static plugin::InfoSchemaTable *tab_names_table= NULL;
-static plugin::InfoSchemaTable *var_table= NULL;
-
-/**
- * Populate the vectors of columns for each I_S table.
- *
- * @return false on success; true on failure.
- */
-static bool initTableColumns()
-{
-  bool retval= false;
-
-  if ((retval= createCharSetColumns(char_set_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createCollationColumns(collation_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createCollCharSetColumns(coll_char_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createColColumns(col_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createKeyColUsageColumns(key_col_usage_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createOpenTabColumns(open_tab_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createModulesColumns(modules_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createPluginsColumns(plugins_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createProcessListColumns(processlist_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createRefConstraintColumns(ref_constraint_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createSchemataColumns(schemata_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createStatsColumns(stats_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createStatusColumns(status_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createTabConstraintsColumns(tab_constraints_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createTablesColumns(tables_columns)) == true)
-  {
-    return true;
-  }
-
-  if ((retval= createTabNamesColumns(tab_names_columns)) == true)
-  {
-    return true;
-  }
-
-  return false;
-}
-
-/**
- * Clear the vectors of columns for each I_S table.
- */
-static void cleanupTableColumns()
-{
-  clearColumns(char_set_columns);
-  clearColumns(collation_columns);
-  clearColumns(coll_char_columns);
-  clearColumns(col_columns);
-  clearColumns(key_col_usage_columns);
-  clearColumns(open_tab_columns);
-  clearColumns(modules_columns);
-  clearColumns(plugins_columns);
-  clearColumns(processlist_columns);
-  clearColumns(ref_constraint_columns);
-  clearColumns(schemata_columns);
-  clearColumns(stats_columns);
-  clearColumns(status_columns);
-  clearColumns(tab_constraints_columns);
-  clearColumns(tables_columns);
-  clearColumns(tab_names_columns);
-}
-
-/**
- * Initialize the methods for each I_S table.
- *
- * @return false on success; true on failure
- */
-static bool initTableMethods()
-{
-  if ((char_set_methods= new(nothrow) CharSetISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((collation_methods= new(nothrow) CollationISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((coll_char_methods= new(nothrow) CollCharISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((columns_methods= new(nothrow) ColumnsISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((key_col_usage_methods= new(nothrow) KeyColUsageISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((open_tab_methods= new(nothrow) OpenTablesISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((modules_methods= new(nothrow) ModulesISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((plugins_methods= new(nothrow) PluginsISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((processlist_methods= new(nothrow) ProcessListISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((ref_constraint_methods= new(nothrow) RefConstraintsISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((schemata_methods= new(nothrow) SchemataISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((stats_methods= new(nothrow) StatsISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((status_methods= new(nothrow) StatusISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((tab_constraints_methods= new(nothrow) TabConstraintsISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((tables_methods= new(nothrow) TablesISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((tab_names_methods= new(nothrow) TabNamesISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  if ((variables_methods= new(nothrow) VariablesISMethods()) == NULL)
-  {
-    return true;
-  }
-
-  return false;
-}
-
-/**
- * Delete memory allocated for the I_S table methods.
- */
-static void cleanupTableMethods()
-{
-  delete char_set_methods;
-  delete collation_methods;
-  delete coll_char_methods;
-  delete columns_methods;
-  delete key_col_usage_methods;
-  delete open_tab_methods;
-  delete modules_methods;
-  delete plugins_methods;
-  delete processlist_methods;
-  delete ref_constraint_methods;
-  delete schemata_methods;
-  delete stats_methods;
-  delete status_methods;
-  delete tab_constraints_methods;
-  delete tables_methods;
-  delete tab_names_methods;
-  delete variables_methods;
-}
-
-/**
- * Initialize the I_S tables.
- *
- * @return false on success; true on failure
- */
-static bool initTables()
-{
-
-  char_set_table= new(nothrow) plugin::InfoSchemaTable("CHARACTER_SETS",
-                                                       char_set_columns,
-                                                       -1, -1, false, false, 0,
-                                                       char_set_methods);
-  if (char_set_table == NULL)
-  {
-    return true;
-  }
-
-  collation_table= new(nothrow) plugin::InfoSchemaTable("COLLATIONS",
-                                                        collation_columns,
-                                                        -1, -1, false, false, 0,
-                                                        collation_methods);
-  if (collation_table == NULL)
-  {
-    return true;
-  }
-
-  coll_char_set_table= new(nothrow) plugin::InfoSchemaTable("COLLATION_CHARACTER_SET_APPLICABILITY",
-                                                            coll_char_columns,
-                                                            -1, -1, false, false, 0,
-                                                            coll_char_methods);
-  if (coll_char_set_table == NULL)
-  {
-    return true;
-  }
-
-  columns_table= new(nothrow) plugin::InfoSchemaTable("COLUMNS",
-                                                      col_columns,
-                                                      1, 2, false, true,
-                                                      OPTIMIZE_I_S_TABLE,
-                                                      columns_methods);
-  if (columns_table == NULL)
-  {
-    return true;
-  }
-
-  key_col_usage_table= new(nothrow) plugin::InfoSchemaTable("KEY_COLUMN_USAGE",
-                                                            key_col_usage_columns,
-                                                            4, 5, false, true,
-                                                            OPEN_TABLE_ONLY,
-                                                            key_col_usage_methods);
-  if (key_col_usage_table == NULL)
-  {
-    return true;
-  }
-
-  global_stat_table= new(nothrow) plugin::InfoSchemaTable("GLOBAL_STATUS",
-                                                          status_columns,
-                                                          -1, -1, false, false,
-                                                          0, status_methods);
-  if (global_stat_table == NULL)
-  {
-    return true;
-  }
-
-  global_var_table= new(nothrow) plugin::InfoSchemaTable("GLOBAL_VARIABLES",
-                                                         status_columns,
-                                                         -1, -1, false, false,
-                                                         0, variables_methods);
-  if (global_var_table == NULL)
-  {
-    return true;
-  }
-
-  open_tab_table= new(nothrow) plugin::InfoSchemaTable("OPEN_TABLES",
-                                                       open_tab_columns,
-                                                       -1, -1, true, false, 0,
-                                                       open_tab_methods);
-  if (open_tab_table == NULL)
-  {
-    return true;
-  }
-
-  modules_table= new(nothrow) plugin::InfoSchemaTable("MODULES",
-                                                      modules_columns,
-                                                      -1, -1, false, false, 0,
-                                                      modules_methods);
-  if (modules_table == NULL)
-  {
-    return true;
-  }
-
-  plugins_table= new(nothrow) plugin::InfoSchemaTable("PLUGINS",
-                                                      plugins_columns,
-                                                      -1, -1, false, false, 0,
-                                                      plugins_methods);
-  if (plugins_table == NULL)
-  {
-    return true;
-  }
-
-  processlist_table= new(nothrow) plugin::InfoSchemaTable("PROCESSLIST",
-                                                          processlist_columns,
-                                                          -1, -1, false, false, 0,
-                                                          processlist_methods);
-  if (processlist_table == NULL)
-  {
-    return true;
-  }
-
-  ref_constraint_table= new(nothrow) plugin::InfoSchemaTable("REFERENTIAL_CONSTRAINTS",
-                                                             ref_constraint_columns,
-                                                             1, 9, false, true,
-                                                             OPEN_TABLE_ONLY,
-                                                             ref_constraint_methods);
-  if (ref_constraint_table == NULL)
-  {
-    return true;
-  }
-
-  schemata_table= new(nothrow) plugin::InfoSchemaTable("SCHEMATA",
-                                                       schemata_columns,
-                                                       1, -1, false, false, 0,
-                                                       schemata_methods);
-  if (schemata_table == NULL)
-  {
-    return true;
-  }
-
-  sess_stat_table= new(nothrow) plugin::InfoSchemaTable("SESSION_STATUS",
-                                                        status_columns,
-                                                        -1, -1, false, false,
-                                                        0, status_methods);
-  if (sess_stat_table == NULL)
-  {
-    return true;
-  }
-
-  sess_var_table= new(nothrow) plugin::InfoSchemaTable("SESSION_VARIABLES",
-                                                       status_columns,
-                                                       -1, -1, false, false, 0,
-                                                       variables_methods);
-  if (sess_var_table == NULL)
-  {
-    return true;
-  }
-
-  stats_table= new(nothrow) plugin::InfoSchemaTable("STATISTICS",
-                                                    stats_columns,
-                                                    1, 2, false, true,
-                                                    OPEN_TABLE_ONLY | OPTIMIZE_I_S_TABLE,
-                                                    stats_methods);
-  if (stats_table == NULL)
-  {
-    return true;
-  }
-
-  status_table= new(nothrow) plugin::InfoSchemaTable("STATUS",
-                                                     status_columns,
-                                                     -1, -1, true, false, 0,
-                                                     status_methods);
-  if (status_table == NULL)
-  {
-    return true;
-  }
-
-  tab_constraints_table= new(nothrow) plugin::InfoSchemaTable("TABLE_CONSTRAINTS",
-                                                              tab_constraints_columns,
-                                                              3, 4, false, true,
-                                                              OPEN_TABLE_ONLY,
-                                                              tab_constraints_methods);
-  if (tab_constraints_table == NULL)
-  {
-    return true;
-  }
-
-  tables_table= new(nothrow) plugin::InfoSchemaTable("TABLES",
-                                                     tables_columns,
-                                                     1, 2, false, true,
-                                                     OPTIMIZE_I_S_TABLE,
-                                                     tables_methods);
-  if (tables_table == NULL)
-  {
-    return true;
-  }
-
-  tab_names_table= new(nothrow) plugin::InfoSchemaTable("TABLE_NAMES",
-                                                        tab_names_columns,
-                                                        1, 2, true, true, 0,
-                                                        tab_names_methods);
-  if (tab_names_table == NULL)
-  {
-    return true;
-  }
-
-  var_table= new(nothrow) plugin::InfoSchemaTable("VARIABLES",
-                                                  status_columns,
-                                                  -1, -1, true, false, 0,
-                                                  variables_methods);
-  if (var_table == NULL)
-  {
-    return true;
-  }
-
-  return false;
-}
-
-/**
- * Delete memory allocated for the I_S tables.
- */
-static void cleanupTables()
-{
-  delete char_set_table;
-  delete collation_table;
-  delete coll_char_set_table;
-  delete columns_table;
-  delete key_col_usage_table;
-  delete global_stat_table;
-  delete global_var_table;
-  delete open_tab_table;
-  delete plugins_table;
-  delete modules_table;
-  delete processlist_table;
-  delete ref_constraint_table;
-  delete schemata_table;
-  delete sess_stat_table;
-  delete sess_var_table;
-  delete stats_table;
-  delete status_table;
-  delete tab_constraints_table;
-  delete tables_table;
-  delete tab_names_table;
-  delete var_table;
-}
-
 /**
  * Initialize the I_S plugin.
  *
@@ -577,44 +59,27 @@
  */
 static int infoSchemaInit(drizzled::plugin::Registry& registry)
 {
-  bool retval= false;
-
-  if ((retval= initTableMethods()) == true)
-  {
-    return 1;
-  }
-
-  if ((retval= initTableColumns()) == true)
-  {
-    return 1;
-  }
-
-  if ((retval= initTables()) == true)
-  {
-    return 1;
-  }
-
-  registry.add(char_set_table);
-  registry.add(collation_table);
-  registry.add(coll_char_set_table);
-  registry.add(columns_table);
-  registry.add(key_col_usage_table);
-  registry.add(global_stat_table);
-  registry.add(global_var_table);
-  registry.add(open_tab_table);
-  registry.add(modules_table);
-  registry.add(plugins_table);
-  registry.add(processlist_table);
-  registry.add(ref_constraint_table);
-  registry.add(schemata_table);
-  registry.add(sess_stat_table);
-  registry.add(sess_var_table);
-  registry.add(stats_table);
-  registry.add(status_table);
-  registry.add(tab_constraints_table);
-  registry.add(tables_table);
-  registry.add(tab_names_table);
-  registry.add(var_table);
+  registry.add(CharacterSetIS::getTable());
+  registry.add(CollationIS::getTable());
+  registry.add(CollationCharSetIS::getTable());
+  registry.add(ColumnsIS::getTable());
+  registry.add(KeyColumnUsageIS::getTable());
+  registry.add(GlobalStatusIS::getTable());
+  registry.add(GlobalVariablesIS::getTable());
+  registry.add(OpenTablesIS::getTable());
+  registry.add(ModulesIS::getTable());
+  registry.add(PluginsIS::getTable());
+  registry.add(ProcessListIS::getTable());
+  registry.add(ReferentialConstraintsIS::getTable());
+  registry.add(SchemataIS::getTable());
+  registry.add(SessionStatusIS::getTable());
+  registry.add(SessionVariablesIS::getTable());
+  registry.add(StatisticsIS::getTable());
+  registry.add(StatusIS::getTable());
+  registry.add(TableConstraintsIS::getTable());
+  registry.add(TablesIS::getTable());
+  registry.add(TableNamesIS::getTable());
+  registry.add(VariablesIS::getTable());
 
   return 0;
 }
@@ -627,31 +92,68 @@
  */
 static int infoSchemaDone(drizzled::plugin::Registry& registry)
 {
-  registry.remove(char_set_table);
-  registry.remove(collation_table);
-  registry.remove(coll_char_set_table);
-  registry.remove(columns_table);
-  registry.remove(key_col_usage_table);
-  registry.remove(global_stat_table);
-  registry.remove(global_var_table);
-  registry.remove(open_tab_table);
-  registry.remove(modules_table);
-  registry.remove(plugins_table);
-  registry.remove(processlist_table);
-  registry.remove(ref_constraint_table);
-  registry.remove(schemata_table);
-  registry.remove(sess_stat_table);
-  registry.remove(sess_var_table);
-  registry.remove(stats_table);
-  registry.remove(status_table);
-  registry.remove(tab_constraints_table);
-  registry.remove(tables_table);
-  registry.remove(tab_names_table);
-  registry.remove(var_table);
-
-  cleanupTableMethods();
-  cleanupTableColumns();
-  cleanupTables();
+  registry.remove(CharacterSetIS::getTable());
+  CharacterSetIS::cleanup();
+
+  registry.remove(CollationIS::getTable());
+  CollationIS::cleanup();
+
+  registry.remove(CollationCharSetIS::getTable());
+  CollationCharSetIS::cleanup();
+
+  registry.remove(ColumnsIS::getTable());
+  ColumnsIS::cleanup();
+
+  registry.remove(KeyColumnUsageIS::getTable());
+  KeyColumnUsageIS::cleanup();
+
+  registry.remove(GlobalStatusIS::getTable());
+  GlobalStatusIS::cleanup();
+
+  registry.remove(GlobalVariablesIS::getTable());
+  GlobalVariablesIS::cleanup();
+
+  registry.remove(OpenTablesIS::getTable());
+  OpenTablesIS::cleanup();
+
+  registry.remove(ModulesIS::getTable());
+  ModulesIS::cleanup();
+
+  registry.remove(PluginsIS::getTable());
+  PluginsIS::cleanup();
+
+  registry.remove(ProcessListIS::getTable());
+  ProcessListIS::cleanup();
+
+  registry.remove(ReferentialConstraintsIS::getTable());
+  ReferentialConstraintsIS::cleanup();
+
+  registry.remove(SchemataIS::getTable());
+  SchemataIS::cleanup();
+
+  registry.remove(SessionStatusIS::getTable());
+  SessionStatusIS::cleanup();
+
+  registry.remove(SessionVariablesIS::getTable());
+  SessionVariablesIS::cleanup();
+
+  registry.remove(StatisticsIS::getTable());
+  StatisticsIS::cleanup();
+
+  registry.remove(StatusIS::getTable());
+  StatusIS::cleanup();
+
+  registry.remove(TableConstraintsIS::getTable());
+  TableConstraintsIS::cleanup();
+
+  registry.remove(TablesIS::getTable());
+  TablesIS::cleanup();
+
+  registry.remove(TableNamesIS::getTable());
+  TableNamesIS::cleanup();
+
+  registry.remove(VariablesIS::getTable());
+  VariablesIS::cleanup();
 
   return 0;
 }
@@ -659,7 +161,7 @@
 drizzle_declare_plugin
 {
   "info_schema",
-  "0.1",
+  "1.1",
   "Padraig O'Sullivan",
   "I_S plugin",
   PLUGIN_LICENSE_GPL,

=== removed file 'plugin/info_schema/info_schema_columns.cc'
--- plugin/info_schema/info_schema_columns.cc	2009-10-27 00:04:25 +0000
+++ plugin/info_schema/info_schema_columns.cc	1970-01-01 00:00:00 +0000
@@ -1,1941 +0,0 @@
-/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
- *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
- *
- *  Copyright (C) 2009 Sun Microsystems
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- */
-
-/**
- * @file
- *   Contains methods for creating the various columns for 
- *   each I_S table.
- */
-
-#include <drizzled/server_includes.h>
-#include <drizzled/session.h>
-#include <drizzled/show.h>
-
-#include "info_schema_columns.h"
-
-#define LIST_PROCESS_HOST_LEN 64
-
-using namespace std;
-
-bool createCharSetColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  /*
-   * Create each column for the CHARACTER_SET table.
-   */
-  const drizzled::plugin::ColumnInfo *name_col= new(std::nothrow) drizzled::plugin::ColumnInfo("CHARACTER_SET_NAME",
-                                                           64,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Charset",
-                                                           SKIP_OPEN_TABLE);
-  if (name_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *collate_col= new(std::nothrow) drizzled::plugin::ColumnInfo("DEFAULT_COLLATE_NAME",
-                                                              64,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Default collation",
-                                                              SKIP_OPEN_TABLE);
-  if (collate_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *descrip_col= new(std::nothrow) drizzled::plugin::ColumnInfo("DESCRIPTION",
-                                                              60,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Description",
-                                                              SKIP_OPEN_TABLE);
-  if (descrip_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *len_col= new(std::nothrow) drizzled::plugin::ColumnInfo("MAXLEN",
-                                                          3,
-                                                          DRIZZLE_TYPE_LONGLONG,
-                                                          0,
-                                                          0,
-                                                          "Maxlen",
-                                                          SKIP_OPEN_TABLE);
-  if (len_col == NULL)
-  {
-    return true;
-  }
-
-  /*
-   * Add the columns to the vector.
-   */
-  cols.push_back(name_col);
-  cols.push_back(collate_col);
-  cols.push_back(descrip_col);
-  cols.push_back(len_col);
-
-  return false;
-}
-
-bool createCollationColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  /*
-   * Create each column for the COLLATION table.
-   */
-  const drizzled::plugin::ColumnInfo *name_col= new(std::nothrow) drizzled::plugin::ColumnInfo("COLLATION_NAME",
-                                                           64,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Collation",
-                                                           SKIP_OPEN_TABLE);
-  if (name_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *char_set_col= new(std::nothrow) drizzled::plugin::ColumnInfo("CHARACTER_SET_NAME",
-                                                               64,
-                                                               DRIZZLE_TYPE_VARCHAR,
-                                                               0,
-                                                               0,
-                                                               "Default collation",
-                                                               SKIP_OPEN_TABLE);
-  if (char_set_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *descrip_col= new(std::nothrow) drizzled::plugin::ColumnInfo("DESCRIPTION",
-                                                              60,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Charset",
-                                                              SKIP_OPEN_TABLE);
-  if (descrip_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *id_col= new(std::nothrow) drizzled::plugin::ColumnInfo("ID",
-                                                         MY_INT32_NUM_DECIMAL_DIGITS,
-                                                         DRIZZLE_TYPE_LONGLONG,
-                                                         0,
-                                                         0,
-                                                         "Id",
-                                                         SKIP_OPEN_TABLE);
-  if (id_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *default_col= new(std::nothrow) drizzled::plugin::ColumnInfo("IS_DEFAULT",
-                                                              3,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Default",
-                                                              SKIP_OPEN_TABLE);
-  if (default_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *compiled_col= new(std::nothrow) drizzled::plugin::ColumnInfo("IS_COMPILED",
-                                                               3,
-                                                               DRIZZLE_TYPE_VARCHAR,
-                                                               0,
-                                                               0,
-                                                               "Compiled",
-                                                               SKIP_OPEN_TABLE);
-  if (compiled_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sortlen_col= new(std::nothrow) drizzled::plugin::ColumnInfo("SORTLEN",
-                                                              3,
-                                                              DRIZZLE_TYPE_LONGLONG,
-                                                              0,
-                                                              0,
-                                                              "Sortlen",
-                                                              SKIP_OPEN_TABLE);
-  if (sortlen_col == NULL)
-  {
-    return true;
-  }
-
-  /*
-   * Add the columns to the vector.
-   */
-  cols.push_back(name_col);
-  cols.push_back(char_set_col);
-  cols.push_back(descrip_col);
-  cols.push_back(id_col);
-  cols.push_back(default_col);
-  cols.push_back(compiled_col);
-  cols.push_back(sortlen_col);
-
-  return false;
-}
-
-bool createCollCharSetColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  /*
-   * Create each column for the table.
-   */
-  const drizzled::plugin::ColumnInfo *name_col= new(std::nothrow) drizzled::plugin::ColumnInfo("COLLATION_NAME",
-                                                           64,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           SKIP_OPEN_TABLE);
-  if (name_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *char_set_col= new(std::nothrow) drizzled::plugin::ColumnInfo("CHARACTER_SET_NAME",
-                                                               64,
-                                                               DRIZZLE_TYPE_VARCHAR,
-                                                               0,
-                                                               0,
-                                                               "",
-                                                               SKIP_OPEN_TABLE);
-  if (char_set_col == NULL)
-  {
-    return true;
-  }
-
-  /*
-   * Add the columns to the vector.
-   */
-  cols.push_back(name_col);
-  cols.push_back(char_set_col);
-
-  return false;
-}
-
-bool createColColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  /*
-   * Create each column for the COLUMNS table.
-   */
-  const drizzled::plugin::ColumnInfo *tab_cat= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_CATALOG",
-                                                          FN_REFLEN,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          1,
-                                                          "",
-                                                          OPEN_FRM_ONLY);
-  if (tab_cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_sch= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_SCHEMA",
-                                                          NAME_CHAR_LEN,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          0,
-                                                          "",
-                                                          OPEN_FRM_ONLY);
-  if (tab_sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FRM_ONLY);
-  if (tab_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_name= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Field",
-                                                           OPEN_FRM_ONLY);
-  if (col_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ord_pos= new(std::nothrow) drizzled::plugin::ColumnInfo("ORDINAL_POSITION",
-                                                          MY_INT64_NUM_DECIMAL_DIGITS,
-                                                          DRIZZLE_TYPE_LONGLONG,
-                                                          0,
-                                                          MY_I_S_UNSIGNED,
-                                                          "",
-                                                          OPEN_FRM_ONLY);
-  if (ord_pos == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_def= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_DEFAULT",
-                                                          MAX_FIELD_VARCHARLENGTH,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          1,
-                                                          "Default",
-                                                          OPEN_FRM_ONLY);
-  if (col_def == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *is_nullable= new(std::nothrow) drizzled::plugin::ColumnInfo("IS_NULLABLE",
-                                                              3,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Null",
-                                                              OPEN_FRM_ONLY);
-  if (is_nullable == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *data_type= new(std::nothrow) drizzled::plugin::ColumnInfo("DATA_TYPE",
-                                                            NAME_CHAR_LEN,
-                                                            DRIZZLE_TYPE_VARCHAR,
-                                                            0,
-                                                            0,
-                                                            "",
-                                                            OPEN_FRM_ONLY);
-  if (data_type == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *max_len= new(std::nothrow) drizzled::plugin::ColumnInfo("CHARACTER_MAXIMUM_LENGTH",
-                                                          MY_INT64_NUM_DECIMAL_DIGITS,
-                                                          DRIZZLE_TYPE_LONGLONG,
-                                                          0,
-                                                          (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                          "",
-                                                          OPEN_FRM_ONLY);
-  if (max_len == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *octet_len= new(std::nothrow) drizzled::plugin::ColumnInfo("CHARACTER_OCTET_LENGTH",
-                                                            MY_INT64_NUM_DECIMAL_DIGITS,
-                                                            DRIZZLE_TYPE_LONGLONG,
-                                                            0,
-                                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                            "",
-                                                            OPEN_FRM_ONLY);
-  if (octet_len == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *num_prec= new(std::nothrow) drizzled::plugin::ColumnInfo("NUMERIC_PRECISION",
-                                                           MY_INT64_NUM_DECIMAL_DIGITS,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                           "",
-                                                           OPEN_FRM_ONLY);
-  if (num_prec == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *num_scale= new(std::nothrow) drizzled::plugin::ColumnInfo("NUMERIC_SCALE",
-                                                            MY_INT64_NUM_DECIMAL_DIGITS,
-                                                            DRIZZLE_TYPE_LONGLONG,
-                                                            0,
-                                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                            "",
-                                                            OPEN_FRM_ONLY);
-  if (num_scale == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *char_set_name= new(std::nothrow) drizzled::plugin::ColumnInfo("CHARACTER_SET_NAME",
-                                                                64,
-                                                                DRIZZLE_TYPE_VARCHAR,
-                                                                0,
-                                                                1,
-                                                                "",
-                                                                OPEN_FRM_ONLY);
-  if (char_set_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *coll_name= new(std::nothrow) drizzled::plugin::ColumnInfo("COLLATION_NAME",
-                                                            64,
-                                                            DRIZZLE_TYPE_VARCHAR,
-                                                            0,
-                                                            1,
-                                                            "Collation",
-                                                            OPEN_FRM_ONLY);
-  if (coll_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_type= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_TYPE",
-                                                           65535,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Type",
-                                                           OPEN_FRM_ONLY);
-  if (col_type == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_key= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_KEY",
-                                                          3,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          0,
-                                                          "Key",
-                                                          OPEN_FRM_ONLY);
-  if (col_key == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *extra= new(std::nothrow) drizzled::plugin::ColumnInfo("EXTRA",
-                                                        27,
-                                                        DRIZZLE_TYPE_VARCHAR,
-                                                        0,
-                                                        0,
-                                                        "Extra",
-                                                        OPEN_FRM_ONLY);
-  if (extra == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *priv= new(std::nothrow) drizzled::plugin::ColumnInfo("PRIVILEGES",
-                                                       80,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "Privileges",
-                                                       OPEN_FRM_ONLY);
-  if (priv == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_comment= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_COMMENT",
-                                                              COLUMN_COMMENT_MAXLEN,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Comment",
-                                                              OPEN_FRM_ONLY);
-  if (col_comment == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *storage= new(std::nothrow) drizzled::plugin::ColumnInfo("STORAGE",
-                                                          8,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          0,
-                                                          "Storage",
-                                                          OPEN_FRM_ONLY);
-  if (storage == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *format= new(std::nothrow) drizzled::plugin::ColumnInfo("FORMAT",
-                                                         8,
-                                                         DRIZZLE_TYPE_VARCHAR,
-                                                         0,
-                                                         0,
-                                                         "Format",
-                                                         OPEN_FRM_ONLY);
-  if (format == NULL)
-  {
-    return true;
-  }
-
-  /*
-   * Add the columns to the vector.
-   */
-  cols.push_back(tab_cat);
-  cols.push_back(tab_sch);
-  cols.push_back(tab_name);
-  cols.push_back(col_name);
-  cols.push_back(ord_pos);
-  cols.push_back(col_def);
-  cols.push_back(is_nullable);
-  cols.push_back(data_type);
-  cols.push_back(max_len);
-  cols.push_back(octet_len);
-  cols.push_back(num_prec);
-  cols.push_back(num_scale);
-  cols.push_back(char_set_name);
-  cols.push_back(coll_name);
-  cols.push_back(col_type);
-  cols.push_back(col_key);
-  cols.push_back(extra);
-  cols.push_back(priv);
-  cols.push_back(col_comment);
-  cols.push_back(storage);
-  cols.push_back(format);
-
-  return false;
-}
-
-bool createKeyColUsageColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *cat= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_CATALOG",
-                                                      FN_REFLEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      1,
-                                                      "",
-                                                      OPEN_FULL_TABLE);
-  if (cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_SCHEMA",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "",
-                                                      OPEN_FULL_TABLE);
-  if (sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_NAME",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "",
-                                                       OPEN_FULL_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_cat= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_CATALOG",
-                                                          FN_REFLEN,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          1,
-                                                          "",
-                                                          OPEN_FULL_TABLE);
-  if (tab_cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_sch= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_SCHEMA",
-                                                          NAME_CHAR_LEN,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          0,
-                                                          "",
-                                                          OPEN_FULL_TABLE);
-  if (tab_sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (tab_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_name= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (col_name == NULL)
-  {
-    return true;
-  }
-  const drizzled::plugin::ColumnInfo *ord_pos= new(std::nothrow) drizzled::plugin::ColumnInfo("ORDINAL_POSITION",
-                                                          10,
-                                                          DRIZZLE_TYPE_LONGLONG,
-                                                          0,
-                                                          0,
-                                                          "",
-                                                          OPEN_FULL_TABLE);
-  if (ord_pos == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *pos_in_uniq= new(std::nothrow) drizzled::plugin::ColumnInfo("POSITION_IN_UNIQUE_CONSTRAINT",
-                                                              10,
-                                                              DRIZZLE_TYPE_LONGLONG,
-                                                              0,
-                                                              1,
-                                                              "",
-                                                              OPEN_FULL_TABLE);
-  if (pos_in_uniq == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ref_tab_sch= new(std::nothrow) drizzled::plugin::ColumnInfo("REFERENCED_TABLE_SCHEMA",
-                                                              NAME_CHAR_LEN,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              1,
-                                                              "",
-                                                              OPEN_FULL_TABLE);
-  if (ref_tab_sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ref_tab_name= new(std::nothrow) drizzled::plugin::ColumnInfo("REFERENCED_TABLE_NAME",
-                                                               NAME_CHAR_LEN,
-                                                               DRIZZLE_TYPE_VARCHAR,
-                                                               0,
-                                                               1,
-                                                               "",
-                                                               OPEN_FULL_TABLE);
-  if (ref_tab_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ref_col_name= new(std::nothrow) drizzled::plugin::ColumnInfo("REFERENCED_COLUMN_NAME",
-                                                               NAME_CHAR_LEN,
-                                                               DRIZZLE_TYPE_VARCHAR,
-                                                               0,
-                                                               1,
-                                                               "",
-                                                               OPEN_FULL_TABLE);
-  if (ref_col_name == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(cat);
-  cols.push_back(sch);
-  cols.push_back(name);
-  cols.push_back(tab_cat);
-  cols.push_back(tab_sch);
-  cols.push_back(tab_name);
-  cols.push_back(col_name);
-  cols.push_back(ord_pos);
-  cols.push_back(pos_in_uniq);
-  cols.push_back(ref_tab_sch);
-  cols.push_back(ref_tab_name);
-  cols.push_back(ref_col_name);
-
-  return false;
-}
-
-bool createOpenTabColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *db= new(std::nothrow) drizzled::plugin::ColumnInfo("Database",
-                                                     NAME_CHAR_LEN,
-                                                     DRIZZLE_TYPE_VARCHAR,
-                                                     0,
-                                                     0,
-                                                     "Database",
-                                                     SKIP_OPEN_TABLE);
-  if (db == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab= new(std::nothrow) drizzled::plugin::ColumnInfo("Table",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "Table",
-                                                      SKIP_OPEN_TABLE);
-  if (tab == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *in_use= new(std::nothrow) drizzled::plugin::ColumnInfo("In_use",
-                                                         1,
-                                                         DRIZZLE_TYPE_LONGLONG,
-                                                         0,
-                                                         0,
-                                                         "In_use",
-                                                         SKIP_OPEN_TABLE);
-  if (in_use == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *name_locked= new(std::nothrow) drizzled::plugin::ColumnInfo("Name_locked",
-                                                              4,
-                                                              DRIZZLE_TYPE_LONGLONG,
-                                                              0,
-                                                              0,
-                                                              "Name_locked",
-                                                              SKIP_OPEN_TABLE);
-  if (name_locked == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(db);
-  cols.push_back(tab);
-  cols.push_back(in_use);
-  cols.push_back(name_locked);
-
-  return false;
-}
-
-bool createModulesColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *name=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_NAME",
-                                                    NAME_CHAR_LEN,
-                                                    DRIZZLE_TYPE_VARCHAR,
-                                                    0,
-                                                    0,
-                                                    "Name",
-                                                    SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ver=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_VERSION",
-                                                   20,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   0,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (ver == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *aut=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_AUTHOR",
-                                                   NAME_CHAR_LEN,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   1,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (aut == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *is_builtin=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("IS_BUILTIN",
-                                                   3,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   0,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (is_builtin == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *library=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_LIBRARY",
-                                                   65535,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   1,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (library == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *descrip=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_DESCRIPTION",
-                                                   65535,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   1,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (descrip == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *lic=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_LICENSE",
-                                                   80,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   1,
-                                                   "License",
-                                                   SKIP_OPEN_TABLE);
-  if (lic == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(name);
-  cols.push_back(ver);
-  cols.push_back(aut);
-  cols.push_back(is_builtin);
-  cols.push_back(library);
-  cols.push_back(descrip);
-  cols.push_back(lic);
-
-  return false;
-}
-
-bool createPluginsColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *name=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("PLUGIN_NAME",
-                                                   NAME_CHAR_LEN,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   0,
-                                                   "Name",
-                                                   SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ptype=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("PLUGIN_TYPE",
-                                                   NAME_CHAR_LEN,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   0,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *is_active=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("IS_ACTIVE",
-                                                   3,
-                                                   DRIZZLE_TYPE_VARCHAR,
-                                                   0,
-                                                   0,
-                                                   "",
-                                                   SKIP_OPEN_TABLE);
-  if (is_active == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *mod_name=
-    new(std::nothrow) drizzled::plugin::ColumnInfo("MODULE_NAME",
-                                                    NAME_CHAR_LEN,
-                                                    DRIZZLE_TYPE_VARCHAR,
-                                                    0,
-                                                    0,
-                                                    "Name",
-                                                    SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-  cols.push_back(name);
-  cols.push_back(ptype);
-  cols.push_back(is_active);
-  cols.push_back(mod_name);
-
-  return false;
-}
-
-
-bool createProcessListColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  /*
-   * Create each column for the PROCESSLIST table.
-   */
-  const drizzled::plugin::ColumnInfo *id_col= new(std::nothrow) drizzled::plugin::ColumnInfo("ID", 
-                                                         4,
-                                                         DRIZZLE_TYPE_LONGLONG,
-                                                         0,
-                                                         0,
-                                                         "Id",
-                                                         SKIP_OPEN_TABLE);
-  if (id_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *user_col= new(std::nothrow) drizzled::plugin::ColumnInfo("USER",
-                                                           16,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "User",
-                                                           SKIP_OPEN_TABLE);
-  if (user_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *host_col= new(std::nothrow) drizzled::plugin::ColumnInfo("HOST",
-                                                           LIST_PROCESS_HOST_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Host",
-                                                           SKIP_OPEN_TABLE);
-  if (host_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *db_col= new(std::nothrow) drizzled::plugin::ColumnInfo("DB",
-                                                         NAME_CHAR_LEN,
-                                                         DRIZZLE_TYPE_VARCHAR,
-                                                         0,
-                                                         1,
-                                                         "Db",
-                                                         SKIP_OPEN_TABLE);
-  if (db_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *command_col= new(std::nothrow) drizzled::plugin::ColumnInfo("COMMAND",
-                                                              16,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Command",
-                                                              SKIP_OPEN_TABLE);
-  if (command_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *time_col= new(std::nothrow) drizzled::plugin::ColumnInfo("TIME",
-                                                           7,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           0,
-                                                           "Time",
-                                                           SKIP_OPEN_TABLE);
-  if (time_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *state_col= new(std::nothrow) drizzled::plugin::ColumnInfo("STATE",
-                                                            64,
-                                                            DRIZZLE_TYPE_VARCHAR,
-                                                            0,
-                                                            1,
-                                                            "State",
-                                                            SKIP_OPEN_TABLE);
-  if (state_col == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *info_col= new(std::nothrow) drizzled::plugin::ColumnInfo("INFO",
-                                                           PROCESS_LIST_INFO_WIDTH,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           1,
-                                                           "Info",
-                                                           SKIP_OPEN_TABLE);
-  if (info_col == NULL)
-  {
-    return true;
-  }
-
-  /*
-   * Add the columns to the vector.
-   */
-  cols.push_back(id_col);
-  cols.push_back(user_col);
-  cols.push_back(host_col);
-  cols.push_back(db_col);
-  cols.push_back(command_col);
-  cols.push_back(time_col);
-  cols.push_back(state_col);
-  cols.push_back(info_col);
-
-  return false;
-}
-
-bool createRefConstraintColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  /*
-   * Create the columns for the table.
-   */
-  const drizzled::plugin::ColumnInfo *cat= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_CATALOG",
-                                                      FN_REFLEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      1,
-                                                      "",
-                                                      OPEN_FULL_TABLE);
-
-  if (cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_SCHEMA",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "",
-                                                      OPEN_FULL_TABLE);
-  if (sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_NAME",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "",
-                                                       OPEN_FULL_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *uniq_cat= new(std::nothrow) drizzled::plugin::ColumnInfo("UNIQUE_CONSTRAINT_CATALOG",
-                                                           FN_REFLEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           1,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (uniq_cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *uniq_sch= new(std::nothrow) drizzled::plugin::ColumnInfo("UNIQUE_CONSTRAINT_SCHEMA",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (uniq_sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *uniq_name= new(std::nothrow) drizzled::plugin::ColumnInfo("UNIQUE_CONSTRAINT_NAME",
-                                                            NAME_CHAR_LEN,
-                                                            DRIZZLE_TYPE_VARCHAR,
-                                                            0,
-                                                            MY_I_S_MAYBE_NULL,
-                                                            "",
-                                                            OPEN_FULL_TABLE);
-  if (uniq_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *match= new(std::nothrow) drizzled::plugin::ColumnInfo("MATCH_OPTION",
-                                                        NAME_CHAR_LEN,
-                                                        DRIZZLE_TYPE_VARCHAR,
-                                                        0,
-                                                        0,
-                                                        "",
-                                                        OPEN_FULL_TABLE);
-  if (match == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *update= new(std::nothrow) drizzled::plugin::ColumnInfo("UPDATE_RULE",
-                                                         NAME_CHAR_LEN,
-                                                         DRIZZLE_TYPE_VARCHAR,
-                                                         0,
-                                                         0,
-                                                         "",
-                                                         OPEN_FULL_TABLE);
-  if (update == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *del_rule= new(std::nothrow) drizzled::plugin::ColumnInfo("DELETE_RULE",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (del_rule == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (tab_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *ref_name= new(std::nothrow) drizzled::plugin::ColumnInfo("REFERENCED_TABLE_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (ref_name == NULL)
-  {
-    return true;
-  }
-
-  /*
-   * Add the columns to the vector.
-   */
-  cols.push_back(cat);
-  cols.push_back(sch);
-  cols.push_back(name);
-  cols.push_back(uniq_cat);
-  cols.push_back(uniq_sch);
-  cols.push_back(uniq_name);
-  cols.push_back(match);
-  cols.push_back(update);
-  cols.push_back(del_rule);
-  cols.push_back(tab_name);
-  cols.push_back(ref_name);
-
-  return false;
-}
-
-bool createSchemataColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *cat_name= new(std::nothrow) drizzled::plugin::ColumnInfo("CATALOG_NAME",
-                                                           FN_REFLEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0, 
-                                                           1, 
-                                                           "", 
-                                                           SKIP_OPEN_TABLE);
-  if (cat_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch_name= new(std::nothrow) drizzled::plugin::ColumnInfo("SCHEMA_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0, 
-                                                           0, 
-                                                           "Database", 
-                                                           SKIP_OPEN_TABLE);
-  if (sch_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *cs_name= new(std::nothrow) drizzled::plugin::ColumnInfo("DEFAULT_CHARACTER_SET_NAME",
-                                                          64, 
-                                                          DRIZZLE_TYPE_VARCHAR, 
-                                                          0, 
-                                                          0, 
-                                                          "",
-                                                          SKIP_OPEN_TABLE);
-  if (cs_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *coll_name= new(std::nothrow) drizzled::plugin::ColumnInfo("DEFAULT_COLLATION_NAME",
-                                                            64, 
-                                                            DRIZZLE_TYPE_VARCHAR, 
-                                                            0, 
-                                                            0, 
-                                                            "",
-                                                            SKIP_OPEN_TABLE);
-  if (coll_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sql_path= new(std::nothrow) drizzled::plugin::ColumnInfo("SQL_PATH",
-                                                           FN_REFLEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0, 
-                                                           1, 
-                                                           "", 
-                                                           SKIP_OPEN_TABLE);
-  if (sql_path == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(cat_name);
-  cols.push_back(sch_name);
-  cols.push_back(cs_name);
-  cols.push_back(coll_name);
-  cols.push_back(sql_path);
-
-  return false;
-}
-
-bool createStatsColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *cat= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_CATALOG",
-                                                      FN_REFLEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      1,
-                                                      "",
-                                                      OPEN_FRM_ONLY);
-  if (cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_SCHEMA",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "",
-                                                      OPEN_FRM_ONLY);
-  if (sch == NULL)
-  {
-    return true;
-  }
-  
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "Table",
-                                                       OPEN_FRM_ONLY);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *uniq= new(std::nothrow) drizzled::plugin::ColumnInfo("NON_UNIQUE",
-                                                       1,
-                                                       DRIZZLE_TYPE_LONGLONG,
-                                                       0,
-                                                       0,
-                                                       "Non_unique",
-                                                       OPEN_FRM_ONLY);
-  if (uniq == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *idx_sch= new(std::nothrow) drizzled::plugin::ColumnInfo("INDEX_SCHEMA",
-                                                          NAME_CHAR_LEN,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          0,
-                                                          "",
-                                                          OPEN_FRM_ONLY);
-  if (idx_sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *idx_name= new(std::nothrow) drizzled::plugin::ColumnInfo("INDEX_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Key_name",
-                                                           OPEN_FRM_ONLY);
-  if (idx_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *seq_in_idx= new(std::nothrow) drizzled::plugin::ColumnInfo("SEQ_IN_INDEX",
-                                                             2,
-                                                             DRIZZLE_TYPE_LONGLONG,
-                                                             0,
-                                                             0,
-                                                             "Seq_in_index",
-                                                             OPEN_FRM_ONLY);
-  if (seq_in_idx == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *col_name= new(std::nothrow) drizzled::plugin::ColumnInfo("COLUMN_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Column_name",
-                                                           OPEN_FRM_ONLY);
-  if (col_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *coll= new(std::nothrow) drizzled::plugin::ColumnInfo("COLLATION",
-                                                       1,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       1,
-                                                       "Collation",
-                                                       OPEN_FRM_ONLY);
-  if (coll == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *card= new(std::nothrow) drizzled::plugin::ColumnInfo("CARDINALITY",
-                                                       MY_INT64_NUM_DECIMAL_DIGITS,
-                                                       DRIZZLE_TYPE_LONGLONG,
-                                                       0,
-                                                       1,
-                                                       "Cardinality",
-                                                       OPEN_FULL_TABLE);
-  if (card == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sub_part= new(std::nothrow) drizzled::plugin::ColumnInfo("SUB_PART",
-                                                           3,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           1,
-                                                           "Sub_part",
-                                                           OPEN_FRM_ONLY);
-  if (sub_part == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *packed= new(std::nothrow) drizzled::plugin::ColumnInfo("PACKED",
-                                                         10,
-                                                         DRIZZLE_TYPE_VARCHAR,
-                                                         0,
-                                                         1,
-                                                         "Packed",
-                                                         OPEN_FRM_ONLY);
-  if (packed == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *nullable= new(std::nothrow) drizzled::plugin::ColumnInfo("NULLABLE",
-                                                           3,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Null",
-                                                           OPEN_FRM_ONLY);
-  if (nullable == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *idx_type= new(std::nothrow) drizzled::plugin::ColumnInfo("INDEX_TYPE",
-                                                           16,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "Index_type",
-                                                           OPEN_FULL_TABLE);
-  if (idx_type == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *comment= new(std::nothrow) drizzled::plugin::ColumnInfo("COMMENT",
-                                                          16,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          1,
-                                                          "Comment",
-                                                          OPEN_FRM_ONLY);
-  if (comment == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *idx_comment= new(std::nothrow) drizzled::plugin::ColumnInfo("INDEX_COMMENT",
-                                                              INDEX_COMMENT_MAXLEN,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Index_Comment",
-                                                              OPEN_FRM_ONLY);
-  if (idx_comment == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(cat);
-  cols.push_back(sch);
-  cols.push_back(name);
-  cols.push_back(uniq);
-  cols.push_back(idx_sch);
-  cols.push_back(idx_name);
-  cols.push_back(seq_in_idx);
-  cols.push_back(col_name);
-  cols.push_back(coll);
-  cols.push_back(card);
-  cols.push_back(sub_part);
-  cols.push_back(packed);
-  cols.push_back(nullable);
-  cols.push_back(idx_type);
-  cols.push_back(comment);
-  cols.push_back(idx_comment);
-
-  return false;
-}
-
-bool createStatusColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("VARIABLE_NAME",
-                                                       64,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "Variable_name",
-                                                       SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *value= new(std::nothrow) drizzled::plugin::ColumnInfo("VARIABLE_VALUE",
-                                                        16300,
-                                                        DRIZZLE_TYPE_VARCHAR,
-                                                        0,
-                                                        1,
-                                                        "Value",
-                                                        SKIP_OPEN_TABLE);
-  if (value == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(name);
-  cols.push_back(value);
-
-  return false;
-}
-
-bool createTabConstraintsColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *cat= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_CATALOG",
-                                                      FN_REFLEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      1,
-                                                      "",
-                                                      OPEN_FULL_TABLE);
-  if (cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_SCHEMA",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "",
-                                                      OPEN_FULL_TABLE);
-  if (sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_NAME",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "",
-                                                       OPEN_FULL_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_sch= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_SCHEMA",
-                                                          NAME_CHAR_LEN,
-                                                          DRIZZLE_TYPE_VARCHAR,
-                                                          0,
-                                                          0,
-                                                          "",
-                                                          OPEN_FULL_TABLE);
-  if (tab_sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                           NAME_CHAR_LEN,
-                                                           DRIZZLE_TYPE_VARCHAR,
-                                                           0,
-                                                           0,
-                                                           "",
-                                                           OPEN_FULL_TABLE);
-  if (tab_name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *type= new(std::nothrow) drizzled::plugin::ColumnInfo("CONSTRAINT_TYPE",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "",
-                                                       OPEN_FULL_TABLE);
-  if (type == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(cat);
-  cols.push_back(sch);
-  cols.push_back(name);
-  cols.push_back(tab_sch);
-  cols.push_back(tab_name);
-  cols.push_back(type);
-
-  return false;
-}
-
-bool createTablesColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *cat= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_CATALOG",
-                                                      FN_REFLEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      1,
-                                                      "",
-                                                      SKIP_OPEN_TABLE);
-  if (cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_SCHEMA",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "",
-                                                      SKIP_OPEN_TABLE);
-  if (sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "Name",
-                                                       SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *type= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_TYPE",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "",
-                                                       OPEN_FRM_ONLY);
-  if (type == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *engine= new(std::nothrow) drizzled::plugin::ColumnInfo("ENGINE",
-                                                         NAME_CHAR_LEN,
-                                                         DRIZZLE_TYPE_VARCHAR,
-                                                         0,
-                                                         1,
-                                                         "Engine",
-                                                         OPEN_FRM_ONLY);
-  if (engine == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *version= new(std::nothrow) drizzled::plugin::ColumnInfo("VERSION",
-                                                          MY_INT64_NUM_DECIMAL_DIGITS,
-                                                          DRIZZLE_TYPE_LONGLONG,
-                                                          0,
-                                                          (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                          "Version",
-                                                          OPEN_FRM_ONLY);
-  if (version == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *row_format= new(std::nothrow) drizzled::plugin::ColumnInfo("ROW_FORMAT",
-                                                             10,
-                                                             DRIZZLE_TYPE_VARCHAR,
-                                                             0,
-                                                             1,
-                                                             "Row_format",
-                                                             OPEN_FULL_TABLE);
-  if (row_format == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_rows= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_ROWS",
-                                                           MY_INT64_NUM_DECIMAL_DIGITS,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                           "Rows",
-                                                           OPEN_FULL_TABLE);
-  if (tab_rows == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *avg_row_len= new(std::nothrow) drizzled::plugin::ColumnInfo("AVG_ROW_LENGTH",
-                                                              MY_INT64_NUM_DECIMAL_DIGITS,
-                                                              DRIZZLE_TYPE_LONGLONG,
-                                                              0,
-                                                              (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                              "Avg_row_length",
-                                                              OPEN_FULL_TABLE);
-  if (avg_row_len == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *data_len= new(std::nothrow) drizzled::plugin::ColumnInfo("DATA_LENGTH",
-                                                           MY_INT64_NUM_DECIMAL_DIGITS,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                           "Data_length",
-                                                           OPEN_FULL_TABLE);
-  if (data_len == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *max_data_len= new(std::nothrow) drizzled::plugin::ColumnInfo("MAX_DATA_LENGTH",
-                                                               MY_INT64_NUM_DECIMAL_DIGITS,
-                                                               DRIZZLE_TYPE_LONGLONG,
-                                                               0,
-                                                               (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                               "Max_data_length",
-                                                               OPEN_FULL_TABLE);
-  if (max_data_len == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *idx_len= new(std::nothrow) drizzled::plugin::ColumnInfo("INDEX_LENGTH",
-                                                          MY_INT64_NUM_DECIMAL_DIGITS,
-                                                          DRIZZLE_TYPE_LONGLONG,
-                                                          0,
-                                                          (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                          "Index_length",
-                                                          OPEN_FULL_TABLE);
-  if (idx_len == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *data_free= new(std::nothrow) drizzled::plugin::ColumnInfo("DATA_FREE",
-                                                           MY_INT64_NUM_DECIMAL_DIGITS,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                           "Data_free",
-                                                           OPEN_FULL_TABLE);
-  if (data_free == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *auto_inc= new(std::nothrow) drizzled::plugin::ColumnInfo("AUTO_INCREMENT",
-                                                           MY_INT64_NUM_DECIMAL_DIGITS,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                           "Auto_increment",
-                                                           OPEN_FULL_TABLE);
-  if (auto_inc == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *create_time= new(std::nothrow) drizzled::plugin::ColumnInfo("CREATE_TIME",
-                                                              0,
-                                                              DRIZZLE_TYPE_DATETIME,
-                                                              0,
-                                                              1,
-                                                              "Create_time",
-                                                              OPEN_FULL_TABLE);
-  if (create_time == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *update_time= new(std::nothrow) drizzled::plugin::ColumnInfo("UPDATE_TIME",
-                                                              0,
-                                                              DRIZZLE_TYPE_DATETIME,
-                                                              0,
-                                                              1,
-                                                              "Update_time",
-                                                              OPEN_FULL_TABLE);
-  if (update_time == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *check_time= new(std::nothrow) drizzled::plugin::ColumnInfo("CHECK_TIME",
-                                                             0,
-                                                             DRIZZLE_TYPE_DATETIME,
-                                                             0,
-                                                             1,
-                                                             "Check_time",
-                                                             OPEN_FULL_TABLE);
-  if (check_time == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *table_coll= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_COLLATION",
-                                                             64,
-                                                             DRIZZLE_TYPE_VARCHAR,
-                                                             0,
-                                                             1,
-                                                             "Collation",
-                                                             OPEN_FRM_ONLY);
-  if (table_coll == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *checksum= new(std::nothrow) drizzled::plugin::ColumnInfo("CHECKSUM",
-                                                           MY_INT64_NUM_DECIMAL_DIGITS,
-                                                           DRIZZLE_TYPE_LONGLONG,
-                                                           0,
-                                                           (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
-                                                           "Checksum",
-                                                           OPEN_FULL_TABLE);
-  if (checksum == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *create_opt= new(std::nothrow) drizzled::plugin::ColumnInfo("CREATE_OPTIONS",
-                                                             255,
-                                                             DRIZZLE_TYPE_VARCHAR,
-                                                             0,
-                                                             1,
-                                                             "Create_options",
-                                                             OPEN_FRM_ONLY);
-  if (create_opt == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *tab_comment= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_COMMENT",
-                                                              TABLE_COMMENT_MAXLEN,
-                                                              DRIZZLE_TYPE_VARCHAR,
-                                                              0,
-                                                              0,
-                                                              "Comment",
-                                                              OPEN_FRM_ONLY);
-  if (tab_comment == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(cat);
-  cols.push_back(sch);
-  cols.push_back(name);
-  cols.push_back(type);
-  cols.push_back(engine);
-  cols.push_back(version);
-  cols.push_back(row_format);
-  cols.push_back(tab_rows);
-  cols.push_back(avg_row_len);
-  cols.push_back(data_len);
-  cols.push_back(max_data_len);
-  cols.push_back(idx_len);
-  cols.push_back(data_free);
-  cols.push_back(auto_inc);
-  cols.push_back(create_time);
-  cols.push_back(update_time);
-  cols.push_back(check_time);
-  cols.push_back(table_coll);
-  cols.push_back(checksum);
-  cols.push_back(create_opt);
-  cols.push_back(tab_comment);
-
-  return false;
-}
-
-
-bool createTabNamesColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  const drizzled::plugin::ColumnInfo *cat= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_CATALOG",
-                                                      FN_REFLEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      1,
-                                                      "",
-                                                      SKIP_OPEN_TABLE);
-  if (cat == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *sch= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_SCHEMA",
-                                                      NAME_CHAR_LEN,
-                                                      DRIZZLE_TYPE_VARCHAR,
-                                                      0,
-                                                      0,
-                                                      "",
-                                                      SKIP_OPEN_TABLE);
-  if (sch == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *name= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_NAME",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "Tables_in_",
-                                                       SKIP_OPEN_TABLE);
-  if (name == NULL)
-  {
-    return true;
-  }
-
-  const drizzled::plugin::ColumnInfo *type= new(std::nothrow) drizzled::plugin::ColumnInfo("TABLE_TYPE",
-                                                       NAME_CHAR_LEN,
-                                                       DRIZZLE_TYPE_VARCHAR,
-                                                       0,
-                                                       0,
-                                                       "Table_type",
-                                                       OPEN_FRM_ONLY);
-  if (type == NULL)
-  {
-    return true;
-  }
-
-  cols.push_back(cat);
-  cols.push_back(sch);
-  cols.push_back(name);
-  cols.push_back(type);
-
-  return false;
-}
-
-/*
- * Function object used for deleting the memory allocated
- * for the columns contained with the vector of columns.
- */
-class DeleteColumns
-{
-public:
-  template<typename T>
-  inline void operator()(const T *ptr) const
-  {
-    delete ptr;
-  }
-};
-
-void clearColumns(vector<const drizzled::plugin::ColumnInfo *>& cols)
-{
-  for_each(cols.begin(), cols.end(), DeleteColumns());
-  cols.clear();
-}

=== removed file 'plugin/info_schema/info_schema_columns.h'
--- plugin/info_schema/info_schema_columns.h	2009-10-27 00:04:25 +0000
+++ plugin/info_schema/info_schema_columns.h	1970-01-01 00:00:00 +0000
@@ -1,186 +0,0 @@
-/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
- *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
- *
- *  Copyright (C) 2009 Sun Microsystems
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- */
-
-#ifndef PLUGIN_INFO_SCHEMA_INFO_SCHEMA_COLUMNS_H
-#define PLUGIN_INFO_SCHEMA_INFO_SCHEMA_COLUMNS_H
-
-#include "drizzled/plugin/info_schema_table.h"
-
-#include <vector>
-
-/**
- * Create the various columns for the CHARACTER_SET I_S table and add them
- * to the std::vector of columns for the CHARACTER_SET table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createCharSetColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the Collations I_S table and add them
- * to the std::vector of columns for the Collations table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createCollationColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the character set applicability
- * I_S table and add them to the std::vector of columns for the
- * table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createCollCharSetColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the COLUMNS
- * I_S table and add them to the std::vector of columns for the
- * table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createColColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the key column usage
- * I_S table and add them to the std::vector of columns for the
- * table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createKeyColUsageColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the OPEN_TABLES
- * I_S table and add them to the std::vector of columns for the
- * table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createOpenTabColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the PLUGINS
- * I_S table and add them to the std::vector of columns for the
- * table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createPluginsColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various columns for the MODULES
- * I_S table and add them to the std::vector of columns for the
- * table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createModulesColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the PROCESSLIST I_S table and add them
- * to the std::vector of columns for the PROCESSLIST table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createProcessListColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the REFERENTIAL_CONSTRAINTS I_S table 
- * and add them to the std::vector of columns for this table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createRefConstraintColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the SCHEMATA I_S table 
- * and add them to the std::vector of columns for this table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createSchemataColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the STATISTICS I_S table 
- * and add them to the std::vector of columns for this table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createStatsColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for numerous I_S tables 
- * and add them to the std::vector of columns for these table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createStatusColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the TABLE_CONSTRAINTS I_S table 
- * and add them to the std::vector of columns for this table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createTabConstraintsColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the TABLE_NAMES I_S table 
- * and add them to the std::vector of columns for this table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createTablesColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Create the various volumns for the TABLE_NAMES I_S table 
- * and add them to the std::vector of columns for this table.
- *
- * @param[out] cols vector to add columns to
- * @return false on success; true on failure
- */
-bool createTabNamesColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-/**
- * Iterate through the given vector of columns and delete the memory that
- * has been allocated for them.
- *
- * @param[out] cols vector to clear and de-allocate memory from
- */
-void clearColumns(std::vector<const drizzled::plugin::ColumnInfo *>& cols);
-
-#endif /* PLUGIN_INFO_SCHEMA_INFO_SCHEMA_COLUMNS_H */

=== added file 'plugin/info_schema/key_column_usage.cc'
--- plugin/info_schema/key_column_usage.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/key_column_usage.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,297 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   key column usage I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "key_column_usage.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the key column usage I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the key column usage I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * key column usage I_S table.
+ */
+static plugin::InfoSchemaTable *key_col_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *KeyColumnUsageIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("ORDINAL_POSITION",
+                                            10,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("POSITION_IN_UNIQUE_CONSTRAINT",
+                                            10,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("REFERENCED_TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("REFERENCED_TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("REFERENCED_COLUMN_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *KeyColumnUsageIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new KeyColUsageISMethods();
+  }
+
+  if (key_col_table == NULL)
+  {
+    key_col_table= new plugin::InfoSchemaTable("KEY_COLUMN_USAGE",
+                                               *columns,
+                                               4, 5, false, true,
+                                               OPEN_TABLE_ONLY,
+                                               methods);
+  }
+
+  return key_col_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void KeyColumnUsageIS::cleanup()
+{
+  clearColumns(*columns);
+  delete key_col_table;
+  delete methods;
+  delete columns;
+}
+
+int KeyColUsageISMethods::processTable(Session *session,
+                                       TableList *tables,
+                                       Table *table, bool res,
+                                       LEX_STRING *db_name,
+                                       LEX_STRING *table_name) const
+{
+  if (res)
+  {
+    if (session->is_error())
+      push_warning(session, DRIZZLE_ERROR::WARN_LEVEL_WARN,
+                   session->main_da.sql_errno(), session->main_da.message());
+    session->clear_error();
+    return(0);
+  }
+  else
+  {
+    List<FOREIGN_KEY_INFO> f_key_list;
+    Table *show_table= tables->table;
+    KEY *key_info=show_table->key_info;
+    uint32_t primary_key= show_table->s->primary_key;
+    show_table->cursor->info(HA_STATUS_VARIABLE |
+                             HA_STATUS_NO_LOCK |
+                             HA_STATUS_TIME);
+    for (uint32_t i=0 ; i < show_table->s->keys ; i++, key_info++)
+    {
+      if (i != primary_key && !(key_info->flags & HA_NOSAME))
+      {
+        continue;
+      }
+      uint32_t f_idx= 0;
+      KEY_PART_INFO *key_part= key_info->key_part;
+      for (uint32_t j=0 ; j < key_info->key_parts ; j++,key_part++)
+      {
+        if (key_part->field)
+        {
+          f_idx++;
+          table->restoreRecordAsDefault();
+          store_key_column_usage(table, db_name, table_name,
+                                 key_info->name,
+                                 strlen(key_info->name),
+                                 key_part->field->field_name,
+                                 strlen(key_part->field->field_name),
+                                 (int64_t) f_idx);
+          if (schema_table_store_record(session, table))
+          {
+            return (1);
+          }
+        }
+      }
+    }
+
+    show_table->cursor->get_foreign_key_list(session, &f_key_list);
+    FOREIGN_KEY_INFO *f_key_info;
+    List_iterator_fast<FOREIGN_KEY_INFO> fkey_it(f_key_list);
+    while ((f_key_info= fkey_it++))
+    {
+      LEX_STRING *f_info;
+      LEX_STRING *r_info;
+      List_iterator_fast<LEX_STRING> it(f_key_info->foreign_fields),
+        it1(f_key_info->referenced_fields);
+      uint32_t f_idx= 0;
+      while ((f_info= it++))
+      {
+        r_info= it1++;
+        f_idx++;
+        table->restoreRecordAsDefault();
+        store_key_column_usage(table, db_name, table_name,
+                               f_key_info->forein_id->str,
+                               f_key_info->forein_id->length,
+                               f_info->str, f_info->length,
+                               (int64_t) f_idx);
+        table->field[8]->store((int64_t) f_idx, true);
+        table->field[8]->set_notnull();
+        table->field[9]->store(f_key_info->referenced_db->str,
+                               f_key_info->referenced_db->length,
+                               system_charset_info);
+        table->field[9]->set_notnull();
+        table->field[10]->store(f_key_info->referenced_table->str,
+                                f_key_info->referenced_table->length,
+                                system_charset_info);
+        table->field[10]->set_notnull();
+        table->field[11]->store(r_info->str, r_info->length,
+                                system_charset_info);
+        table->field[11]->set_notnull();
+        if (schema_table_store_record(session, table))
+        {
+          return (1);
+        }
+      }
+    }
+  }
+  return (res);
+}

=== added file 'plugin/info_schema/key_column_usage.h'
--- plugin/info_schema/key_column_usage.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/key_column_usage.h	2009-11-16 18:35:38 +0000
@@ -0,0 +1,55 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_KEY_COLUMN_USAGE_H
+#define PLUGIN_INFO_SCHEMA_KEY_COLUMN_USAGE_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class KeyColUsageISMethods
+ * @brief
+ *   Class which implements any methods that the key column usage
+ *   I_S table needs besides the default methods
+ */
+class KeyColUsageISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int processTable(Session *session, TableList *tables,
+                           Table *table, bool res, LEX_STRING *db_name,
+                           LEX_STRING *table_name) const;
+};
+
+class KeyColumnUsageIS
+{
+  /**
+   * Create the various columns for the KEY_COLUMN_USAGE I_S table and add them
+   * to the std::vector of columns for the KEY_COLUMN_USAGE table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_KEY_COLUMN_USAGE_H */

=== added file 'plugin/info_schema/modules.cc'
--- plugin/info_schema/modules.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/modules.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,259 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Modules I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "modules.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the modules I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the modules I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * modules I_S table.
+ */
+static plugin::InfoSchemaTable *mods_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *ModulesIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Name",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_VERSION",
+                                            20,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_AUTHOR",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("IS_BUILTIN",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_LIBRARY",
+                                            65535,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_DESCRIPTION",
+                                            65535,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_LICENSE",
+                                            80,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "License",
+                                            SKIP_OPEN_TABLE));
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *ModulesIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new ModulesISMethods();
+  }
+
+  if (mods_table == NULL)
+  {
+    mods_table= new plugin::InfoSchemaTable("MODULES",
+                                            *columns,
+                                            -1, -1, false, false, 0,
+                                            methods);
+  }
+
+  return mods_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void ModulesIS::cleanup()
+{
+  clearColumns(*columns);
+  delete mods_table;
+  delete methods;
+  delete columns;
+}
+
+class ShowModules : public unary_function<drizzled::plugin::Module *, bool>
+{
+  Session *session;
+  Table *table;
+public:
+  ShowModules(Session *session_arg, Table *table_arg)
+    : session(session_arg), table(table_arg) {}
+
+  result_type operator() (argument_type module)
+  {
+    const drizzled::plugin::Manifest &manifest= module->getManifest();
+    const CHARSET_INFO * const cs= system_charset_info;
+
+    table->restoreRecordAsDefault();
+
+    table->field[0]->store(module->getName().c_str(),
+                           module->getName().size(), cs);
+
+    if (manifest.version)
+    {
+      table->field[1]->store(manifest.version, strlen(manifest.version), cs);
+      table->field[1]->set_notnull();
+    }
+    else
+      table->field[1]->set_null();
+
+    if (manifest.author)
+    {
+      table->field[2]->store(manifest.author, strlen(manifest.author), cs);
+      table->field[2]->set_notnull();
+    }
+    else
+    {
+      table->field[2]->set_null();
+    }
+
+    if (module->plugin_dl == NULL)
+    {
+      table->field[3]->store(STRING_WITH_LEN("YES"),cs);
+      table->field[4]->set_null();
+    }
+    else
+    {
+      table->field[3]->store(STRING_WITH_LEN("NO"),cs);
+      table->field[4]->store(module->plugin_dl->dl.str,
+                             module->plugin_dl->dl.length, cs);
+    }
+
+    if (manifest.descr)
+    {
+      table->field[5]->store(manifest.descr, strlen(manifest.descr), cs);
+      table->field[5]->set_notnull();
+    }
+    else
+    {
+      table->field[5]->set_null();
+    }
+
+    switch (manifest.license) {
+    case PLUGIN_LICENSE_GPL:
+      table->field[6]->store(drizzled::plugin::LICENSE_GPL_STRING.c_str(),
+                             drizzled::plugin::LICENSE_GPL_STRING.size(), cs);
+      break;
+    case PLUGIN_LICENSE_BSD:
+      table->field[6]->store(drizzled::plugin::LICENSE_BSD_STRING.c_str(),
+                             drizzled::plugin::LICENSE_BSD_STRING.size(), cs);
+      break;
+    case PLUGIN_LICENSE_LGPL:
+      table->field[6]->store(drizzled::plugin::LICENSE_LGPL_STRING.c_str(),
+                             drizzled::plugin::LICENSE_LGPL_STRING.size(), cs);
+      break;
+    default:
+      table->field[6]->store(drizzled::plugin::LICENSE_PROPRIETARY_STRING.c_str(),
+                             drizzled::plugin::LICENSE_PROPRIETARY_STRING.size(),
+                             cs);
+      break;
+    }
+    table->field[6]->set_notnull();
+
+    return schema_table_store_record(session, table);
+  }
+};
+
+int ModulesISMethods::fillTable(Session *session, TableList *tables)
+{
+  Table *table= tables->table;
+
+  drizzled::plugin::Registry &registry= drizzled::plugin::Registry::singleton();
+  vector<drizzled::plugin::Module *> modules= registry.getList(true);
+  vector<drizzled::plugin::Module *>::iterator iter=
+    find_if(modules.begin(), modules.end(), ShowModules(session, table));
+  if (iter != modules.end())
+  {
+    return 1;
+  }
+  return 0;
+}

=== added file 'plugin/info_schema/modules.h'
--- plugin/info_schema/modules.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/modules.h	2009-11-17 04:45:13 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_MODULES_H
+#define PLUGIN_INFO_SCHEMA_MODULES_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class ModulesISMethods
+ * @brief
+ *   Class which implements any methods that the MODULES
+ *   I_S table needs besides the default methods
+ */
+class ModulesISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+};
+
+class ModulesIS
+{
+  /**
+   * Create the various columns for the MODULES I_S table and add them
+   * to the std::vector of columns for the MODULES table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_MODULES_H */

=== added file 'plugin/info_schema/open_tables.cc'
--- plugin/info_schema/open_tables.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/open_tables.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,163 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Open tables I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "open_tables.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the open tables I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the open tables I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * open tables I_S table.
+ */
+static plugin::InfoSchemaTable *open_tabs_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *OpenTablesIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("Database",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Database",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("Table",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Table",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("In_use",
+                                            1,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "In_use",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("Name_locked",
+                                            4,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Name_locked",
+                                            SKIP_OPEN_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *OpenTablesIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new OpenTablesISMethods();
+  }
+
+  if (open_tabs_table == NULL)
+  {
+    open_tabs_table= new plugin::InfoSchemaTable("OPEN_TABLES",
+                                                 *columns,
+                                                 -1, -1, true, false, 0,
+                                                 methods);
+  }
+
+  return open_tabs_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void OpenTablesIS::cleanup()
+{
+  clearColumns(*columns);
+  delete open_tabs_table;
+  delete methods;
+  delete columns;
+}
+
+inline bool open_list_store(Table *table, open_table_list_st& open_list);
+inline bool open_list_store(Table *table, open_table_list_st& open_list)
+{
+  table->restoreRecordAsDefault();
+  table->field[0]->store(open_list.db.c_str(), open_list.db.length(), system_charset_info);
+  table->field[1]->store(open_list.table.c_str(), open_list.table.length(), system_charset_info);
+  table->field[2]->store((int64_t) open_list.in_use, true);
+  table->field[3]->store((int64_t) open_list.locked, true);
+  if (schema_table_store_record(table->in_use, table))
+    return true;
+
+  return false;
+}
+
+int OpenTablesISMethods::fillTable(Session *session, TableList *tables)
+{
+  const char *wild= session->lex->wild ? session->lex->wild->ptr() : NULL;
+
+  if ((list_open_tables(session->lex->select_lex.db, wild, open_list_store, tables->table) == true) && session->is_fatal_error)
+    return 1;
+
+  return 0;
+}
+

=== added file 'plugin/info_schema/open_tables.h'
--- plugin/info_schema/open_tables.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/open_tables.h	2009-11-17 05:51:20 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_OPEN_TABLES_H
+#define PLUGIN_INFO_SCHEMA_OPEN_TABLES_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class OpenTablesISMethods
+ * @brief
+ *   Class which implements any methods that the OPEN_TABLES
+ *   I_S table needs besides the default methods
+ */
+class OpenTablesISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+};
+
+class OpenTablesIS
+{
+  /**
+   * Create the various columns for the OPEN_TABLES I_S table and add them
+   * to the std::vector of columns for the OPEN_TABLES table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_OPEN_TABLES_H */

=== modified file 'plugin/info_schema/plugin.ini'
--- plugin/info_schema/plugin.ini	2009-11-10 20:47:10 +0000
+++ plugin/info_schema/plugin.ini	2009-11-18 16:39:59 +0000
@@ -2,7 +2,42 @@
 name=info_schema
 title=I_S Plugin
 description=Plugin which contains all the original I_S tables
-sources=info_schema.cc info_schema_methods.cc info_schema_columns.cc
-headers=info_schema_methods.h info_schema_columns.h
+sources=character_set.cc 
+  collation.cc
+  collation_char_set.cc
+  columns.cc
+  key_column_usage.cc
+  helper_methods.cc 
+  info_schema.cc 
+  modules.cc
+  open_tables.cc
+  plugins.cc
+  processlist.cc
+  referential_constraints.cc
+  schemata.cc
+  table_constraints.cc
+  tables.cc
+  table_names.cc
+  statistics.cc
+  status.cc
+  variables.cc
+headers=character_set.h 
+  collation.h
+  collation_char_set.h
+  columns.h
+  key_column_usage.h
+  helper_methods.h 
+  modules.h
+  open_tables.h
+  plugins.h
+  processlist.h
+  referential_constraints.h
+  schemata.h
+  table_constraints.h
+  tables.h
+  table_names.h
+  statistics.h
+  status.h
+  variables.h
 load_by_default=yes
 static=yes

=== added file 'plugin/info_schema/plugins.cc'
--- plugin/info_schema/plugins.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/plugins.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,190 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   Plugins I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "plugins.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the plugins I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the plugins I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * plugins I_S table.
+ */
+static plugin::InfoSchemaTable *plugins_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *PluginsIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("PLUGIN_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Name",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("PLUGIN_TYPE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("IS_ACTIVE",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MODULE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Name",
+                                            SKIP_OPEN_TABLE));
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *PluginsIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new PluginsISMethods();
+  }
+
+  if (plugins_table == NULL)
+  {
+    plugins_table= new plugin::InfoSchemaTable("PLUGINS",
+                                               *columns,
+                                               -1, -1, false, false, 0,
+                                               methods);
+  }
+
+  return plugins_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void PluginsIS::cleanup()
+{
+  clearColumns(*columns);
+  delete plugins_table;
+  delete methods;
+  delete columns;
+}
+
+class ShowPlugins
+ : public unary_function<pair<const string, const drizzled::plugin::Plugin *>, bool>
+{
+  Session *session;
+  Table *table;
+public:
+  ShowPlugins(Session *session_arg, Table *table_arg)
+    : session(session_arg), table(table_arg) {}
+
+  result_type operator() (argument_type plugin)
+  {
+    const CHARSET_INFO * const cs= system_charset_info;
+
+    table->restoreRecordAsDefault();
+
+    table->field[0]->store(plugin.first.c_str(),
+                           plugin.first.size(), cs);
+
+    table->field[1]->store(plugin.second->getTypeName().c_str(),
+                           plugin.second->getTypeName().size(), cs);
+
+    if (plugin.second->isActive())
+    {
+      table->field[2]->store(STRING_WITH_LEN("YES"),cs);
+    }
+    else
+    {
+      table->field[2]->store(STRING_WITH_LEN("NO"), cs);
+    }
+
+    table->field[3]->store(plugin.second->getModuleName().c_str(),
+                           plugin.second->getModuleName().size(), cs);
+
+    return schema_table_store_record(session, table);
+  }
+};
+
+int PluginsISMethods::fillTable(Session *session, TableList *tables)
+{
+  Table *table= tables->table;
+
+  drizzled::plugin::Registry &registry= drizzled::plugin::Registry::singleton();
+  const map<string, const drizzled::plugin::Plugin *> &plugin_map=
+    registry.getPluginsMap();
+  map<string, const drizzled::plugin::Plugin *>::const_iterator iter=
+    find_if(plugin_map.begin(), plugin_map.end(), ShowPlugins(session, table));
+  if (iter != plugin_map.end())
+  {
+    return 1;
+  }
+  return (0);
+}

=== added file 'plugin/info_schema/plugins.h'
--- plugin/info_schema/plugins.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/plugins.h	2009-11-17 05:28:24 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_PLUGINS_H
+#define PLUGIN_INFO_SCHEMA_PLUGINS_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class PluginsISMethods
+ * @brief
+ *   Class which implements any methods that the PLUGINS
+ *   I_S table needs besides the default methods
+ */
+class PluginsISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+};
+
+class PluginsIS
+{
+  /**
+   * Create the various columns for the PLUGINS I_S table and add them
+   * to the std::vector of columns for the PLUGINS table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_PLUGINS_H */

=== added file 'plugin/info_schema/processlist.cc'
--- plugin/info_schema/processlist.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/processlist.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,268 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   proceslist I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/plugin/client.h"
+
+#include "helper_methods.h"
+#include "processlist.h"
+
+#include <vector>
+
+#define LIST_PROCESS_HOST_LEN 64
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the processlist I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the processlist I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * processlist I_S table.
+ */
+static plugin::InfoSchemaTable *pl_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *ProcessListIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  /*
+   * Create each column for the PROCESSLIST table.
+   */
+  columns->push_back(new plugin::ColumnInfo("ID", 
+                                            4,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Id",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("USER",
+                                            16,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "User",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("HOST",
+                                            LIST_PROCESS_HOST_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Host",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DB",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Db",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("COMMAND",
+                                            16,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Command",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TIME",
+                                            7,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Time",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("STATE",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "State",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("INFO",
+                                            PROCESS_LIST_INFO_WIDTH,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Info",
+                                            SKIP_OPEN_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *ProcessListIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new ProcessListISMethods();
+  }
+
+  if (pl_table == NULL)
+  {
+    pl_table= new plugin::InfoSchemaTable("PROCESSLIST",
+                                          *columns,
+                                          -1, -1, false, false, 0,
+                                          methods);
+  }
+
+  return pl_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void ProcessListIS::cleanup()
+{
+  clearColumns(*columns);
+  delete pl_table;
+  delete methods;
+  delete columns;
+}
+
+int ProcessListISMethods::fillTable(Session* session, TableList* tables)
+{
+  Table *table= tables->table;
+  const CHARSET_INFO * const cs= system_charset_info;
+  time_t now= time(NULL);
+  size_t length;
+
+  if (now == (time_t)-1)
+    return 1;
+
+  pthread_mutex_lock(&LOCK_thread_count);
+
+  if (! session->killed)
+  {
+    Session* tmp;
+
+    for (vector<Session*>::iterator it= session_list.begin(); it != session_list.end(); ++it)
+    {
+      tmp= *it;
+      Security_context *tmp_sctx= &tmp->security_ctx;
+      struct st_my_thread_var *mysys_var;
+      const char *val;
+
+      if (! tmp->client->isConnected())
+        continue;
+
+      table->restoreRecordAsDefault();
+      /* ID */
+      table->field[0]->store((int64_t) tmp->thread_id, true);
+      /* USER */
+      val= tmp_sctx->user.c_str() ? tmp_sctx->user.c_str() : "unauthenticated user";
+      table->field[1]->store(val, strlen(val), cs);
+      /* HOST */
+      table->field[2]->store(tmp_sctx->ip.c_str(), strlen(tmp_sctx->ip.c_str()), cs);
+      /* DB */
+      if (! tmp->db.empty())
+      {
+        table->field[3]->store(tmp->db.c_str(), tmp->db.length(), cs);
+        table->field[3]->set_notnull();
+      }
+
+      if ((mysys_var= tmp->mysys_var))
+        pthread_mutex_lock(&mysys_var->mutex);
+      /* COMMAND */
+      if ((val= (char *) (tmp->killed == Session::KILL_CONNECTION? "Killed" : 0)))
+        table->field[4]->store(val, strlen(val), cs);
+      else
+        table->field[4]->store(command_name[tmp->command].str,
+                               command_name[tmp->command].length, cs);
+      /* DRIZZLE_TIME */
+      table->field[5]->store((uint32_t)(tmp->start_time ?
+                                      now - tmp->start_time : 0), true);
+      /* STATE */
+      val= (char*) (tmp->client->isWriting() ?
+                    "Writing to net" :
+                    tmp->client->isReading() ?
+                    (tmp->command == COM_SLEEP ?
+                     NULL : "Reading from net") :
+                    tmp->get_proc_info() ? tmp->get_proc_info() :
+                    tmp->mysys_var &&
+                    tmp->mysys_var->current_cond ?
+                    "Waiting on cond" : NULL);
+      if (val)
+      {
+        table->field[6]->store(val, strlen(val), cs);
+        table->field[6]->set_notnull();
+      }
+
+      if (mysys_var)
+        pthread_mutex_unlock(&mysys_var->mutex);
+
+      length= strlen(tmp->process_list_info);
+
+      if (length)
+      {
+        table->field[7]->store(tmp->process_list_info, length, cs);
+        table->field[7]->set_notnull();
+      }
+
+      if (schema_table_store_record(session, table))
+      {
+        pthread_mutex_unlock(&LOCK_thread_count);
+        return(1);
+      }
+    }
+  }
+
+  pthread_mutex_unlock(&LOCK_thread_count);
+  return 0;
+}
+

=== added file 'plugin/info_schema/processlist.h'
--- plugin/info_schema/processlist.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/processlist.h	2009-11-17 17:54:59 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_PROCESSLIST_H
+#define PLUGIN_INFO_SCHEMA_PROCESSLIST_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class ProcessListISMethods
+ * @brief
+ *   Class which implements any methods that the PROCESSLIST
+ *   I_S table needs besides the default methods
+ */
+class ProcessListISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+};
+
+class ProcessListIS
+{
+  /**
+   * Create the various columns for the PROCESSLIST I_S table and add them
+   * to the std::vector of columns for the PROCESSLIST table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_PROCESSLIST_H */

=== added file 'plugin/info_schema/referential_constraints.cc'
--- plugin/info_schema/referential_constraints.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/referential_constraints.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,265 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   referential constraints I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "referential_constraints.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the referential constraints I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the referential constraints I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * processlist I_S table.
+ */
+static plugin::InfoSchemaTable *rc_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *ReferentialConstraintsIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("UNIQUE_CONSTRAINT_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("UNIQUE_CONSTRAINT_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("UNIQUE_CONSTRAINT_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            MY_I_S_MAYBE_NULL,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MATCH_OPTION",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("UPDATE_RULE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DELETE_RULE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("REFERENCED_TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *ReferentialConstraintsIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new RefConstraintsISMethods();
+  }
+
+  if (rc_table == NULL)
+  {
+    rc_table= new plugin::InfoSchemaTable("REFERENTIAL_CONSTRAINTS",
+                                          *columns,
+                                          1, 9, false, true,
+                                          OPEN_TABLE_ONLY,
+                                          methods);
+  }
+
+  return rc_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void ReferentialConstraintsIS::cleanup()
+{
+  clearColumns(*columns);
+  delete rc_table;
+  delete methods;
+  delete columns;
+}
+
+int
+RefConstraintsISMethods::processTable(Session *session, 
+                                      TableList *tables,
+                                      Table *table, 
+                                      bool res,
+                                      LEX_STRING *db_name, 
+                                      LEX_STRING *table_name)
+  const
+{
+  const CHARSET_INFO * const cs= system_charset_info;
+
+  if (res)
+  {
+    if (session->is_error())
+    {
+      push_warning(session, 
+                   DRIZZLE_ERROR::WARN_LEVEL_WARN,
+                   session->main_da.sql_errno(), 
+                   session->main_da.message());
+    }
+    session->clear_error();
+    return 0;
+  }
+
+  {
+    List<FOREIGN_KEY_INFO> f_key_list;
+    Table *show_table= tables->table;
+    show_table->cursor->info(HA_STATUS_VARIABLE |
+                             HA_STATUS_NO_LOCK |
+                             HA_STATUS_TIME);
+
+    show_table->cursor->get_foreign_key_list(session, &f_key_list);
+    FOREIGN_KEY_INFO *f_key_info;
+    List_iterator_fast<FOREIGN_KEY_INFO> it(f_key_list);
+    while ((f_key_info= it++))
+    {
+      table->restoreRecordAsDefault();
+      table->field[1]->store(db_name->str, db_name->length, cs);
+      table->field[9]->store(table_name->str, table_name->length, cs);
+      table->field[2]->store(f_key_info->forein_id->str,
+                             f_key_info->forein_id->length, cs);
+      table->field[4]->store(f_key_info->referenced_db->str,
+                             f_key_info->referenced_db->length, cs);
+      table->field[10]->store(f_key_info->referenced_table->str,
+                              f_key_info->referenced_table->length, cs);
+      if (f_key_info->referenced_key_name)
+      {
+        table->field[5]->store(f_key_info->referenced_key_name->str,
+                               f_key_info->referenced_key_name->length, cs);
+        table->field[5]->set_notnull();
+      }
+      else
+      {
+        table->field[5]->set_null();
+      }
+      table->field[6]->store(STRING_WITH_LEN("NONE"), cs);
+      table->field[7]->store(f_key_info->update_method->str,
+                             f_key_info->update_method->length, cs);
+      table->field[8]->store(f_key_info->delete_method->str,
+                             f_key_info->delete_method->length, cs);
+      if (schema_table_store_record(session, table))
+      {
+        return 1;
+      }
+    }
+  }
+  return 0;
+}
+

=== added file 'plugin/info_schema/referential_constraints.h'
--- plugin/info_schema/referential_constraints.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/referential_constraints.h	2009-11-17 18:13:13 +0000
@@ -0,0 +1,68 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_REFERENTIAL_CONSTRAINTS_H
+#define PLUGIN_INFO_SCHEMA_REFERENTIAL_CONSTRAINTS_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class RefConstraintISMethods
+ * @brief
+ *   Class which implements any methods that the REFERENTIAL_CONSTRAINTS
+ *   I_S table needs besides the default methods
+ */
+class RefConstraintsISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  /**
+   * Fill and store records into I_S.referential_constraints table
+   *
+   * @param[in] session   thread handle
+   * @param[in] tables    table list struct(processed table)
+   * @param[in] table     I_S table
+   * @param[in] res       1 means the error during opening of the processed table
+   *                      0 means processed table is opened without error
+   * @param[in] base_name db name
+   * @param[in] file_name table name
+   *
+   * @return 0 on success; 1 on failure
+   */
+  virtual int processTable(Session *session, TableList *tables,
+                           Table *table, bool res, LEX_STRING *db_name,
+                           LEX_STRING *table_name) const;
+};
+
+class ReferentialConstraintsIS
+{
+  /**
+   * Create the various columns for the REFERENTIAL_CONSTRAINTS I_S table and add them
+   * to the std::vector of columns for the REFERENTIAL_CONSTRAINTS table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_REFERENTIAL_CONSTRAINTS_H */

=== added file 'plugin/info_schema/schemata.cc'
--- plugin/info_schema/schemata.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/schemata.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,273 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   schemata I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/join_table.h"
+
+#include "helper_methods.h"
+#include "schemata.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the schemata I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the schemata I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * schemata I_S table.
+ */
+static plugin::InfoSchemaTable *sch_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *SchemataIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("CATALOG_NAME",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0, 
+                                            1, 
+                                            "", 
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("SCHEMA_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0, 
+                                            0, 
+                                            "Database", 
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DEFAULT_CHARACTER_SET_NAME",
+                                            64, 
+                                            DRIZZLE_TYPE_VARCHAR, 
+                                            0, 
+                                            0, 
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DEFAULT_COLLATION_NAME",
+                                            64, 
+                                            DRIZZLE_TYPE_VARCHAR, 
+                                            0, 
+                                            0, 
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("SQL_PATH",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0, 
+                                            1, 
+                                            "", 
+                                            SKIP_OPEN_TABLE));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *SchemataIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new SchemataISMethods();
+  }
+
+  if (sch_table == NULL)
+  {
+    sch_table= new plugin::InfoSchemaTable("SCHEMATA",
+                                           *columns,
+                                           1, -1, false, false, 0,
+                                           methods);
+  }
+
+  return sch_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void SchemataIS::cleanup()
+{
+  clearColumns(*columns);
+  delete sch_table;
+  delete methods;
+  delete columns;
+}
+
+static bool store_schema_schemata(Session* session, 
+                                  Table *table, 
+                                  LEX_STRING *db_name,
+                                  const CHARSET_INFO * const cs)
+{
+  table->restoreRecordAsDefault();
+  table->field[1]->store(db_name->str, db_name->length, system_charset_info);
+  table->field[2]->store(cs->csname, strlen(cs->csname), system_charset_info);
+  table->field[3]->store(cs->name, strlen(cs->name), system_charset_info);
+  return schema_table_store_record(session, table);
+}
+
+int SchemataISMethods::fillTable(Session *session, TableList *tables)
+{
+  /*
+    TODO: fill_schema_shemata() is called when new client is connected.
+    Returning error status in this case leads to client hangup.
+  */
+
+  LOOKUP_FIELD_VALUES lookup_field_vals;
+  vector<LEX_STRING*> db_names;
+  bool with_i_schema;
+  Table *table= tables->table;
+  /* the WHERE condition */
+  COND *cond= table->reginfo.join_tab->select_cond;
+
+  if (get_lookup_field_values(session, cond, tables, &lookup_field_vals))
+  {
+    return 0;
+  }
+
+  if (make_db_list(session, 
+                   db_names, 
+                   &lookup_field_vals,
+                   &with_i_schema))
+  {
+    return 1;
+  }
+
+  /*
+    If we have lookup db value we should check that the database exists
+  */
+  if (lookup_field_vals.db_value.str && 
+      ! lookup_field_vals.wild_db_value &&
+      ! with_i_schema)
+  {
+    char path[FN_REFLEN+16];
+    uint32_t path_len;
+    struct stat stat_info;
+    if (! lookup_field_vals.db_value.str[0])
+    {
+      return 0;
+    }
+
+    path_len= build_table_filename(path, 
+                                   sizeof(path),
+                                   lookup_field_vals.db_value.str, 
+                                   "", 
+                                   false);
+    path[path_len-1]= 0;
+    if (stat(path,&stat_info))
+    {
+      return 0;
+    }
+  }
+
+  vector<LEX_STRING*>::iterator db_name= db_names.begin();
+  while (db_name != db_names.end())
+  {
+    if (with_i_schema)       // information schema name is always first in list
+    {
+      if (store_schema_schemata(session, table, *db_name, system_charset_info))
+      {
+        return 1;
+      }
+      with_i_schema= 0;
+    }
+    else
+    {
+      const CHARSET_INFO *cs= get_default_db_collation((*db_name)->str);
+
+      if (store_schema_schemata(session, table, *db_name, cs))
+      {
+        return 1;
+      }
+    }
+    ++db_name;
+  }
+  return 0;
+}
+
+int SchemataISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
+  const
+{
+  char tmp[128];
+  LEX *lex= session->lex;
+  Select_Lex *sel= lex->current_select;
+  Name_resolution_context *context= &sel->context;
+  const drizzled::plugin::InfoSchemaTable::Columns sch_columns= schema_table->getColumns();
+
+  if (! sel->item_list.elements)
+  {
+    const drizzled::plugin::ColumnInfo *column= sch_columns[1];
+    String buffer(tmp,sizeof(tmp), system_charset_info);
+    Item_field *field= new Item_field(context,
+                                      NULL, 
+                                      NULL, 
+                                      column->getName().c_str());
+    if (! field || session->add_item_to_list(field))
+    {
+      return 1;
+    }
+    buffer.length(0);
+    buffer.append(column->getOldName().c_str());
+    if (lex->wild && lex->wild->ptr())
+    {
+      buffer.append(STRING_WITH_LEN(" ("));
+      buffer.append(lex->wild->ptr());
+      buffer.append(')');
+    }
+    field->set_name(buffer.ptr(), buffer.length(), system_charset_info);
+  }
+  return 0;
+}
+

=== added file 'plugin/info_schema/schemata.h'
--- plugin/info_schema/schemata.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/schemata.h	2009-11-17 18:35:55 +0000
@@ -0,0 +1,56 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_SCHEMATA_H
+#define PLUGIN_INFO_SCHEMA_SCHEMATA_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class SchemataISMethods
+ * @brief
+ *   Class which implements any methods that the SCHEMATA
+ *   I_S table needs besides the default methods
+ */
+class SchemataISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session,
+                        TableList *tables);
+  virtual int oldFormat(Session *session,
+                        drizzled::plugin::InfoSchemaTable *schema_table) const;
+};
+
+class SchemataIS
+{
+  /**
+   * Create the various columns for the SCHEMATA I_S table and add them
+   * to the std::vector of columns for the SCHEMATA table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_SCHEMATA_H */

=== added file 'plugin/info_schema/statistics.cc'
--- plugin/info_schema/statistics.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/statistics.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,348 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   statistics I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/tztime.h"
+
+#include "helper_methods.h"
+#include "statistics.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the statistics I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the statistics I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * statistics I_S table.
+ */
+static plugin::InfoSchemaTable *stats_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *StatisticsIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FRM_ONLY));
+  
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Table",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("NON_UNIQUE",
+                                            1,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Non_unique",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("INDEX_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("INDEX_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Key_name",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("SEQ_IN_INDEX",
+                                            2,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            0,
+                                            "Seq_in_index",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLUMN_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Column_name",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("COLLATION",
+                                            1,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Collation",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("CARDINALITY",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            1,
+                                            "Cardinality",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("SUB_PART",
+                                            3,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            1,
+                                            "Sub_part",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("PACKED",
+                                            10,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Packed",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("NULLABLE",
+                                            3,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Null",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("INDEX_TYPE",
+                                            16,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Index_type",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("COMMENT",
+                                            16,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Comment",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("INDEX_COMMENT",
+                                            INDEX_COMMENT_MAXLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Index_Comment",
+                                            OPEN_FRM_ONLY));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *StatisticsIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new StatsISMethods();
+  }
+
+  if (stats_table == NULL)
+  {
+    stats_table= new plugin::InfoSchemaTable("STATISTICS",
+                                             *columns,
+                                             1, 2, false, true,
+                                             OPEN_TABLE_ONLY | OPTIMIZE_I_S_TABLE,
+                                             methods);
+  }
+
+  return stats_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void StatisticsIS::cleanup()
+{
+  clearColumns(*columns);
+  delete stats_table;
+  delete methods;
+  delete columns;
+}
+
+int StatsISMethods::processTable(Session *session, 
+                                 TableList *tables,
+                                 Table *table, 
+                                 bool res,
+                                 LEX_STRING *db_name,
+                                 LEX_STRING *table_name) const
+{
+  const CHARSET_INFO * const cs= system_charset_info;
+  if (res)
+  {
+    if (session->lex->sql_command != SQLCOM_SHOW_KEYS)
+    {
+      /*
+        I.e. we are in SELECT FROM INFORMATION_SCHEMA.STATISTICS
+        rather than in SHOW KEYS
+      */
+      if (session->is_error())
+      {
+        push_warning(session, 
+                     DRIZZLE_ERROR::WARN_LEVEL_WARN,
+                     session->main_da.sql_errno(), 
+                     session->main_da.message());
+      }
+      session->clear_error();
+      res= 0;
+    }
+    return res;
+  }
+  else
+  {
+    Table *show_table= tables->table;
+    KEY *key_info=show_table->s->key_info;
+    if (show_table->cursor)
+    {
+      show_table->cursor->info(HA_STATUS_VARIABLE |
+                               HA_STATUS_NO_LOCK |
+                               HA_STATUS_TIME);
+    }
+    for (uint32_t i= 0; i < show_table->s->keys; i++, key_info++)
+    {
+      KEY_PART_INFO *key_part= key_info->key_part;
+      const char *str;
+      for (uint32_t j= 0; j < key_info->key_parts; j++, key_part++)
+      {
+        table->restoreRecordAsDefault();
+        table->field[1]->store(db_name->str, db_name->length, cs);
+        table->field[2]->store(table_name->str, table_name->length, cs);
+        table->field[3]->store((int64_t) ((key_info->flags &
+                                            HA_NOSAME) ? 0 : 1), true);
+        table->field[4]->store(db_name->str, db_name->length, cs);
+        table->field[5]->store(key_info->name, strlen(key_info->name), cs);
+        table->field[6]->store((int64_t) (j+1), true);
+        str= (key_part->field ? key_part->field->field_name :
+             "?unknown field?");
+        table->field[7]->store(str, strlen(str), cs);
+        if (show_table->cursor)
+        {
+          if (show_table->cursor->index_flags(i, j, 0) & HA_READ_ORDER)
+          {
+            table->field[8]->store(((key_part->key_part_flag &
+                                     HA_REVERSE_SORT) ?
+                                    "D" : "A"), 1, cs);
+            table->field[8]->set_notnull();
+          }
+          KEY *key= show_table->key_info + i;
+          if (key->rec_per_key[j])
+          {
+            ha_rows records=(show_table->cursor->stats.records /
+                             key->rec_per_key[j]);
+            table->field[9]->store((int64_t) records, true);
+            table->field[9]->set_notnull();
+          }
+          str= show_table->cursor->index_type(i);
+          table->field[13]->store(str, strlen(str), cs);
+        }
+        if ((key_part->field &&
+             key_part->length !=
+             show_table->s->field[key_part->fieldnr-1]->key_length()))
+        {
+          table->field[10]->store((int64_t) key_part->length /
+                                  key_part->field->charset()->mbmaxlen, true);
+          table->field[10]->set_notnull();
+        }
+        uint32_t flags= key_part->field ? key_part->field->flags : 0;
+        const char *pos= (char*) ((flags & NOT_NULL_FLAG) ? "" : "YES");
+        table->field[12]->store(pos, strlen(pos), cs);
+        if (!show_table->s->keys_in_use.test(i))
+        {
+          table->field[14]->store(STRING_WITH_LEN("disabled"), cs);
+        }
+        else
+        {
+          table->field[14]->store("", 0, cs);
+        }
+        table->field[14]->set_notnull();
+        assert(test(key_info->flags & HA_USES_COMMENT) ==
+                   (key_info->comment.length > 0));
+        if (key_info->flags & HA_USES_COMMENT)
+        {
+          table->field[15]->store(key_info->comment.str,
+                                  key_info->comment.length, cs);
+        }
+        if (schema_table_store_record(session, table))
+        {
+          return 1;
+        }
+      }
+    }
+  }
+  return res;
+}

=== added file 'plugin/info_schema/statistics.h'
--- plugin/info_schema/statistics.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/statistics.h	2009-11-18 04:33:30 +0000
@@ -0,0 +1,55 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_STATISTICS_H
+#define PLUGIN_INFO_SCHEMA_STATISTICS_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class StatsISMethods
+ * @brief
+ *   Class which implements any methods that the SCHEMATA
+ *   I_S table needs besides the default methods
+ */
+class StatsISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int processTable(Session *session, TableList *tables,
+                           Table *table, bool res, LEX_STRING *db_name,
+                           LEX_STRING *table_name) const;
+};
+
+class StatisticsIS
+{
+  /**
+   * Create the various columns for the STATISTICS I_S table and add them
+   * to the std::vector of columns for the STATISTICS table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_STATISTICS_H */

=== added file 'plugin/info_schema/status.cc'
--- plugin/info_schema/status.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/status.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,226 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   status I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/tztime.h"
+
+#include "helper_methods.h"
+#include "status.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the status I_S tables.
+ */
+vector<const plugin::ColumnInfo *> *status_columns= NULL;
+
+/*
+ * Methods for the status related I_S tables.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * status I_S tables.
+ */
+static plugin::InfoSchemaTable *glob_status_table= NULL;
+static plugin::InfoSchemaTable *sess_status_table= NULL;
+static plugin::InfoSchemaTable *status_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *GlobalStatusIS::createColumns()
+{
+  if (status_columns == NULL)
+  {
+    status_columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*status_columns);
+  }
+
+  status_columns->push_back(new plugin::ColumnInfo("VARIABLE_NAME",
+                                                   64,
+                                                   DRIZZLE_TYPE_VARCHAR,
+                                                   0,
+                                                   0,
+                                                   "Variable_name",
+                                                   SKIP_OPEN_TABLE));
+
+  status_columns->push_back(new plugin::ColumnInfo("VARIABLE_VALUE",
+                                                   16300,
+                                                   DRIZZLE_TYPE_VARCHAR,
+                                                   0,
+                                                   1,
+                                                   "Value",
+                                                   SKIP_OPEN_TABLE));
+
+  return status_columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *GlobalStatusIS::getTable()
+{
+  status_columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new StatusISMethods();
+  }
+
+  if (glob_status_table == NULL)
+  {
+    glob_status_table= new plugin::InfoSchemaTable("GLOBAL_STATUS",
+                                                   *status_columns,
+                                                   -1, -1, false, false,
+                                                   0, 
+                                                   methods);
+  }
+
+  return glob_status_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void GlobalStatusIS::cleanup()
+{
+  clearColumns(*status_columns);
+  delete glob_status_table;
+  delete methods;
+  delete status_columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *SessionStatusIS::getTable()
+{
+  if (sess_status_table == NULL)
+  {
+    sess_status_table= new plugin::InfoSchemaTable("SESSION_STATUS",
+                                                   *status_columns,
+                                                   -1, -1, false, false,
+                                                   0, 
+                                                   methods);
+  }
+
+  return sess_status_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void SessionStatusIS::cleanup()
+{
+  delete sess_status_table;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *StatusIS::getTable()
+{
+  if (status_table == NULL)
+  {
+    status_table= new plugin::InfoSchemaTable("STATUS",
+                                              *status_columns,
+                                              -1, -1, true, false, 0,
+                                              methods);
+  }
+
+  return status_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void StatusIS::cleanup()
+{
+  delete status_table;
+}
+
+int StatusISMethods::fillTable(Session *session, TableList *tables)
+{
+  LEX *lex= session->lex;
+  const char *wild= lex->wild ? lex->wild->ptr() : NULL;
+  int res= 0;
+  STATUS_VAR *tmp1, tmp;
+  const string schema_table_name= tables->schema_table->getTableName();
+  enum enum_var_type option_type;
+  bool upper_case_names= (schema_table_name.compare("STATUS") != 0);
+
+  if (schema_table_name.compare("STATUS") == 0)
+  {
+    option_type= lex->option_type;
+    if (option_type == OPT_GLOBAL)
+    {
+      tmp1= &tmp;
+    }
+    else
+    {
+      tmp1= session->initial_status_var;
+    }
+  }
+  else if (schema_table_name.compare("GLOBAL_STATUS") == 0)
+  {
+    option_type= OPT_GLOBAL;
+    tmp1= &tmp;
+  }
+  else
+  {
+    option_type= OPT_SESSION;
+    tmp1= &session->status_var;
+  }
+
+  pthread_mutex_lock(&LOCK_status);
+  if (option_type == OPT_GLOBAL)
+  {
+    calc_sum_of_all_status(&tmp);
+  }
+  res= show_status_array(session, wild,
+                         getFrontOfStatusVars(),
+                         option_type, tmp1, "", tables->table,
+                         upper_case_names);
+  pthread_mutex_unlock(&LOCK_status);
+  return res;
+}
+

=== added file 'plugin/info_schema/status.h'
--- plugin/info_schema/status.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/status.h	2009-11-18 05:15:57 +0000
@@ -0,0 +1,68 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_STATUS_H
+#define PLUGIN_INFO_SCHEMA_STATUS_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class StatusISMethods
+ * @brief
+ *   Class which implements any methods that the STATUS
+ *   I_S table needs besides the default methods
+ */
+class StatusISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session, 
+                        TableList *tables);
+};
+
+class GlobalStatusIS
+{
+  /**
+   * Create the various columns for the GLOBAL_STATUS I_S table and add them
+   * to the std::vector of columns for the GLOBAL_STATUS table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+class SessionStatusIS
+{
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+class StatusIS
+{
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_STATUS_H */

=== added file 'plugin/info_schema/table_constraints.cc'
--- plugin/info_schema/table_constraints.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/table_constraints.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,257 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   table constraints I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "table_constraints.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the table constraints I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the table constraints I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * table constraints I_S table.
+ */
+static plugin::InfoSchemaTable *tc_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *TableConstraintsIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CONSTRAINT_TYPE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FULL_TABLE));
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *TableConstraintsIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new TabConstraintsISMethods();
+  }
+
+  if (tc_table == NULL)
+  {
+    tc_table= new plugin::InfoSchemaTable("TABLE_CONSTRAINTS",
+                                          *columns,
+                                          3, 4, false, true,
+                                          OPEN_TABLE_ONLY,
+                                          methods);
+  }
+
+  return tc_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void TableConstraintsIS::cleanup()
+{
+  clearColumns(*columns);
+  delete tc_table;
+  delete methods;
+  delete columns;
+}
+
+static bool store_constraints(Session *session, 
+                              Table *table, 
+                              LEX_STRING *db_name,
+                              LEX_STRING *table_name, 
+                              const char *key_name,
+                              uint32_t key_len, 
+                              const char *con_type, 
+                              uint32_t con_len)
+{
+  const CHARSET_INFO * const cs= system_charset_info;
+  table->restoreRecordAsDefault();
+  table->field[1]->store(db_name->str, db_name->length, cs);
+  table->field[2]->store(key_name, key_len, cs);
+  table->field[3]->store(db_name->str, db_name->length, cs);
+  table->field[4]->store(table_name->str, table_name->length, cs);
+  table->field[5]->store(con_type, con_len, cs);
+  return schema_table_store_record(session, table);
+}
+
+int TabConstraintsISMethods::processTable(Session *session, 
+                                          TableList *tables,
+                                          Table *table, 
+                                          bool res,
+                                          LEX_STRING *db_name,
+                                          LEX_STRING *table_name) const
+{
+  if (res)
+  {
+    if (session->is_error())
+    {
+      push_warning(session, 
+                   DRIZZLE_ERROR::WARN_LEVEL_WARN,
+                   session->main_da.sql_errno(), 
+                   session->main_da.message());
+    }
+    session->clear_error();
+    return 0;
+  }
+  else
+  {
+    List<FOREIGN_KEY_INFO> f_key_list;
+    Table *show_table= tables->table;
+    KEY *key_info=show_table->key_info;
+    uint32_t primary_key= show_table->s->primary_key;
+    show_table->cursor->info(HA_STATUS_VARIABLE |
+                             HA_STATUS_NO_LOCK |
+                             HA_STATUS_TIME);
+    for (uint32_t i= 0; i < show_table->s->keys; i++, key_info++)
+    {
+      if (i != primary_key && ! (key_info->flags & HA_NOSAME))
+      {
+        continue;
+      }
+
+      if (i == primary_key && is_primary_key(key_info))
+      {
+        if (store_constraints(session, 
+                              table, 
+                              db_name, 
+                              table_name, 
+                              key_info->name,
+                              strlen(key_info->name),
+                              STRING_WITH_LEN("PRIMARY KEY")))
+        {
+          return 1;
+        }
+      }
+      else if (key_info->flags & HA_NOSAME)
+      {
+        if (store_constraints(session, 
+                              table, 
+                              db_name, 
+                              table_name, 
+                              key_info->name,
+                              strlen(key_info->name),
+                              STRING_WITH_LEN("UNIQUE")))
+        {
+          return 1;
+        }
+      }
+    }
+
+    show_table->cursor->get_foreign_key_list(session, &f_key_list);
+    FOREIGN_KEY_INFO *f_key_info= NULL;
+    List_iterator_fast<FOREIGN_KEY_INFO> it(f_key_list);
+    while ((f_key_info= it++))
+    {
+      if (store_constraints(session, 
+                            table, 
+                            db_name, 
+                            table_name,
+                            f_key_info->forein_id->str,
+                            strlen(f_key_info->forein_id->str),
+                            "FOREIGN KEY", 11))
+      {
+        return 1;
+      }
+    }
+  }
+  return res;
+}
+

=== added file 'plugin/info_schema/table_constraints.h'
--- plugin/info_schema/table_constraints.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/table_constraints.h	2009-11-18 02:35:54 +0000
@@ -0,0 +1,55 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_TABLE_CONSTRAINTS_H
+#define PLUGIN_INFO_SCHEMA_TABLE_CONSTRAINTS_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class TabConstraintsISMethods
+ * @brief
+ *   Class which implements any methods that the TABLE_CONSTRAINTS
+ *   I_S table needs besides the default methods
+ */
+class TabConstraintsISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int processTable(Session *session, TableList *tables,
+                           Table *table, bool res, LEX_STRING *db_name,
+                           LEX_STRING *table_name) const;
+};
+
+class TableConstraintsIS
+{
+  /**
+   * Create the various columns for the TABLE_CONSTRAINTS I_S table and add them
+   * to the std::vector of columns for the TABLE_CONSTRAINTS table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_TABLE_CONSTRAINTS_H */

=== added file 'plugin/info_schema/table_names.cc'
--- plugin/info_schema/table_names.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/table_names.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,181 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   table names I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/tztime.h"
+
+#include "helper_methods.h"
+#include "table_names.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the table names I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the table names I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * table names I_S table.
+ */
+static plugin::InfoSchemaTable *tn_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *TableNamesIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Tables_in_",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_TYPE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Table_type",
+                                            OPEN_FRM_ONLY));
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *TableNamesIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new TabNamesISMethods();
+  }
+
+  if (tn_table == NULL)
+  {
+    tn_table= new plugin::InfoSchemaTable("TABLE_NAMES",
+                                          *columns,
+                                          1, 2, true, true, 0,
+                                          methods);
+  }
+
+  return tn_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void TableNamesIS::cleanup()
+{
+  clearColumns(*columns);
+  delete tn_table;
+  delete methods;
+  delete columns;
+}
+
+int TabNamesISMethods::oldFormat(Session *session, drizzled::plugin::InfoSchemaTable *schema_table)
+  const
+{
+  char tmp[128];
+  String buffer(tmp,sizeof(tmp), session->charset());
+  LEX *lex= session->lex;
+  Name_resolution_context *context= &lex->select_lex.context;
+  const drizzled::plugin::InfoSchemaTable::Columns tab_columns= schema_table->getColumns();
+
+  const drizzled::plugin::ColumnInfo *column= tab_columns[2];
+  buffer.length(0);
+  buffer.append(column->getOldName().c_str());
+  buffer.append(lex->select_lex.db);
+  if (lex->wild && lex->wild->ptr())
+  {
+    buffer.append(STRING_WITH_LEN(" ("));
+    buffer.append(lex->wild->ptr());
+    buffer.append(')');
+  }
+  Item_field *field= new Item_field(context,
+                                    NULL, NULL, column->getName().c_str());
+  if (session->add_item_to_list(field))
+  {
+    return 1;
+  }
+  field->set_name(buffer.ptr(), buffer.length(), system_charset_info);
+  if (session->lex->verbose)
+  {
+    field->set_name(buffer.ptr(), buffer.length(), system_charset_info);
+    column= tab_columns[3];
+    field= new Item_field(context, NULL, NULL, column->getName().c_str());
+    if (session->add_item_to_list(field))
+    {
+      return 1;
+    }
+    field->set_name(column->getOldName().c_str(),
+                    column->getOldName().length(),
+                    system_charset_info);
+  }
+  return 0;
+}
+

=== added file 'plugin/info_schema/table_names.h'
--- plugin/info_schema/table_names.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/table_names.h	2009-11-18 03:50:30 +0000
@@ -0,0 +1,54 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_TABLE_NAMES_H
+#define PLUGIN_INFO_SCHEMA_TABLE_NAMES_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class TabNamesISMethods
+ * @brief
+ *   Class which implements any methods that the TABLE_NAMES
+ *   I_S table needs besides the default methods
+ */
+class TabNamesISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int oldFormat(Session *session,
+                        drizzled::plugin::InfoSchemaTable *schema_table) const;
+};
+
+class TableNamesIS
+{
+  /**
+   * Create the various columns for the TABLE_NAMES I_S table and add them
+   * to the std::vector of columns for the TABLE_NAMES table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_TABLE_NAMES_H */

=== added file 'plugin/info_schema/tables.cc'
--- plugin/info_schema/tables.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/tables.cc	2009-11-18 21:40:45 +0000
@@ -0,0 +1,460 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   tables I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+#include "drizzled/tztime.h"
+
+#include "helper_methods.h"
+#include "tables.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the tables I_S table.
+ */
+static vector<const plugin::ColumnInfo *> *columns= NULL;
+
+/*
+ * Methods for the tables I_S table.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * tables I_S table.
+ */
+static plugin::InfoSchemaTable *tbls_table= NULL;
+
+/**
+ * Populate the vectors of columns for the I_S table.
+ *
+ * @return a pointer to a std::vector of Columns.
+ */
+vector<const plugin::ColumnInfo *> *TablesIS::createColumns()
+{
+  if (columns == NULL)
+  {
+    columns= new vector<const plugin::ColumnInfo *>;
+  }
+  else
+  {
+    clearColumns(*columns);
+  }
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_CATALOG",
+                                            FN_REFLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_SCHEMA",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_NAME",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Name",
+                                            SKIP_OPEN_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_TYPE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("ENGINE",
+                                            NAME_CHAR_LEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Engine",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("VERSION",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Version",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("ROW_FORMAT",
+                                            10,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Row_format",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_ROWS",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Rows",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("AVG_ROW_LENGTH",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Avg_row_length",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DATA_LENGTH",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Data_length",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("MAX_DATA_LENGTH",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Max_data_length",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("INDEX_LENGTH",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Index_length",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("DATA_FREE",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Data_free",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("AUTO_INCREMENT",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Auto_increment",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CREATE_TIME",
+                                            0,
+                                            DRIZZLE_TYPE_DATETIME,
+                                            0,
+                                            1,
+                                            "Create_time",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("UPDATE_TIME",
+                                            0,
+                                            DRIZZLE_TYPE_DATETIME,
+                                            0,
+                                            1,
+                                            "Update_time",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CHECK_TIME",
+                                            0,
+                                            DRIZZLE_TYPE_DATETIME,
+                                            0,
+                                            1,
+                                            "Check_time",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_COLLATION",
+                                            64,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Collation",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("CHECKSUM",
+                                            MY_INT64_NUM_DECIMAL_DIGITS,
+                                            DRIZZLE_TYPE_LONGLONG,
+                                            0,
+                                            (MY_I_S_MAYBE_NULL | MY_I_S_UNSIGNED),
+                                            "Checksum",
+                                            OPEN_FULL_TABLE));
+
+  columns->push_back(new plugin::ColumnInfo("CREATE_OPTIONS",
+                                            255,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            1,
+                                            "Create_options",
+                                            OPEN_FRM_ONLY));
+
+  columns->push_back(new plugin::ColumnInfo("TABLE_COMMENT",
+                                            TABLE_COMMENT_MAXLEN,
+                                            DRIZZLE_TYPE_VARCHAR,
+                                            0,
+                                            0,
+                                            "Comment",
+                                            OPEN_FRM_ONLY));
+
+  return columns;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *TablesIS::getTable()
+{
+  columns= createColumns();
+
+  if (methods == NULL)
+  {
+    methods= new TablesISMethods();
+  }
+
+  if (tbls_table == NULL)
+  {
+    tbls_table= new plugin::InfoSchemaTable("TABLES",
+                                            *columns,
+                                            1, 2, false, true,
+                                            OPTIMIZE_I_S_TABLE,
+                                            methods);
+  }
+
+  return tbls_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void TablesIS::cleanup()
+{
+  clearColumns(*columns);
+  delete tbls_table;
+  delete methods;
+  delete columns;
+}
+
+int TablesISMethods::processTable(Session *session, 
+                                  TableList *tables,
+                                  Table *table, 
+                                  bool res,
+                                  LEX_STRING *db_name,
+                                  LEX_STRING *table_name) const
+{
+  const char *tmp_buff= NULL;
+  DRIZZLE_TIME time;
+  const CHARSET_INFO * const cs= system_charset_info;
+
+  table->restoreRecordAsDefault();
+  table->field[1]->store(db_name->str, db_name->length, cs);
+  table->field[2]->store(table_name->str, table_name->length, cs);
+  if (res)
+  {
+    /*
+      there was errors during opening tables
+    */
+    const char *error= session->is_error() ? session->main_da.message() : "";
+    if (tables->schema_table)
+    {
+      table->field[3]->store(STRING_WITH_LEN("SYSTEM VIEW"), cs);
+    }
+    else
+    {
+      table->field[3]->store(STRING_WITH_LEN("BASE Table"), cs);
+    }
+    table->field[20]->store(error, strlen(error), cs);
+    session->clear_error();
+  }
+  else
+  {
+    char option_buff[400];
+    char *ptr= NULL;
+    Table *show_table= tables->table;
+    TableShare *share= show_table->s;
+    Cursor *cursor= show_table->cursor;
+    drizzled::plugin::StorageEngine *tmp_db_type= share->db_type();
+
+    if (share->tmp_table == SYSTEM_TMP_TABLE)
+    {
+      table->field[3]->store(STRING_WITH_LEN("SYSTEM VIEW"), cs);
+    }
+    else if (share->tmp_table)
+    {
+      table->field[3]->store(STRING_WITH_LEN("LOCAL TEMPORARY"), cs);
+    }
+    else
+    {
+      table->field[3]->store(STRING_WITH_LEN("BASE Table"), cs);
+    }
+
+    for (int i= 4; i < 20; i++)
+    {
+      if (i == 7 || (i > 12 && i < 17) || i == 18)
+      {
+        continue;
+      }
+      table->field[i]->set_notnull();
+    }
+    const string &engine_name= drizzled::plugin::StorageEngine::resolveName(tmp_db_type);
+    table->field[4]->store(engine_name.c_str(), engine_name.size(), cs);
+    table->field[5]->store((int64_t) 0, true);
+
+    ptr=option_buff;
+
+    if (share->db_create_options & HA_OPTION_PACK_KEYS)
+    {
+      ptr= strcpy(ptr," pack_keys=1")+12;
+    }
+    if (share->db_create_options & HA_OPTION_NO_PACK_KEYS)
+    {
+      ptr= strcpy(ptr," pack_keys=0")+12;
+    }
+    if (share->row_type != ROW_TYPE_DEFAULT)
+    {
+      ptr+= sprintf(ptr, " row_format=%s", ha_row_type[(uint32_t)share->row_type]);
+    }
+    if (share->block_size)
+    {
+      ptr= strcpy(ptr, " block_size=")+12;
+      ptr= int64_t10_to_str(share->block_size, ptr, 10);
+    }
+
+    table->field[19]->store(option_buff+1,
+                            (ptr == option_buff ? 0 :
+                             (uint32_t) (ptr-option_buff)-1), cs);
+
+    tmp_buff= (share->table_charset ?
+               share->table_charset->name : "default");
+    table->field[17]->store(tmp_buff, strlen(tmp_buff), cs);
+
+    if (share->hasComment())
+      table->field[20]->store(share->getComment(),
+                              share->getCommentLength(), cs);
+
+    if (cursor)
+    {
+      cursor->info(HA_STATUS_VARIABLE | 
+                   HA_STATUS_TIME | 
+                   HA_STATUS_AUTO |
+                   HA_STATUS_NO_LOCK);
+      enum row_type row_type = cursor->get_row_type();
+      switch (row_type) 
+      {
+      case ROW_TYPE_NOT_USED:
+      case ROW_TYPE_DEFAULT:
+        tmp_buff= ((share->db_options_in_use &
+                    HA_OPTION_COMPRESS_RECORD) ? "Compressed" :
+                   (share->db_options_in_use & HA_OPTION_PACK_RECORD) ?
+                   "Dynamic" : "Fixed");
+        break;
+      case ROW_TYPE_FIXED:
+        tmp_buff= "Fixed";
+        break;
+      case ROW_TYPE_DYNAMIC:
+        tmp_buff= "Dynamic";
+        break;
+      case ROW_TYPE_COMPRESSED:
+        tmp_buff= "Compressed";
+        break;
+      case ROW_TYPE_REDUNDANT:
+        tmp_buff= "Redundant";
+        break;
+      case ROW_TYPE_COMPACT:
+        tmp_buff= "Compact";
+        break;
+      case ROW_TYPE_PAGE:
+        tmp_buff= "Paged";
+        break;
+      }
+      table->field[6]->store(tmp_buff, strlen(tmp_buff), cs);
+      if (! tables->schema_table)
+      {
+        table->field[7]->store((int64_t) cursor->stats.records, true);
+        table->field[7]->set_notnull();
+      }
+      table->field[8]->store((int64_t) cursor->stats.mean_rec_length, true);
+      table->field[9]->store((int64_t) cursor->stats.data_file_length, true);
+      if (cursor->stats.max_data_file_length)
+      {
+        table->field[10]->store((int64_t) cursor->stats.max_data_file_length,
+                                true);
+      }
+      table->field[11]->store((int64_t) cursor->stats.index_file_length, true);
+      table->field[12]->store((int64_t) cursor->stats.delete_length, true);
+      if (show_table->found_next_number_field)
+      {
+        table->field[13]->store((int64_t) cursor->stats.auto_increment_value, true);
+        table->field[13]->set_notnull();
+      }
+      if (cursor->stats.create_time)
+      {
+        session->variables.time_zone->gmt_sec_to_TIME(&time,
+                                                  (time_t) cursor->stats.create_time);
+        table->field[14]->store_time(&time, DRIZZLE_TIMESTAMP_DATETIME);
+        table->field[14]->set_notnull();
+      }
+      if (cursor->stats.update_time)
+      {
+        session->variables.time_zone->gmt_sec_to_TIME(&time,
+                                                  (time_t) cursor->stats.update_time);
+        table->field[15]->store_time(&time, DRIZZLE_TIMESTAMP_DATETIME);
+        table->field[15]->set_notnull();
+      }
+      if (cursor->stats.check_time)
+      {
+        session->variables.time_zone->gmt_sec_to_TIME(&time,
+                                                  (time_t) cursor->stats.check_time);
+        table->field[16]->store_time(&time, DRIZZLE_TIMESTAMP_DATETIME);
+        table->field[16]->set_notnull();
+      }
+      if (cursor->ha_table_flags() & (ulong) HA_HAS_CHECKSUM)
+      {
+        table->field[18]->store((int64_t) cursor->checksum(), true);
+        table->field[18]->set_notnull();
+      }
+    }
+  }
+  return (schema_table_store_record(session, table));
+}
+

=== added file 'plugin/info_schema/tables.h'
--- plugin/info_schema/tables.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/tables.h	2009-11-18 02:55:28 +0000
@@ -0,0 +1,55 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_TABLES_H
+#define PLUGIN_INFO_SCHEMA_TABLES_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class TablesISMethods
+ * @brief
+ *   Class which implements any methods that the TABLE_NAMES
+ *   I_S table needs besides the default methods
+ */
+class TablesISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int processTable(Session *session, TableList *tables,
+                           Table *table, bool res, LEX_STRING *db_name,
+                           LEX_STRING *table_name) const;
+};
+
+class TablesIS
+{
+  /**
+   * Create the various columns for the TABLES I_S table and add them
+   * to the std::vector of columns for the TABLES table.
+   *
+   * @return cols vector to add columns to
+   */
+  static std::vector<const drizzled::plugin::ColumnInfo *>
+    *createColumns();
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_TABLES_H */

=== added file 'plugin/info_schema/variables.cc'
--- plugin/info_schema/variables.cc	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/variables.cc	2009-11-18 05:30:00 +0000
@@ -0,0 +1,162 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *   variables I_S table methods.
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/show.h"
+
+#include "helper_methods.h"
+#include "variables.h"
+
+#include <vector>
+
+using namespace drizzled;
+using namespace std;
+
+/*
+ * Vectors of columns for the variables I_S tables.
+ */
+extern vector<const plugin::ColumnInfo *> *status_columns;
+
+/*
+ * Methods for the variables related I_S tables.
+ */
+static plugin::InfoSchemaMethods *methods= NULL;
+
+/*
+ * variables I_S tables.
+ */
+static plugin::InfoSchemaTable *glob_var_table= NULL;
+static plugin::InfoSchemaTable *sess_var_table= NULL;
+static plugin::InfoSchemaTable *var_table= NULL;
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *GlobalVariablesIS::getTable()
+{
+  if (methods == NULL)
+  {
+    methods= new VariablesISMethods();
+  }
+
+  if (glob_var_table == NULL)
+  {
+    glob_var_table= new plugin::InfoSchemaTable("GLOBAL_VARIABLES",
+                                                *status_columns,
+                                                -1, -1, false, false,
+                                                0, 
+                                                methods);
+  }
+
+  return glob_var_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void GlobalVariablesIS::cleanup()
+{
+  delete glob_var_table;
+  delete methods;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *SessionVariablesIS::getTable()
+{
+  if (sess_var_table == NULL)
+  {
+    sess_var_table= new plugin::InfoSchemaTable("SESSION_VARIABLES",
+                                                *status_columns,
+                                                -1, -1, false, false, 0,
+                                                methods);
+  }
+
+  return sess_var_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void SessionVariablesIS::cleanup()
+{
+  delete sess_var_table;
+}
+
+/**
+ * Initialize the I_S table.
+ *
+ * @return a pointer to an I_S table
+ */
+plugin::InfoSchemaTable *VariablesIS::getTable()
+{
+  if (var_table == NULL)
+  {
+    var_table= new plugin::InfoSchemaTable("VARIABLES",
+                                           *status_columns,
+                                           -1, -1, true, false, 0,
+                                           methods);
+  }
+
+  return var_table;
+}
+
+/**
+ * Delete memory allocated for the table, columns and methods.
+ */
+void VariablesIS::cleanup()
+{
+  delete var_table;
+}
+
+int VariablesISMethods::fillTable(Session *session, TableList *tables)
+{
+  int res= 0;
+  LEX *lex= session->lex;
+  const char *wild= lex->wild ? lex->wild->ptr() : NULL;
+  const string schema_table_name= tables->schema_table->getTableName();
+  enum enum_var_type option_type= OPT_SESSION;
+  bool upper_case_names= (schema_table_name.compare("VARIABLES") != 0);
+  bool sorted_vars= (schema_table_name.compare("VARIABLES") == 0);
+
+  if (lex->option_type == OPT_GLOBAL ||
+      schema_table_name.compare("GLOBAL_VARIABLES") == 0)
+  {
+    option_type= OPT_GLOBAL;
+  }
+
+  pthread_rwlock_rdlock(&LOCK_system_variables_hash);
+  res= show_status_array(session, wild, enumerate_sys_vars(session, sorted_vars),
+                         option_type, NULL, "", tables->table, upper_case_names);
+  pthread_rwlock_unlock(&LOCK_system_variables_hash);
+  return res;
+}
+

=== added file 'plugin/info_schema/variables.h'
--- plugin/info_schema/variables.h	1970-01-01 00:00:00 +0000
+++ plugin/info_schema/variables.h	2009-11-18 05:15:57 +0000
@@ -0,0 +1,61 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_INFO_SCHEMA_VARIABLES_H
+#define PLUGIN_INFO_SCHEMA_VARIABLES_H
+
+#include "drizzled/plugin/info_schema_table.h"
+
+/**
+ * @class VariablesISMethods
+ * @brief
+ *   Class which implements any methods that the VARIABLES
+ *   I_S table needs besides the default methods
+ */
+class VariablesISMethods : public drizzled::plugin::InfoSchemaMethods
+{
+public:
+  virtual int fillTable(Session *session, 
+                        TableList *tables);
+};
+
+
+class GlobalVariablesIS
+{
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+class SessionVariablesIS
+{
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+class VariablesIS
+{
+public:
+  static drizzled::plugin::InfoSchemaTable *getTable();
+  static void cleanup();
+};
+
+#endif /* PLUGIN_INFO_SCHEMA_VARIABLES_H */

=== modified file 'plugin/information_engine/information_cursor.cc'
--- plugin/information_engine/information_cursor.cc	2009-11-12 03:09:31 +0000
+++ plugin/information_engine/information_cursor.cc	2009-11-17 04:16:37 +0000
@@ -38,12 +38,10 @@
 
 int InformationCursor::open(const char *name, int, uint32_t)
 {
-  InformationEngine::Share *shareable= ((InformationEngine *)engine)->getShare(name);
+  share= ((InformationEngine *)engine)->getShare(name);
 
-  if (! shareable)
-  {
-    return HA_ERR_OUT_OF_MEM;
-  }
+  assert(share);
+  thr_lock_data_init(&share->lock, &lock, NULL);
 
   return 0;
 }

=== modified file 'plugin/information_engine/information_cursor.h'
--- plugin/information_engine/information_cursor.h	2009-11-12 03:09:31 +0000
+++ plugin/information_engine/information_cursor.h	2009-11-16 20:30:19 +0000
@@ -35,14 +35,6 @@
   InformationCursor(drizzled::plugin::StorageEngine &engine, TableShare &table_arg);
   ~InformationCursor() {}
 
-  /*
-    The name of the index type that will be used for display
-    don't implement this method unless you really have indexes
-  */
-  uint64_t table_flags() const
-  {
-    return 0;
-  }
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
   /* The following defines can be increased if necessary */
   int open(const char *name, int mode, uint32_t test_if_locked);

=== modified file 'plugin/information_engine/information_engine.cc'
--- plugin/information_engine/information_engine.cc	2009-11-13 18:55:43 +0000
+++ plugin/information_engine/information_engine.cc	2009-11-17 07:21:27 +0000
@@ -97,7 +97,6 @@
     {
     case DRIZZLE_TYPE_LONG:
       proto_field->set_type(message::Table::Field::INTEGER);
-      field_options->set_length(MAX_INT_WIDTH);
       break;
     case DRIZZLE_TYPE_DOUBLE:
       proto_field->set_type(message::Table::Field::DOUBLE);
@@ -111,7 +110,6 @@
       break;
     case DRIZZLE_TYPE_LONGLONG:
       proto_field->set_type(message::Table::Field::BIGINT);
-      field_options->set_length(MAX_BIGINT_WIDTH);
       break;
     case DRIZZLE_TYPE_DATETIME:
       proto_field->set_type(message::Table::Field::DATETIME);
@@ -127,7 +125,6 @@
         proto_field->mutable_string_options();
       proto_field->set_type(message::Table::Field::VARCHAR);
       str_field_options->set_length(column->getLength());
-      field_options->set_length(column->getLength() * 4);
       field_options->set_default_value("");
       str_field_options->set_collation_id(default_charset_info->number);
       str_field_options->set_collation(default_charset_info->name);

=== modified file 'plugin/information_engine/information_engine.h'
--- plugin/information_engine/information_engine.h	2009-11-12 16:49:34 +0000
+++ plugin/information_engine/information_engine.h	2009-11-17 04:16:37 +0000
@@ -44,16 +44,20 @@
     pthread_mutex_destroy(&mutex);
   }
 
+  uint64_t table_flags() const
+  {
+    return 0;
+  }
+
   class Share 
   {
   private:
     uint32_t count;
     drizzled::plugin::InfoSchemaTable *table;
+
+  public:
     THR_LOCK lock;
 
-
-  public:
-
     Share(const std::string &in_name) :
       count(1)
     {

=== modified file 'plugin/innobase/handler/ha_innodb.cc'
--- plugin/innobase/handler/ha_innodb.cc	2009-11-13 18:55:43 +0000
+++ plugin/innobase/handler/ha_innodb.cc	2009-11-16 20:30:19 +0000
@@ -250,6 +250,17 @@
     addAlias("INNOBASE");
   }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NULL_IN_KEY |
+            HA_CAN_INDEX_BLOBS |
+            HA_PRIMARY_KEY_REQUIRED_FOR_POSITION |
+            HA_PRIMARY_KEY_IN_READ_INDEX |
+            HA_PARTIAL_COLUMN_READ |
+            HA_TABLE_SCAN_ON_INDEX | 
+            HA_MRR_CANT_SORT);
+  }
+
   virtual
   int
   close_connection(
@@ -378,6 +389,9 @@
                                 const char* from, 
                                 const char* to);
   UNIV_INTERN int doDropTable(Session& session, const string table_path);
+
+  UNIV_INTERN virtual bool get_error_message(int error, String *buf);
+
 };
 
 /** @brief Initialize the default value of innodb_commit_concurrency.
@@ -1352,14 +1366,6 @@
 ha_innobase::ha_innobase(drizzled::plugin::StorageEngine &engine_arg,
                          TableShare &table_arg)
   :Cursor(engine_arg, table_arg),
-  int_table_flags(HA_REC_NOT_IN_SEQ |
-		  HA_NULL_IN_KEY |
-		  HA_CAN_INDEX_BLOBS |
-		  HA_PRIMARY_KEY_REQUIRED_FOR_POSITION |
-		  HA_PRIMARY_KEY_IN_READ_INDEX |
-		  HA_PARTIAL_COLUMN_READ |
-		  HA_TABLE_SCAN_ON_INDEX | 
-                  HA_MRR_CANT_SORT),
   primary_key(0), /* needs initialization because index_flags() may be called 
                      before this is set to the real value. It's ok to have any 
                      value here because it doesn't matter if we return the
@@ -2494,18 +2500,6 @@
 }
 
 
-
-/****************************************************************//**
-Get the table flags to use for the statement.
-@return	table flags */
-UNIV_INTERN
-Cursor::Table_flags
-ha_innobase::table_flags() const
-/*============================*/
-{
-        return int_table_flags;
-}
-
 /****************************************************************//**
 Returns the index type. */
 UNIV_INTERN
@@ -8059,15 +8053,15 @@
 
 	innobase_reset_autoinc(value);
 
-	return(0);
+	return 0;
 }
 
 /* See comment in Cursor.cc */
 UNIV_INTERN
 bool
-ha_innobase::get_error_message(int, String *buf)
+InnobaseEngine::get_error_message(int, String *buf)
 {
-	trx_t*	trx = check_trx_exists(ha_session());
+	trx_t*	trx = check_trx_exists(current_session);
 
 	buf->copy(trx->detailed_error, (uint) strlen(trx->detailed_error),
 		system_charset_info);

=== modified file 'plugin/innobase/handler/ha_innodb.h'
--- plugin/innobase/handler/ha_innodb.h	2009-11-13 18:55:43 +0000
+++ plugin/innobase/handler/ha_innodb.h	2009-11-16 20:30:19 +0000
@@ -71,7 +71,6 @@
 	ulong		upd_and_key_val_buff_len;
 					/* the length of each of the previous
 					two buffers */
-	Table_flags	int_table_flags;
 	uint		primary_key;
 	ulong		start_of_scan;	/*!< this is set to 1 when we are
 					starting a table scan but have not
@@ -109,7 +108,6 @@
 	UNIV_INTERN enum row_type get_row_type() const;
 
 	UNIV_INTERN const char* index_type(uint key_number);
-	UNIV_INTERN Table_flags table_flags() const;
 	UNIV_INTERN uint32_t index_flags(uint idx, uint part, bool all_parts) const;
 	UNIV_INTERN uint32_t max_supported_keys() const;
 	UNIV_INTERN uint32_t max_supported_key_length() const;
@@ -178,8 +176,6 @@
                                                     uint64_t *nb_reserved_values);
         UNIV_INTERN int reset_auto_increment(uint64_t value);
 
-        UNIV_INTERN virtual bool get_error_message(int error, String *buf);
-
 	UNIV_INTERN bool primary_key_is_clustered();
 	UNIV_INTERN int cmp_ref(const unsigned char *ref1, const unsigned char *ref2);
 	/** Fast index creation (smart ALTER TABLE) @see handler0alter.cc @{ */

=== modified file 'plugin/logging_gearman/logging_gearman.cc'
--- plugin/logging_gearman/logging_gearman.cc	2009-11-13 19:51:29 +0000
+++ plugin/logging_gearman/logging_gearman.cc	2009-11-17 02:26:31 +0000
@@ -237,12 +237,7 @@
     unsigned char qs[255];
   
     // to avoid trying to printf %s something that is potentially NULL
-    const char *dbs= (session->db) ? session->db : "";
-    int dbl= 0;
-    if (dbs != NULL)
-      dbl= session->db_length;
-  
-
+    const char *dbs= session->db.empty() ? "" : session->db.c_str();
   
     msgbuf_len=
       snprintf(msgbuf, MAX_MSG_LEN,
@@ -253,7 +248,7 @@
                session->thread_id,
                session->getQueryId(),
                // dont need to quote the db name, always CSV safe
-               dbl, dbs,
+               (int)session->db.length(), dbs,
                // do need to quote the query
                quotify((unsigned char *)session->getQueryString(),
                        session->getQueryLength(), qs, sizeof(qs)),

=== modified file 'plugin/logging_query/logging_query.cc'
--- plugin/logging_query/logging_query.cc	2009-11-13 19:51:29 +0000
+++ plugin/logging_query/logging_query.cc	2009-11-17 02:26:31 +0000
@@ -278,10 +278,7 @@
     unsigned char qs[255];
   
     // to avoid trying to printf %s something that is potentially NULL
-    const char *dbs= (session->db) ? session->db : "";
-    int dbl= 0;
-    if (dbs != NULL)
-      dbl= session->db_length;
+    const char *dbs= session->db.empty() ? "" : session->db.c_str();
   
     msgbuf_len=
       snprintf(msgbuf, MAX_MSG_LEN,
@@ -292,7 +289,7 @@
                session->thread_id,
                session->getQueryId(),
                // dont need to quote the db name, always CSV safe
-               dbl, dbs,
+               (int)session->db.length(), dbs,
                // do need to quote the query
                quotify((unsigned char *)session->getQueryString(),
                        session->getQueryLength(), qs, sizeof(qs)),

=== modified file 'plugin/logging_syslog/logging_syslog.cc'
--- plugin/logging_syslog/logging_syslog.cc	2009-11-10 20:47:10 +0000
+++ plugin/logging_syslog/logging_syslog.cc	2009-11-17 02:26:31 +0000
@@ -136,10 +136,7 @@
   
     /* to avoid trying to printf %s something that is potentially NULL */
   
-    const char *dbs= (session->db) ? session->db : "";
-    int dbl= 0;
-    if (dbs)
-      dbl= session->db_length;
+    const char *dbs= session->db.empty() ? "" : session->db.c_str();
   
     const char *qys= (session->getQueryString()) ? session->getQueryString() : "";
     int qyl= 0;
@@ -156,7 +153,7 @@
            " tmp_table=%ld total_warn_count=%ld\n",
            (unsigned long) session->thread_id,
            (unsigned long) session->getQueryId(),
-           dbl, dbs,
+           (int)session->db.length(), dbs,
            qyl, qys,
            (int) command_name[session->command].length,
            command_name[session->command].str,

=== modified file 'plugin/myisam/ha_myisam.cc'
--- plugin/myisam/ha_myisam.cc	2009-11-13 18:55:43 +0000
+++ plugin/myisam/ha_myisam.cc	2009-11-17 17:45:47 +0000
@@ -68,6 +68,19 @@
   ~MyisamEngine()
   { }
 
+  uint64_t table_flags() const
+  {
+    return (HA_NULL_IN_KEY |
+            HA_DUPLICATE_POS |
+            HA_CAN_INDEX_BLOBS |
+            HA_AUTO_PART_KEY |
+            HA_NO_TRANSACTIONS |
+            HA_HAS_RECORDS |
+            HA_STATS_RECORDS_IS_EXACT |
+            HA_NEED_READ_RANGE_BUFFER |
+            HA_MRR_CANT_SORT);
+  }
+
   virtual Cursor *create(TableShare &table,
                           MEM_ROOT *mem_root)
   {
@@ -106,7 +119,7 @@
                                        const bool,
                                        drizzled::message::Table *table_proto)
 {
-  int error= 1;
+  int error= ENOENT;
   ProtoCache::iterator iter;
 
   pthread_mutex_lock(&proto_cache_mutex);
@@ -501,18 +514,10 @@
 ha_myisam::ha_myisam(drizzled::plugin::StorageEngine &engine_arg,
                      TableShare &table_arg)
   : Cursor(engine_arg, table_arg),
-    file(0),
-    int_table_flags(HA_NULL_IN_KEY |
-                    HA_DUPLICATE_POS |
-                    HA_CAN_INDEX_BLOBS |
-                    HA_AUTO_PART_KEY |
-                    HA_NO_TRANSACTIONS |
-                    HA_HAS_RECORDS |
-                    HA_STATS_RECORDS_IS_EXACT |
-                    HA_NEED_READ_RANGE_BUFFER |
-                    HA_MRR_CANT_SORT),
-     can_enable_indexes(1)
-{}
+  file(0),
+  can_enable_indexes(true),
+  is_ordered(true)
+{ }
 
 Cursor *ha_myisam::clone(MEM_ROOT *mem_root)
 {
@@ -575,7 +580,8 @@
   if (!(test_if_locked & HA_OPEN_WAIT_IF_LOCKED))
     mi_extra(file, HA_EXTRA_WAIT_LOCK, 0);
   if (!table->s->db_record_offset)
-    int_table_flags|=HA_REC_NOT_IN_SEQ;
+    is_ordered= false;
+
 
   keys_with_parts.reset();
   for (i= 0; i < table->s->keys; i++)

=== modified file 'plugin/myisam/ha_myisam.h'
--- plugin/myisam/ha_myisam.h	2009-11-11 19:42:27 +0000
+++ plugin/myisam/ha_myisam.h	2009-11-17 17:45:47 +0000
@@ -35,9 +35,9 @@
 class ha_myisam: public Cursor
 {
   MI_INFO *file;
-  uint64_t int_table_flags;
   char    *data_file_name, *index_file_name;
   bool can_enable_indexes;
+  bool is_ordered;
   int repair(Session *session, MI_CHECK &param, bool optimize);
 
  public:
@@ -45,7 +45,6 @@
   ~ha_myisam() {}
   Cursor *clone(MEM_ROOT *mem_root);
   const char *index_type(uint32_t key_number);
-  uint64_t table_flags() const { return int_table_flags; }
   int index_init(uint32_t idx, bool sorted);
   int index_end();
   uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
@@ -101,6 +100,12 @@
                        bool eq_range_arg, bool sorted);
   int read_range_next();
   int reset_auto_increment(uint64_t value);
+
+  virtual bool isOrdered(void)
+  {
+    return false;
+  }
+
 private:
   key_map keys_with_parts;
 };

=== modified file 'plugin/pool_of_threads/plugin.ac'
--- plugin/pool_of_threads/plugin.ac	2009-05-14 05:53:45 +0000
+++ plugin/pool_of_threads/plugin.ac	2009-11-24 18:25:09 +0000
@@ -1,15 +1,5 @@
-AC_LIB_HAVE_LINKFLAGS(event,,
-  [
-  #include <sys/types.h>
-  #include <sys/time.h>
-  #include <stdlib.h>
-  #include <event.h>
-  ],[
-    struct bufferevent bev;
-    bufferevent_settimeout(&bev, 1, 1);
-    event_init();
-    event_loop(EVLOOP_ONCE);
-  ]) 
+PANDORA_HAVE_LIBEVENT
 AS_IF([test x$ac_cv_libevent = xno],
   AC_MSG_WARN([Couldnt find a recent libevent pool-of-threads scheduler will not be built. On Debian this can be found in libevent-dev. On RedHat its in libevent-devel.]))
 
+

=== modified file 'tests/r/create.result'
--- tests/r/create.result	2009-08-17 21:04:20 +0000
+++ tests/r/create.result	2009-11-18 03:04:25 +0000
@@ -1441,6 +1441,8 @@
 ) ENGINE=DEFAULT
 drop table имя_таблицы_в_кодировке_утф8_длиной_больше_чем_48;
 create table t1 like information_schema.processlist;
+ERROR HY000: Table storage engine 'INFORMATION_ENGINE' does not support the create option 'TEMPORARY'
+create table t1 like information_schema.processlist engine=innodb;
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
@@ -1452,9 +1454,11 @@
   `TIME` bigint NOT NULL DEFAULT '0',
   `STATE` varchar(64) DEFAULT NULL,
   `INFO` text
-) ENGINE=MyISAM
+) ENGINE=InnoDB
 drop table t1;
 create temporary table t1 like information_schema.processlist;
+ERROR HY000: Table storage engine 'INFORMATION_ENGINE' does not support the create option 'TEMPORARY'
+create temporary table t1 like information_schema.processlist engine=myisam;
 show create table t1;
 Table	Create Table
 t1	CREATE TEMPORARY TABLE `t1` (

=== modified file 'tests/t/create.test'
--- tests/t/create.test	2009-08-17 21:04:20 +0000
+++ tests/t/create.test	2009-11-18 03:04:25 +0000
@@ -1207,10 +1207,14 @@
 #
 # Bug#25629 CREATE TABLE LIKE does not work with INFORMATION_SCHEMA
 #
+--error 1478
 create table t1 like information_schema.processlist;
+create table t1 like information_schema.processlist engine=innodb;
 show create table t1;
 drop table t1;
+--error 1478
 create temporary table t1 like information_schema.processlist;
+create temporary table t1 like information_schema.processlist engine=myisam;
 show create table t1;
 drop table t1;
 

